/*
	$Id: queue.h,v 1.9 1999/08/27 20:27:30 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Expandable queue class template.
*/

//! common_component="Templates"	

#ifndef header_queue
#define header_queue

#include <stdlib.h>
#include <assert.h>

template <class TYPE>
class CL_QueueLink
{
public:
	CL_QueueLink<TYPE> *next;
	TYPE instance;
};

template <class TYPE>
class CL_Queue
//: Queue class.
// The queue class is used to manage queues of any type.
// <Put in some more info on how queues work here>
//!also: CL_Array - Array class
//!also: CL_List - List class
//!also: CL_Stack - Stack class
{
	CL_QueueLink<TYPE> *first;
	CL_QueueLink<TYPE> *last;
	int num_items;

public:
	CL_Queue()
	{
		first = NULL;
		last = NULL;
		num_items = 0;
	}
	//: Constructor which initializes the queue.
	
	CL_Queue(const CL_Queue<TYPE> &clone)
	{
		first = NULL;
		last = NULL;
		
		CL_QueueLink<TYPE> *cur = clone.first;
		while (cur != NULL)
		{
			put(cur->instance);
			cur = cur->next;
		}
	}
	//: Copy constructor.
	//!param: clone - Queue to clone.

	~CL_Queue()
	{
		clear();
	}

	bool is_empty()
	{
		return first == NULL;
	}
	//: Checks if the queue is empty.
	//!retval: true if the queue is empty, false otherwise.
	
	bool has_item()
	{
		return first != NULL;
	}
	//: Checks if the queue has any items.
	//!retval: true if the queue contains any items, false otherwise.
	
	TYPE put(TYPE item)
	{
		CL_QueueLink<TYPE> *cur = new CL_QueueLink<TYPE>;
		
		cur->instance = item;
		cur->next = NULL;
		
		if (first == NULL) first = cur;
		if (last != NULL) last->next = cur;
		last = cur;
		
		num_items++;
		
		return item;
	}
	//: Adds item to end of list.
	//!retval: Item added to list, NULL otherwise.
	
	TYPE get()
	{
		assert(first != NULL);

		CL_QueueLink<TYPE> *old_first = first;
		if (last == old_first) last = NULL;
		first = first->next;

		TYPE ret = old_first->instance;
		delete old_first;
		
		num_items--;
		
		return ret;
	}
	//: Fetches next item in queue.
	//!retval: First available item in queue.
	
	TYPE peek()
	{
		assert(first != NULL);
		return first->instance;
	}
	//: Returns the next item in queue, but unlike get() it doesn't remove
	//: it from the queu.
	//!retval: First available item in queue.
	
	void clear()
	{
		CL_QueueLink<TYPE> *cur_instance = first;
		while (cur_instance != NULL)
		{
			CL_QueueLink<TYPE> *delete_instance = cur_instance;
			cur_instance = cur_instance->next;
			
			delete delete_instance;
		}
		
		first = NULL;
		last = NULL;
	}
	//: Empties the array.
	
	int get_num_items()
	{
		return num_items;
	}
	//: Returns the number of items in the queue.
};

#endif
