/*
  Copyright (c)1999 ClanSoft.
*/

#ifndef header_stack
#define header_stack

#include <assert.h>

template <class TYPE>
class CL_Stack
{
	struct StackElement
	{
		TYPE instance;
		StackElement *next;
	};
	StackElement *first;
	int item_count;
public:

	CL_Stack()
	{
		item_count = 0;
		first = NULL;
	}

	CL_Stack(const CL_Stack &os)
	{
		first = NULL;
		item_count = os.item_count;

		StackElement *last_item = NULL;
		StackElement *cur = os.first;
		while (cur != NULL)
		{
			StackElement *new_elem = new StackElement;
			new_elem->instance = cur->instance;
			new_elem->next = NULL;

			if (last_item == NULL)
			{
				first = last_item = new_elem;
			}
			else
			{
				last_item->next = new_elem;
				last_item = new_elem;
			}
			cur = cur->next;
		}
	}

	~CL_Stack()
	{
		clear();
	}

	TYPE push(TYPE item)
	{
		StackElement *elem = new StackElement;
		elem->instance = item;
		elem->next = first;

		first = elem;
		item_count++;

		return item;
	}

	TYPE pop()
	{
		assert(first != NULL);

		TYPE ret = first->instance;
		item_count--;
		
		StackElement *del_element = first;
		first = first->next;
		delete del_element;

		return ret;
	}
	
	void clear()
	{
		while (first != NULL)
		{
			StackElement *del_element = first;
			first = first->next;
			delete del_element;
		}
	}

	bool is_empty() { return item_count == 0; }
	bool has_item() { return item_count > 0; }
};

#endif
