/*
	$Id: displaycard_dx.cpp,v 1.11 1999/09/08 12:28:34 starch Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#define INITGUID

#include <stdlib.h>
#include <iostream.h>

#include "Layer1/System/API/cl_assert.h"
#include "Layer1/System/Win32/app_windows.h"

#include "display_dx.h"
#include "dx_errors.h"
#include "displaycard_dx.h"
#include "blit_dx.h"
#include "../Generic/colormap.h"
#include "../Generic/blitters.h"

CL_DisplayCard_DX::CL_DisplayCard_DX(
	int card_no,
	LPGUID _card_uid,
	CL_String _card_name,
	int options) : CL_DisplayCard_Generic(card_no)
{
	is_inited = false;
	card_uid = _card_uid;
	cur_videomode = NULL;
	card_name = _card_name;
	m_target = NULL;

	if (options & SYSTEM_BACKBUFFER) video_backbuffer = false;
	else video_backbuffer = true;

	if (options & TRIPLE_BUFFER) num_backbuffers = 2;
	else if (options & SINGLE_BUFFER) num_backbuffers = 0;
	else num_backbuffers = 1;
}

CL_DisplayCard_DX::~CL_DisplayCard_DX()
{
	if (is_inited)
	{
		CL_Iterator<CL_VidMode> counter(videomodes);
		while (counter.next()!=NULL)
		{
			delete counter();
			counter.remove();
		}

		delete cur_videomode;

		directdraw2->FlipToGDISurface();
		delete m_target;
		directdraw2->RestoreDisplayMode();

		cl_assert(directdraw2->Release() == 0);
	}
}

void CL_DisplayCard_DX::initialize_dx()
{
	HRESULT err;
//	LPDIRECTDRAW directdraw;
 
	err = DirectDrawCreate(card_uid, &directdraw2, NULL); 
	cl_assert(err == DD_OK);

//	err = directdraw->QueryInterface(IID_IDirectDraw2, (LPVOID *) &directdraw2); 
//	cl_assert(err == DD_OK);

	err = directdraw2->SetCooperativeLevel(CL_WindowsSystem::hWnd, DDSCL_EXCLUSIVE|DDSCL_FULLSCREEN|DDSCL_ALLOWREBOOT|DDSCL_ALLOWMODEX);
	cl_assert(err == DD_OK); 

//	directdraw->Release();

//	enumerate_displaymodes();
	// disabled for NT 4.0 support

	// Only compress system-memory surfaces, if display hardware
	// does not accelerate system->video blits
	// Code removed, because "hardware" system->video blits
	// appears to be slower than pure software...
	DDCAPS caps = get_hal_caps();
	if ((caps.dwCaps & DDCAPS_CANBLTSYSMEM) == DDCAPS_CANBLTSYSMEM)
	{
		create_system_surfaces = true;
	}
	else 
	{
		create_system_surfaces = false;
	}

	is_inited = true;
}

static HRESULT CALLBACK ResolutionList_callback(LPDDSURFACEDESC lpDDSurfaceDesc, LPVOID lpContext)
{
	CL_List<CL_VidMode> *list=(CL_List<CL_VidMode> *) lpContext;

	DDSURFACEDESC surf_desc = *lpDDSurfaceDesc;
	bool modex = false;
	if ((surf_desc.ddsCaps.dwCaps & DDSCAPS_MODEX) == DDSCAPS_MODEX) modex = true;
	list->add(new CL_VidMode(surf_desc.dwWidth, surf_desc.dwHeight, surf_desc.ddpfPixelFormat.dwRGBBitCount, modex));

	return DDENUMRET_OK;
}

void CL_DisplayCard_DX::enumerate_displaymodes()
{
	HRESULT err = directdraw2->EnumDisplayModes(DDEDM_STANDARDVGAMODES, NULL, (void *) &videomodes, ResolutionList_callback);
	cl_assert(err == DD_OK);
}

CL_Blitters CL_DisplayCard_DX::create_hw_blitters(CL_SurfaceProvider *provider)
{
	CL_Blitters ret;

	CL_Blit_DX *blitter = new CL_Blit_DX(this, provider);
	bool res = blitter->init_surface(&ret);
	if (!res)
	{
		delete blitter;
	}

	return ret;
}

void CL_DisplayCard_DX::set_palette(CL_Palette *pal)
{
	cl_assert(pal != NULL);

	if (!is_inited) return;

	cl_assert(m_target != NULL);
	m_target->set_palette(pal);
}

void CL_DisplayCard_DX::set_palette(char *palette_id, CL_InputSourceProvider *datafile)
{
	if (!is_inited) return;

	if (CL_Display::get_bpp()==8 && m_target != NULL )
	{
		CL_Palette *cur_palette=new CL_Palette(palette_id, datafile);
		m_target->set_palette(cur_palette);
		delete cur_palette;
	}
}

CL_Palette *CL_DisplayCard_DX::get_palette()
{
	return m_target->get_palette();
}

void CL_DisplayCard_DX::set_videomode(CL_VidMode *mode)
{
	cl_assert(mode != NULL);
	set_videomode(mode->width, mode->height, mode->bpp);
}

CL_String CL_DisplayCard_DX::get_name()
{
	return card_name;
}

CL_List<CL_VidMode> CL_DisplayCard_DX::get_videomodes()
{
	return videomodes;
}

void CL_DisplayCard_DX::set_videomode(int width, int height, int bpp)
{
	if (cur_videomode != NULL &&
		width == cur_videomode->width && 
		height == cur_videomode->height &&
		bpp == cur_videomode->bpp) return;

	delete cur_videomode;
	cur_videomode = new CL_VidMode(width, height, bpp);

	if (!is_inited)
	{
		initialize_dx();
	}

	set_gfxmode(width, height, bpp);

	cout << "Setting videomode : " << cur_videomode->asString().get_string() << endl;

	delete m_target;
	m_target = NULL;

	int flags = 0;
	if ((width == 320) && (height == 200) && (bpp == 8))
	{
		flags = DDSDM_STANDARDVGAMODE;
	}
	
	HRESULT err = directdraw2->SetDisplayMode(width, height, bpp/*, 0, flags*/);
	cl_assert(err == DD_OK);

	m_target = new CL_BackbufferTarget_DX(directdraw2, num_backbuffers, video_backbuffer);
}

DDCAPS CL_DisplayCard_DX::get_hel_caps()
{
	HRESULT err;
	DDCAPS hel_caps;
	hel_caps.dwSize = sizeof(DDCAPS);
 
	err = directdraw2->GetCaps(NULL, &hel_caps);	
	cl_assert(err == DD_OK);

	return hel_caps;
}

DDCAPS CL_DisplayCard_DX::get_hal_caps()
{
	HRESULT err;
	DDCAPS hal_caps;
	hal_caps.dwSize = sizeof(DDCAPS);
 
	err = directdraw2->GetCaps(&hal_caps, NULL);
	cl_assert(err == DD_OK);

	return hal_caps;
}

void CL_DisplayCard_DX::clear_display(
	float red, 
	float green, 
	float blue, 
	float alpha)
{
	fill_rect(
		0,
		0,
		get_width(),
		get_height(),
		red,
		green,
		blue,
		alpha);
}

void CL_DisplayCard_DX::fill_rect(
	int x1, 
	int y1, 
	int x2, 
	int y2, 
	float r, 
	float g, 
	float b, 
	float a)
{
	if (a <= 0.01) return;
	if (a <= 0.99 || 
		m_target->m_system_backbuffer != NULL)
	{
		CL_DisplayCard_Generic::fill_rect(x1, y1, x2, y2, r, g, b, a);
		return;
	}

	CL_ClipRect rect(x1, y1, x2, y2);

	CL_ClipRect cur_clip = get_clip_rect();
	if (cur_clip.test_all_clipped(rect)) 
	{
		return;
	}

	CL_ClipRect crect = cur_clip.clip(rect);
	CL_ColorMap cmap(get_target());
	unsigned int fill_color = cmap.calc_color(r, g, b, a);

	DDBLTFX ddbltfx;
	ddbltfx.dwSize=sizeof(ddbltfx);
	ddbltfx.dwFillColor = fill_color;

	LPDIRECTDRAWSURFACE backbuffer;
	DDSCAPS backbuffer_desc;
	backbuffer_desc.dwCaps=DDSCAPS_BACKBUFFER;
	m_target->m_primary_surface->GetAttachedSurface(&backbuffer_desc, &backbuffer);

	RECT dest;
	dest.left = crect.m_x1;
	dest.top = crect.m_y1;
	dest.right = crect.m_x2;
	dest.bottom = crect.m_y2;

	backbuffer->Blt(&dest, NULL, NULL, DDBLT_COLORFILL|DDBLT_WAIT, &ddbltfx);	// clear screen
	backbuffer->Release();
}

void CL_DisplayCard_DX::flip_display(bool sync)
{
	if (m_target != NULL)
	{
		m_target->flip();
	}
}


