/*
	$Id: displaycard_dx.h,v 1.10 1999/09/04 00:40:55 starch Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifndef header_displaycard_dx
#define header_displaycard_dx

#include <ddraw.h>
#include <d3d.h>

#include "Layer1/Display/API/display.h"
#include "Layer1/Display/Generic/displaycard_generic.h"
#include "dx_target.h"

#define RELEASE(s) { s->Release(); s = NULL; }

char *get_dderr(HRESULT err);

static HRESULT CALLBACK ResolutionList_callback(LPDDSURFACEDESC lpDDSurfaceDesc, LPVOID lpContext);

class CL_LowLevelSurface_DX;
class CL_Palette_DX;
class CL_Blit_DX;

class CL_DisplayCard_DX : public CL_DisplayCard_Generic
{
public:
	CL_DisplayCard_DX(int card_no, LPGUID card_uid, CL_String card_name, int options);
	virtual ~CL_DisplayCard_DX();

	// Implementation of CL_DisplayCard specific virtual functions
	virtual void set_palette(char *palette_id, CL_InputSourceProvider *datafile);
	virtual void set_palette(CL_Palette *pal);
	virtual CL_Palette *get_palette();
	virtual void flip_display(bool sync=false);
	virtual void set_videomode(CL_VidMode *mode);
	virtual void set_videomode(int width, int height, int bpp);
	virtual bool is_initialized() { return is_inited; }

	virtual void clear_display(float red=0, float green=0, float blue=0, float alpha=1);
	virtual void fill_rect(int x1, int y1, int x2, int y2, float r, float g, float b, float a);

	virtual CL_String get_name();
	virtual CL_List<CL_VidMode> get_videomodes();
	virtual int get_total_memory() { return -1; }

protected:
	virtual CL_Target *get_target() { return m_target; }
	virtual CL_Target *get_frontbuffer() { return m_target; }
	virtual CL_Blitters create_hw_blitters(CL_SurfaceProvider *provider);

private:
	friend CL_Blit_DX;

	LPDIRECTDRAW directdraw2; 
	CL_BackbufferTarget_DX *m_target;

	bool is_inited;
	LPGUID card_uid;
	CL_String card_name;
	CL_VidMode *cur_videomode;
	CL_List<CL_VidMode> videomodes;

	int num_backbuffers;
	bool video_backbuffer;
	bool create_system_surfaces;

	void initialize_dx();
	void enumerate_displaymodes();

	DDCAPS get_hel_caps();
	DDCAPS get_hal_caps();
};

#endif
