/*
	$Id: dx_target.h,v 1.7 1999/09/04 00:40:55 starch Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifndef header_dx_target
#define header_dx_target

#include <ddraw.h>

#include "../API/target.h"

class CL_Blit_DX;

class CL_Target_DX : public CL_Target
{
public:
	CL_Target_DX();
	virtual ~CL_Target_DX();

	virtual bool lock();
	virtual void unlock();

	virtual void *get_data() { return m_data; }

	virtual unsigned int get_width() { return m_width; }
	virtual unsigned int get_height() { return m_height; }
	virtual unsigned int get_pitch() { return m_pitch; }
	
	virtual unsigned int get_depth() { return m_depth; }
	virtual unsigned int get_red_mask() { return m_red_mask; }
	virtual unsigned int get_green_mask() { return m_green_mask; }
	virtual unsigned int get_blue_mask() { return m_blue_mask; }
	virtual unsigned int get_alpha_mask() { return m_alpha_mask; }

	virtual CL_Palette *get_palette();
	virtual void set_palette(CL_Palette *pal);

protected:
	void set_surface(LPDIRECTDRAWSURFACE surface);

private:
friend CL_Blit_DX;
	void get_palette_from_surface();

	LPDIRECTDRAWSURFACE m_surface;
	void *m_data;
	unsigned int m_width, m_height, m_pitch, m_depth;
	unsigned int m_red_mask, m_green_mask, m_blue_mask, m_alpha_mask;
	CL_Palette m_palette;
};

class CL_OffscreenTarget_DX;
class CL_DisplayCard_DX;

class CL_BackbufferTarget_DX : public CL_Target_DX
{
friend CL_DisplayCard_DX;
friend CL_Blit_DX;
public:
	CL_BackbufferTarget_DX(
		LPDIRECTDRAW ddraw, 
		int backbuffer_count, 
		bool create_video_backbuffer=true);
	virtual ~CL_BackbufferTarget_DX();

	virtual bool is_video() { return m_system_backbuffer==NULL; }
	void flip();

private:
	LPDIRECTDRAWSURFACE m_primary_surface;
	CL_OffscreenTarget_DX *m_system_backbuffer;
	LPDIRECTDRAW m_ddraw;
};

class CL_OffscreenTarget_DX : public CL_Target_DX
{
	friend CL_BackbufferTarget_DX;
public:
	CL_OffscreenTarget_DX(LPDIRECTDRAW ddraw, int width, int height, bool create_video=true);
	virtual ~CL_OffscreenTarget_DX();

	virtual bool is_video() { return m_is_video; }

private:
	LPDIRECTDRAWSURFACE m_surface;
	bool m_is_video;
};

#endif
