/*
	$Id: displaycard_glx.cpp,v 1.3 1999/09/04 14:12:27 vogel Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifdef USE_OPENGL

#include "displaycard_glx.h"
#include "../Generic/GL/blit_gl_generic.h"
#include "System/API/cl_assert.h"
#include "System/API/clanapp.h"


CL_GLX_DisplayCard::CL_GLX_DisplayCard(
	Display *dpy,
	Window root,
	int card_no) :
CL_GL_DisplayCard_Generic(card_no)
{
	this->root = root;
	this->dpy = dpy;

	m_uses_gl = false;
	initialized = false;

	max_texture_size = 220377;
	glGetIntegerv(GL_MAX_TEXTURE_SIZE, &max_texture_size);
}

CL_GLX_DisplayCard::~CL_GLX_DisplayCard()
{
	if (initialized)
	{
		glXDestroyContext(dpy, ctx);
		XDestroyWindow(dpy, win);
	}
}

void CL_GLX_DisplayCard::flip_display(bool sync)
{
	glXSwapBuffers(get_dpy(), get_window());
}

void CL_GLX_DisplayCard::set_videomode(
	int width,
	int height,
	int bpp)
{
	if (initialized) // clean up
	{
		glXDestroyContext(dpy, ctx);
		XDestroyWindow(dpy, win);
		initialized = false;
	}

	set_gfxmode(width, height, bpp);

	int dummy;
	if (!glXQueryExtension(dpy, &dummy, &dummy))
		cl_assert(false);

	static int dblBuf[] = { GLX_RGBA, GLX_DOUBLEBUFFER, None };

	XVisualInfo *visual_info = glXChooseVisual(
		dpy,
		DefaultScreen(dpy),
		dblBuf);

	cl_assert(visual_info != NULL);

	ctx = glXCreateContext(
		dpy,
		visual_info,
		None,
		GL_TRUE);

	cl_assert(ctx != NULL);

	Colormap cmap = XCreateColormap(
		dpy,
		RootWindow(dpy, visual_info->screen),
		visual_info->visual,
		AllocNone);

	XSetWindowAttributes attr;
	attr.colormap = cmap;
	attr.border_pixel = BlackPixel(dpy, DefaultScreen(dpy));

	win = XCreateWindow(
		dpy,
		RootWindow(dpy, visual_info->screen),
		0,
		0,
		get_width(),
		get_height(),
		0,
		visual_info->depth,
		InputOutput,
		visual_info->visual,
		CWBorderPixel | CWBackPixel | CWColormap,
		&attr);
	
	cl_assert(win);

	char *title = CL_ClanApplication::app->get_title();
	XTextProperty text_property;
	XStringListToTextProperty(&title, 1, &text_property);

	XSizeHints size_hints;
	size_hints.x = 0;
	size_hints.y = 0;
	size_hints.width = get_width();
	size_hints.height = get_height();
	size_hints.flags = PSize | PMinSize | PMaxSize;
	size_hints.min_width = get_width();
	size_hints.min_height = get_height();
	size_hints.max_width = get_width();
	size_hints.max_height = get_height();

	size_hints.flags = USPosition | USSize;
	XSetWMProperties(
		dpy,
		win,
		&text_property,
		&text_property,
		0,
		0,
		&size_hints,
		0,
		0);

	glXMakeCurrent(dpy, win, ctx);
	XMapWindow(dpy, win);

	// Listen for keystrokes...
	// ------------------------

	XSelectInput(dpy, win, KeyPressMask|KeyReleaseMask);
	
	// Output OpenGL vendor & renderer
	// -------------------------------
	
	cout << endl;
	cout << "OpenGL status" << endl << endl;
	cout << "Vendor     : " << glGetString(GL_VENDOR) << endl;
	cout << "Renderer   : " << glGetString(GL_RENDERER) << endl;
	cout << "Version    : " << glGetString(GL_VERSION) << endl;
	cout << "Extensions : " << glGetString(GL_EXTENSIONS) << endl << endl;

	int bufferSize, level, renderType, doubleBuffer, stereo, auxBuffers, redSize, greenSize;
	int blueSize, alphaSize, depthSize, stencilSize, acRedSize, acGreenSize, acBlueSize, acAlphaSize;

	glXGetConfig(dpy, visual_info, GLX_BUFFER_SIZE, &bufferSize);
	glXGetConfig(dpy, visual_info, GLX_LEVEL, &level);
	glXGetConfig(dpy, visual_info, GLX_RGBA, &renderType);
	glXGetConfig(dpy, visual_info, GLX_DOUBLEBUFFER, &doubleBuffer);
	glXGetConfig(dpy, visual_info, GLX_STEREO, &stereo);
	glXGetConfig(dpy, visual_info, GLX_AUX_BUFFERS, &auxBuffers);
	glXGetConfig(dpy, visual_info, GLX_RED_SIZE, &redSize);
	glXGetConfig(dpy, visual_info, GLX_GREEN_SIZE, &greenSize);
	glXGetConfig(dpy, visual_info, GLX_BLUE_SIZE, &blueSize);
	glXGetConfig(dpy, visual_info, GLX_ALPHA_SIZE, &alphaSize);
	glXGetConfig(dpy, visual_info, GLX_DEPTH_SIZE, &depthSize);
	glXGetConfig(dpy, visual_info, GLX_STENCIL_SIZE, &stencilSize);
	glXGetConfig(dpy, visual_info, GLX_ACCUM_RED_SIZE, &acRedSize);	
	glXGetConfig(dpy, visual_info, GLX_ACCUM_GREEN_SIZE, &acGreenSize);
	glXGetConfig(dpy, visual_info, GLX_ACCUM_BLUE_SIZE, &acBlueSize);
	glXGetConfig(dpy, visual_info, GLX_ACCUM_ALPHA_SIZE, &acAlphaSize);

	cout << "GLX status" << endl << endl;

/*
	cout << " visual class: ";
	switch (visual_info->class)
	{
		case StaticGray	 : { cout << "StaticGray" << endl; break; }
		case GrayScale 	 : { cout << "GrayScale" << endl; break; }
		case StaticColor : { cout << "StaticColor" << endl; break; }
		case PseudoColor : { cout << "PseudoColor" << endl; break; }
		case TrueColor   : { cout << "TrueColor" << endl; break; }
		case DirectColor : { cout << "DirectColor" << endl; break; }
		default          : { cout << "unknown " << endl; }
	}
*/
	cout << "visual depth: " << visual_info->depth << endl << endl;
	if (glXIsDirect(dpy, ctx)) 
		cout << "direct rendering" << endl;
	if (renderType)
		cout << "RGBA" << endl;
	if (doubleBuffer)
		cout << "double buffer" << endl;
	if (stereo)
		cout << "stereo buffer" << endl;
	cout << endl;

	cout << "GLX_BUFFER_SIZE      : " << bufferSize << endl;
	cout << "GLX_LEVEL            : " <<  level << endl;
//	cout << "GLX_RGBA             : " << renderType << endl;
//	cout << "GLX_DOUBELBUFFER     : " << doubleBuffer << endl;
//	cout << "GLX_STEREO           : " << stereo << endl;
	cout << "GLX_AUX_BUFFERS      : " << auxBuffers << endl;
	cout << "GLX_RED_SIZE         : " << redSize << endl;
	cout << "GLX_GREEN_SIZE       : " << greenSize << endl;
	cout << "GLX_BLUE_SIZE        : " << blueSize << endl;
	cout << "GLX_ALPHA_SIZE       : " << alphaSize << endl;
	cout << "GLX_DEPTH_SIZE       : " << depthSize << endl;
	cout << "GLX_STENCIL_SIZE     : " << stencilSize << endl;
	cout << "GLX_ACCUM_RED_SIZE   : " << acRedSize << endl;	
	cout << "GLX_ACCUM_GREEN_SIZE : " << acGreenSize << endl;
	cout << "GLX_ACCUM_BLUE_SIZE  : " << acBlueSize << endl;
	cout << "GLX_ACCUM_ALPHA_SIZE : " << acAlphaSize << endl;

	cout << endl << endl;
	cout << "GL_MAX_TEXTURE_SIZE  : " << max_texture_size << endl;
	cout << endl << endl;
	
	// Setup OpenGL sanity:
	// --------------------
	if (!uses_gl())	
	{
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluOrtho2D(0.0, (GLdouble) width, (GLdouble) height, 0.0);

		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

		glEnable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);

		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
//		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);	// is default
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );      
	}
	initialized = true;
}

#endif
