/*
	$Id: blit_gl_generic.cpp,v 1.3 1999/09/04 14:12:27 vogel Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifdef USE_OPENGL

#include "blit_gl_generic.h"
#include "../pixeldata.h"

// this can be deleted if the impl. is stable
void check4error()
{
	GLenum error;
	while ((error = glGetError()) != GL_NO_ERROR)
		cout << gluErrorString(error) << endl;
}

CL_Blit_GLTexture::CL_Blit_GLTexture(
	CL_GL_DisplayCard_Generic *card,
	CL_SurfaceProvider *provider)
{
	m_card = card;
	provider->lock();
	
	m_width = provider->get_width();
	m_height = provider->get_height();
	m_no_sprs = provider->get_no_sprs();

//	TODO: use TEXTURE PROXY

	m_texture_width = 2;
	while (m_texture_width<m_width)
		m_texture_width *= 2;

	m_texture_height = 2;
	while (m_texture_height<m_height*m_no_sprs)
		m_texture_height *= 2;
	
/*
	GLint format;
	glTexImage2D(
		GL_PROXY_TEXTURE_2D,
		0,
		GL_RGBA,
		m_texture_width,
		m_texture_height,
		0,
		GL_RGBA,
		GL_UNSIGNED_BYTE,
		NULL);

// MESA segfaults above - not nice!

	glGetTexLevelParameteriv(
		GL_PROXY_TEXTURE_2D, 
		0, 
		GL_TEXTURE_INTERNAL_FORMAT,
		&format);
	cl_assert(format);
*/

	CL_PixelData pixeldata_red(
		255, 0, 0, 0, provider, 1);
	CL_PixelData pixeldata_green(
		0, 255, 0, 0, provider, 1);
	CL_PixelData pixeldata_blue(
		0, 0, 255, 0, provider, 1);
	CL_PixelData pixeldata_alpha(
		0, 0, 0, 255, provider, 1);

//	cout << "texture width  : " << m_texture_width << endl;
//	cout << "texture height : " << m_texture_height << endl << endl;

	unsigned int *texture_data = new unsigned int[m_texture_width*m_texture_height];

	for (int y=0; y<m_height*m_no_sprs; y++)
	{
		unsigned char *data_red = pixeldata_red.get_line_pixel(y);
		unsigned char *data_green = pixeldata_green.get_line_pixel(y);
		unsigned char *data_blue = pixeldata_blue.get_line_pixel(y);
		unsigned char *data_alpha = pixeldata_alpha.get_line_pixel(y);

		// Make sure it is converted correctly!
		for (int x=0; x<m_width; x++)
		{
			unsigned char *texel = (unsigned char*) &texture_data[x+y*m_texture_width];
			texel[0] = data_red[x];
			texel[1] = data_green[x];
			texel[2] = data_blue[x];
			texel[3] = data_alpha[x];
		}
	}
	
// TODO: little error checking

	glGenTextures(1,&m_texture);
	glBindTexture(GL_TEXTURE_2D, m_texture);
	glTexImage2D(
		GL_TEXTURE_2D,
		0,
		GL_RGBA,
		m_texture_width,
		m_texture_height,
		0,
		GL_RGBA,
		GL_UNSIGNED_BYTE,
		texture_data);

	delete[] texture_data;
	provider->unlock();
}

CL_Blit_GLTexture::~CL_Blit_GLTexture()
{
	glDeleteTextures(1,&m_texture);
}

void CL_Blit_GLTexture::blt_noclip(
	CL_Target *target,
	int x,
	int y,
	int spr_no)
{
	if (m_card->uses_gl())
	{
		cl_assert(false);
		
		// the app uses OpenGL, so we have to save state
		// TODO: even more state saving is needed
	
		glPushAttrib(GL_ALL_ATTRIB_BITS);	// narrow range of what is saved
	
		glEnable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);
		glDisable(GL_LIGHTING);
		glDisable(GL_DEPTH_TEST);
		glDisable(GL_FOG);
		glDisable(GL_ALPHA_TEST);
		glDisable(GL_SCISSOR_TEST);		// change this state ?	
		glDisable(GL_STENCIL_TEST);		// change this state ?
	
		glBindTexture(GL_TEXTURE_2D, m_texture);
		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE); // is modulate the right one?
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );

		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
		glMatrixMode(GL_PROJECTION);
		glPushMatrix();
		glLoadIdentity();
		gluOrtho2D(0.0, (GLdouble) m_card->get_width(), 0.0, (GLdouble) m_card->get_height());
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
		glLoadIdentity();
	
		float h1 = ((float) m_height * spr_no) / m_texture_height;
		float h2 = ((float) m_height * (spr_no+1)) / m_texture_height;
		float w = (float) m_width / m_texture_width;
		glBegin(GL_TRIANGLE_STRIP);
			glTexCoord2f(0.0, h1); glVertex2i(x, y);
			glTexCoord2f(  w, h1); glVertex2i(x + m_width, y);
			glTexCoord2f(0.0, h2); glVertex2i(x, y + m_height);
			glTexCoord2f(  w, h2); glVertex2i(x + m_width, y + m_height);
		glEnd();

		glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
		glPopMatrix();
		glPopAttrib();
	}
	else
	{
	    	// the app doesn't care about OpenGL, so we only have to set up things once
		// in displaycard_gl...
//		check4error();

		glBindTexture(GL_TEXTURE_2D, m_texture);
		float h1 = ((float) m_height * spr_no) / m_texture_height;
		float h2 = ((float) m_height * (spr_no+1)) / m_texture_height;
		float w = (float) m_width / m_texture_width;
		glBegin(GL_TRIANGLE_STRIP);
			glTexCoord2f(0.0, h1); glVertex2i(x, y);
			glTexCoord2f(  w, h1); glVertex2i(x + m_width, y);
			glTexCoord2f(0.0, h2); glVertex2i(x, y + m_height);
			glTexCoord2f(  w, h2); glVertex2i(x + m_width, y + m_height);
		glEnd();
	}
}

void CL_Blit_GLTexture::blt_clip(
	CL_Target *target,
	int x,
	int y,
	int spr_no,
	const CL_ClipRect &clip)
{
	// TODO: use glScissor()
	blt_noclip(target,x,y,spr_no); // this will do for now.
}

void CL_Blit_GLTexture::blt_scale_noclip(
	CL_Target *target,
	int x,
	int y,
	int dest_width,
	int dest_height,
	int spr_no)
{
	if (m_card->uses_gl())
	{
		// once the uses_gl part is stable...
	}
	else 
	{
		glBindTexture(GL_TEXTURE_2D, m_texture);
		float h1 = ((float) m_height * spr_no) / m_texture_height;
		float h2 = ((float) m_height * (spr_no+1)) / m_texture_height;
		float w = (float) m_width / m_texture_width;
		glBegin(GL_TRIANGLE_STRIP);
			glTexCoord2f(0.0, h1); glVertex2i(x, y);
			glTexCoord2f(  w, h1); glVertex2i(x + dest_width, y);
			glTexCoord2f(0.0, h2); glVertex2i(x, y + dest_height);
			glTexCoord2f(  w, h2); glVertex2i(x + dest_width, y + dest_height);
		glEnd();
	}	
}

void CL_Blit_GLTexture::blt_scale_clip(
	CL_Target *target,
	int x,
	int y,
	int dest_width,
	int dest_height,
	int spr_no,
	const CL_ClipRect &clip)
{
	// TODO: use glScissor()
	blt_scale_noclip(target, x, y, dest_width, dest_height, spr_no);
}

#endif