/*
	$Id: displaycard_gl_generic.cpp,v 1.3 1999/09/04 15:08:19 vogel Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifdef USE_OPENGL

#include "displaycard_gl_generic.h"
#include "blit_gl_generic.h"
#include "lockable_surface_gl.h"
#include "System/API/cl_assert.h"
#include "System/API/clanapp.h"


CL_LockableSurface* CL_GL_DisplayCard_Generic::create_lockable_surface(
	int width,
	int height,
	int no_sprs,
	int minimum_alpha_mask_size)
{
	return new CL_LockableSurface_GL(
		width,
		height,
		no_sprs,
		minimum_alpha_mask_size,
		this);
}

void CL_GL_DisplayCard_Generic::set_palette(CL_Palette *palette)
{
	memcpy(
		this->palette.palette,
		palette->palette,
		palette->num_colors);
}

CL_Palette *CL_GL_DisplayCard_Generic::get_palette()
{
	return &palette;
}

CL_List<CL_VidMode> CL_GL_DisplayCard_Generic::get_videomodes()
{
	CL_List<CL_VidMode> modes;
	return modes;
}

CL_Target *CL_GL_DisplayCard_Generic::get_target()
{
//	cl_info(1, "get_target not working in generic GL impl");
	return NULL;
}

CL_Target *CL_GL_DisplayCard_Generic::get_frontbuffer()
{
//	cl_info(1, "get_frontbuffer not working in generic GL impl");
	return NULL;
}

void CL_GL_DisplayCard_Generic::sync_buffers()
{
	// TODO: Sync buffers!?
//	cl_info(1, "sync not working in generic GL impl... yet");
}

void CL_GL_DisplayCard_Generic::clear_display(
	float red,
	float green,
	float blue,
	float alpha)
{
	glClearColor(red, green, blue, 1-alpha);	// dv: I think it is "alpha", but Magnus is the one
	glClear(GL_COLOR_BUFFER_BIT);			// who is right more often ;-)
}

void CL_GL_DisplayCard_Generic::fill_rect(
	int x1,
	int y1,
	int x2,
	int y2,
	float r,
	float g,
	float b,
	float a)
{
	// TODO: save & restore state
	glDisable(GL_TEXTURE_2D);
	glBegin(GL_QUADS);
		glColor4f(r,g,b,a);
		glVertex2i(x1, y1);
		glVertex2i(x2, y1);
		glVertex2i(x2, y2);
		glVertex2i(x1, y2);
		glColor4f(1.0, 1.0, 1.0, 1.0);
	glEnd();
	glEnable(GL_TEXTURE_2D);
}

void CL_GL_DisplayCard_Generic::draw_line(
	int x1,
	int y1,
	int x2,
	int y2,
	float r,
	float g,
	float b,
	float a)
{
	// TODO: save & restore state
	glDisable(GL_TEXTURE_2D);
	glBegin(GL_LINES);
		glColor4f(r,g,b,a);
		glVertex2i(x1, y1);
		glVertex2i(x2, y2);
		glColor4f(1.0, 1.0, 1.0, 1.0);
	glEnd();
	glEnable(GL_TEXTURE_2D);
}

CL_Blitters CL_GL_DisplayCard_Generic::create_hw_blitters(CL_SurfaceProvider *provider)
{
	// TODO: Use another blitter if texture is larger than 256x256...
	CL_Blit_GLTexture *blitter = new CL_Blit_GLTexture(this, provider);

	CL_Blitters hw;
	hw.set_noclip(blitter);
	hw.set_clip(blitter);
	hw.set_scale_noclip(blitter);
	hw.set_scale_clip(blitter);

	return hw;
}

#endif
