/*
	$Id: displaycard_gl_generic.h,v 1.5 1999/09/04 15:08:19 vogel Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifndef header_displaycard_gl_generic
#define header_displaycard_gl_generic

#ifdef USE_OPENGL

#include "../displaycard_generic.h"


#ifdef USE_X11
#include "../../X11/display_xwindow.h"
class CL_GL_DisplayCard_Generic : public CL_XWindow_CompatibleCard
#else
class CL_GL_DisplayCard_Generic : public CL_DisplayCard_Generic
#endif
{
public:

#ifdef USE_X11
	CL_GL_DisplayCard_Generic(int card_no) : CL_XWindow_CompatibleCard(card_no) {}	
#else
	CL_GL_DisplayCard_Generic(int card_no) : CL_DisplayCard_Generic(card_no) {}	
#endif

	virtual CL_LockableSurface *create_lockable_surface(
		int width,
		int height,
		int no_sprs,
		int minimum_alpha_mask_size);

	virtual void set_palette(CL_Palette *palette);
	virtual CL_Palette *get_palette();

	virtual CL_String get_name() { return "OpenGL Display"; }
	virtual int get_total_memory() { return -1; }
	virtual bool is_initialized() { return initialized; }
	virtual CL_List<CL_VidMode> get_videomodes();

	virtual CL_Target *get_target();
	virtual CL_Target *get_frontbuffer();

	// whether the app uses OpenGL or not
	virtual bool uses_gl() { return m_uses_gl; }
	virtual int get_max_texture_size() = 0;
	
	// Accelerator functions:
	// ----------------------
	virtual void clear_display(
		float red,
		float green,
		float blue,
		float alpha);
	
	virtual void fill_rect(
		int x1,
		int y1,
		int x2,
		int y2,
		float r,
		float g,
		float b,
		float a);
	
	virtual void draw_line(
		int x1,
		int y1,
		int x2,
		int y2,
		float r,
		float g,
		float b,
		float a);

	virtual void sync_buffers();

protected:
	virtual CL_Blitters create_hw_blitters(CL_SurfaceProvider *provider);
	bool initialized;
	bool m_uses_gl;

private:
	CL_Palette palette;

};

#endif

#endif
