/*
	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	16 bit MMX blitter - Daniel Vogel 99/03/29
*/

#include "blit_transparent_mmx_16.h"

extern "C"
{
	void blit_transparent_mmx_16_asm(
	        unsigned char * src, 
		unsigned char * dest,
		int width,
		int s_add,
		int d_add,
		int height);
}

CL_Blit_Transparent_MMX_16::CL_Blit_Transparent_MMX_16(
        CL_SurfaceProvider *provider,
	int _bytes_per_pixel,
	CL_Target *target) :
CL_Blit_Transparent_MMX(provider, _bytes_per_pixel)
{
	CL_PixelData input(
	        target->get_red_mask(),
		target->get_green_mask(),
		target->get_blue_mask(),
		target->get_alpha_mask(),
		provider,
		bytes_per_pixel);
		
	CL_PixelData alpha_only(0,0,0,255,provider,1);
		
	double * dest = image;

	for (int y=0; y<height*no_sprs; y++)
	{
	        double * src = (double *) input.get_line_pixel(y);
		double * alpha = (double *) alpha_only.get_line_pixel(y);
		
		for (int x=0; x<width/8;x++)
		{
			*(dest++) = *(src++);
			*(dest++) = *(src++);
			*(dest++) = *(alpha++);
		}
		
		unsigned char * dc = (unsigned char *) dest;
		unsigned char * sc = (unsigned char *) src;
		unsigned char * ac = (unsigned char *) alpha;
		
		if (width%8!=0) dest += bytes_per_pixel + 1;  // keep alignement

		for (int x=0; x<width%8;x++)
		{
			*(dc++) = *(sc++);
			*(dc++) = *(sc++);
			*(dc++) = *(ac++);
		}

	}
}

void CL_Blit_Transparent_MMX_16::blt_noclip(
	CL_Target *target,
	int x,
	int y,
	int spr_no)
{
	if (!target->lock()) return;

	unsigned int dest_bytes_per_pixel = (target->get_depth()+7)/8;
	unsigned int dest_pitch = target->get_pitch();
	unsigned char *dest = (unsigned char *) target->get_data();

	dest += x*dest_bytes_per_pixel + y*dest_pitch;
	unsigned char * s = ((unsigned char*) image) + (height*spr_no)*bytes_per_line;
	unsigned char * d = dest;

	blit_transparent_mmx_16_asm(
                s,
		d,
		width, 
		bytes_per_line - (bytes_per_pixel+1)*width, 
		dest_pitch - bytes_per_pixel*width,
		height);
	
	target->unlock();
}
