;	------------------------------------------------------------------------
;	ClanLib, the platform independent game SDK.
;
;	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
;	version 2. See COPYING for details.
;
;	For a total list of contributers see CREDITS.
;	------------------------------------------------------------------------
;
;	16 bit MMX blitter - Daniel Vogel 99/03/29


BITS 32

GLOBAL blit_transparent_mmx_16_asm

SECTION .text

;; blit_transparent_mmx_16_asm(uchar *src, uchar *dest, uint width, uint s_add, uint d_add, uint height)
;; --------------------------------------
;;  4: uchar *src
;;  8: uchar *dest
;; 12: uint width
;; 16: uint s_add
;; 20: uint d_add
;; 24: uint height
;; --------------------------------------
ALIGN 8
blit_transparent_mmx_16_asm:
	push		esi			; save used registers (6)
	push		edi
	push		eax
	push		ebx
	push		ecx
	push		edx

	mov		esi, [esp+24+4]		; src
	mov		edi, [esp+24+8]		; dest
	mov		ecx, [esp+24+12]	; width
	mov		edx, [esp+24+12]	; width
	mov		ebx, [esp+24+24]	; height

	shr		ecx, BYTE 3		; /8
	and		edx, BYTE 7		; %8

ALIGN 4
.loop_q:
	movq		mm0, [esi]
	movq		mm1, [esi+8]
	movq		mm2, [esi+16]
	movq		mm3, [esi+16]		
	movq		mm4, [edi]
	movq		mm5, [edi+8]

	punpcklbw	mm2, mm2
	punpckhbw	mm3, mm3

	pand		mm0, mm2
	pand		mm1, mm3
	
	pandn		mm2, mm4
	pandn		mm3, mm5

	por		mm0, mm2
	por		mm1, mm3

	movq		[edi], mm0
	movq		[edi+8], mm1
	
	add		esi, BYTE 24
	add		edi, BYTE 16

	dec		ecx
	jz		.loop_q_end
	jmp		.loop_q

.loop_q_end:
	and		edx, edx
	jz		.loop_y
	xor		eax, eax

.loop_b:
	mov		cx, [esi]
	mov		al, [esi+2]
	and		eax, eax
	jz		.loop_b_trans
	mov		[edi], cx
	
.loop_b_trans:
	add		esi, BYTE 3
	add 		edi, BYTE 2

	dec		edx
	jz		.loop_y
	jmp		.loop_b

.loop_y:
	mov		ecx, [esp+24+12]	; width
	mov		edx, [esp+24+12]	; width
	add		esi, [esp+24+16]	; src pitch
	add		edi, [esp+24+20]	; dest pitch

	shr		ecx, BYTE 3		; /8
	and		edx, BYTE 7		; %8

	dec		ebx
	jz		.end
	jmp		.loop_q

.end:	
	emms					; reset MMX
	pop		edx			; restore used registers (6)
	pop		ecx
	pop		ebx
	pop		eax
	pop		edi
	pop		esi
	ret
