/*
	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#include "blit_transparent_rle_16.h"

unsigned char * CL_Blit_Transparent_RLE_16::make_rle_line(unsigned char *src_line, unsigned char *alpha_line)
{
	int max_size = calc_rle_size(alpha_line);
	unsigned char *rle = new unsigned char[max_size];
	
	int pos = 0;
	
	int last_state = cmd_end;
	int size_pos = -1;
	int skip_size = 0; // aka. copy_size

	for (int x=0; x<width; x++)
	{
		int i;
		switch(alpha_line[x])
		{
		case 0:    // transparent pixel
				
			if (last_state != cmd_skip)
			{
				if (size_pos != -1)
				{
					unsigned short *rle_s = (unsigned short*) &rle[size_pos];
					*rle_s = skip_size;
				}
			
				rle[pos++] = cmd_skip;
				
				size_pos = pos;
				skip_size = 0;
				pos+=2;
			}
			last_state = cmd_skip;
			skip_size += bytes_per_pixel;
			src_line += bytes_per_pixel;
			break;
			
		case 255:  // opaque

			if (last_state != cmd_copy)
			{
				if (size_pos != -1)
				{
					unsigned short *rle_s = (unsigned short*) &rle[size_pos];
					*rle_s = skip_size;
				}
				
				rle[pos++] = cmd_copy;
				
				size_pos = pos;
				skip_size = 0;
				pos+=2;
			}
			last_state = cmd_copy;
			
			for (i=0; i<bytes_per_pixel; i++) rle[pos++] = *(src_line++);
			skip_size += bytes_per_pixel;
			break;
			
		default:  // use alpha value
			
			if (last_state != cmd_alpha)
			{
				if (size_pos != -1)
				{
					unsigned short *rle_s = (unsigned short*) &rle[size_pos];
					*rle_s = skip_size;
				}
			
				rle[pos++] = cmd_alpha;
				
				size_pos = pos;
				skip_size = 0;
				pos+=2;
			}
			last_state = cmd_alpha;
			
			unsigned short a = alpha_line[x] + 1; // + 1 cause of >> 8 = / 256
			unsigned short c = *((unsigned short*) src_line);
			unsigned char b = (c)  & 31;
			unsigned char g = (c >> 5)  & 31;
			unsigned char r = (c >> 10) & 31;
			
			c = (((b * a) >> 8) & 31)  | 
			   ((((g * a) >> 8) & 31) << 5) |
			   ((((r * a) >> 8) & 31) << 10);
			
			rle[pos++] = c & 255;
			rle[pos++] = c >> 8; 
			
			src_line+=2;
			
			rle[pos++] = ~ alpha_line[x]; // ! alpha
			skip_size++; 
			// size here does tell how many blocks (bytes_per_pixel + alpha) will follow
			break;
		}
	}
	if (size_pos != -1)
	{
		unsigned short *rle_s = (unsigned short*) &rle[size_pos];
		*rle_s = skip_size;
	}

	rle[pos++] = cmd_end;

	cl_assert(pos == max_size); // rle compressor BROKEN!
	
	return rle;
}

void CL_Blit_Transparent_RLE_16::blt_line(unsigned char *s, unsigned char *d)
{
	unsigned char cmd;
	unsigned short size;

	while (true)
	{
		cmd = *(s++);
		switch (cmd)
		{
		case cmd_skip: 
				
			size = *((unsigned short *) s);
			d+=size;
			s+=2;
			break;
			
		case cmd_end:

			return;
			
		case cmd_copy:
			
			size = *((unsigned short *) s);
			s+=2;
			memcpy(d, s, size);
			d+=size;
			s+=size;
			break;

		case cmd_alpha:
			
			size = *((unsigned short *) s);
			s+=2;
			for (int i=0; i<size; i++)
			{
				unsigned short   c1 = *((unsigned short*) s);
				unsigned short * c2 = ((unsigned short*) d);
				
				s+=2;
				d+=2;
				
				unsigned short a = *(s++); // no + 1 as actually was a*(256/255-1)
				// either speed or accuracy - but this is a "minor bug", so I tend
				// towards speed - I'll explain it in more detail if wanted - dv
				unsigned char b1 = (c1) & 31;
				unsigned char g1 = (c1 >> 5) & 31;
				unsigned char r1 = (c1 >> 10) & 31;
				
				unsigned char b2 = (*c2) & 31;
				unsigned char g2 = (*c2 >> 5) & 31;
				unsigned char r2 = (*c2 >> 10) & 31;
				
				*c2 =  ((b1 + ((b2 * a) >> 8)) & 31)  |
				      (((g1 + ((g2 * a) >> 8)) & 31) << 5) |
				      (((r1 + ((r2 * a) >> 8)) & 31) << 10);
	  		
			}
			break;
			
		}
	}
}
