/*
	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#include "blit_transparent_rle_8.h"

unsigned char * CL_Blit_Transparent_RLE_8::make_rle_line(unsigned char *src_line, unsigned char *alpha_line)
{
	int max_size = calc_rle_size(alpha_line);
	unsigned char *rle = new unsigned char[max_size];
	
	int pos = 0;
	
	int last_state = cmd_end;
	int size_pos = -1;
	int skip_size = 0; // aka. copy_size

	for (int x=0; x<width; x++)
	{
		int i;
		switch(alpha_line[x])
		{
		case 0:    // transparent pixel

			if (last_state != cmd_skip)
			{
				if (size_pos != -1)
				{
					unsigned short *rle_s = (unsigned short*) &rle[size_pos];
					*rle_s = skip_size;
				}
				rle[pos++] = cmd_skip;
				size_pos = pos;
				skip_size = 0;
				pos+=2;
			}
			last_state = cmd_skip;
			skip_size += bytes_per_pixel;
			src_line += bytes_per_pixel;
			break;

		case 255:  // opaque

			if (last_state != cmd_copy)
		        {
				if (size_pos != -1)
				{
					unsigned short *rle_s = (unsigned short*) &rle[size_pos];
					*rle_s = skip_size;
				}
				rle[pos++] = cmd_copy;
				size_pos = pos;
				skip_size = 0;
				pos+=2;
			}
			last_state = cmd_copy;

			for (i=0; i<bytes_per_pixel; i++) rle[pos++] = *(src_line++);
			skip_size += bytes_per_pixel;
			break;

		default:  // use alpha value
			cl_info(1, "no alpha support for PAL8");
			cl_assert(false);
		}
	}

	if (size_pos != -1)
	{
		unsigned short *rle_s = (unsigned short*) &rle[size_pos];
		*rle_s = skip_size;
	}

	rle[pos++] = cmd_end;

	cl_assert(pos == max_size); // rle compressor BROKEN!
	
	return rle;
}


void CL_Blit_Transparent_RLE_8::blt_line(unsigned char *s, unsigned char *d)
{
	unsigned char cmd;
	unsigned short size;

	while (true)
	{
		cmd = *(s++);
		switch (cmd)
		{
		case cmd_skip: 
			size = *((unsigned short *) s);
			d+=size;
			s+=2;
			break;
			
		case cmd_end:
			return;
			
		case cmd_copy:
			size = *((unsigned short *) s);
			s+=2;
			memcpy(d, s, size);
			d+=size;
			s+=size;
			break;

		case cmd_alpha:
			cl_info(1, "no alpha support for PAL8");
			cl_assert(false);	
		}
	}
}
