/*
	$Id: displaycard_generic.h,v 1.10 1999/08/08 18:45:45 fischlustig Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifndef header_displaycard_generic
#define header_displaycard_generic

#include "Layer1/Display/API/surfaceprovider.h"
#include "Layer1/Display/API/displaycard.h"
#include "Layer1/Display/API/cliprect.h"
#include "Common/stack.h"

class CL_CardSurface;
class CL_LockableSurface;
class CL_Target;
class CL_CardSurface_Blitter;
class CL_Blitters;

class CL_DisplayCard_Generic : public CL_DisplayCard
{
public:
	CL_DisplayCard_Generic(int card_no)
	{
		m_card_no = card_no;
		m_width = 0;
		m_height = 0;
		m_bpp = 0;
		m_cur_clip = CL_ClipRect(0,0,0,0);
	}

	virtual ~CL_DisplayCard_Generic() {;}
	
	virtual CL_LockableSurface *create_lockable_surface(
		int width,
		int height,
		int no_sprs,
		int minimum_alpha_mask_size);
	
	virtual CL_CardSurface *create_cardsurface_video(
		CL_SurfaceProvider *provider);

	virtual CL_CardSurface *create_cardsurface_system(
		CL_SurfaceProvider *provider);

	virtual int get_card_no() { return m_card_no; }

	virtual int get_width() { return m_width; }
	virtual int get_height() { return m_height; }
	virtual int get_bpp() { return m_bpp; }
	
	virtual void push_clip_rect() { m_clip_stack.push(m_cur_clip); }
	virtual void push_clip_rect(const CL_ClipRect &clip)
	{
		push_clip_rect();
		m_cur_clip = m_cur_clip.clip(clip);
	}
	virtual CL_ClipRect get_clip_rect() { return m_cur_clip; }
	virtual void set_clip_rect(const CL_ClipRect &clip) { m_cur_clip = clip; }
	virtual void pop_clip_rect() { m_cur_clip = m_clip_stack.pop(); }

	virtual void clear_display(float red=0, float green=0, float blue=0, float alpha=1);
	virtual void fill_rect(int x1, int y1, int x2, int y2, float r, float g, float b, float a);
	virtual void draw_line(int x1, int y1, int x2, int y2, float r, float g, float b, float a);
	
	virtual void sync_buffers();

	virtual CL_Target *get_target()=0;
	virtual CL_Target *get_frontbuffer()=0;

protected:
	friend CL_CardSurface_Blitter;

	void set_gfxmode(int width, int height, int bpp)
	{
		m_width = width;
		m_height = height;
		m_bpp = bpp;
		
		m_cur_clip = CL_ClipRect(0, 0, width, height);
		m_clip_stack.clear();
	}

	// Implement those to get hardware support:
	virtual CL_Blitters create_hw_blitters(CL_SurfaceProvider *provider);

	CL_ClipRect m_cur_clip;
	CL_Stack<CL_ClipRect> m_clip_stack;

private:
	int m_card_no;

	int m_width;
	int m_height;
	int m_bpp;

	void fill_rect_opaque(int x1, int y1, int x2, int y2, float r, float g, float b, float a);
	
//  moved to protected, fbdev_card accesses it. 1999/08/08 fischlustig (Denis)
//	CL_Stack<CL_ClipRect> m_clip_stack;
//	CL_ClipRect m_cur_clip;

};

#endif
