/*
	$Id: lockable_surface_software.cpp,v 1.10 1999/09/04 15:08:18 vogel Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#include <stdlib.h>
#include "displaycard_generic.h"
#include "lockable_surface_software.h"
#include "Layer1/System/API/cl_assert.h"

CL_LockableSurface_Software::CL_LockableSurface_Software(
	int width,
	int height,
	int no_sprs,
	int minimum_alpha_mask_size,
	CL_DisplayCard_Generic *card)
{
	m_width = width;
	m_height = height;
	m_no_sprs = no_sprs;
	m_colorkey = -1;
	m_card = card;

	if (minimum_alpha_mask_size > 0) m_alpha_mask = 0x000000ff;
	else m_alpha_mask = 0;
	
	m_data = new int[width*height*no_sprs];
	
	m_handle = Hermes_ConverterInstance(HERMES_CONVERT_NORMAL);
	cl_assert(m_handle != 0);
	
	m_src.bits = 8 * sizeof(int);
	m_src.indexed = false;
	m_src.r = get_red_mask();
	m_src.g = get_green_mask();
	m_src.b = get_blue_mask();
	
	m_target = m_card->get_target();
	cl_assert(m_target);
	
	m_dest.bits = m_target->get_depth();
	m_dest.indexed = false;
	m_dest.r = m_target->get_red_mask();
	m_dest.g = m_target->get_green_mask();
	m_dest.b = m_target->get_blue_mask();
}

CL_LockableSurface_Software::~CL_LockableSurface_Software()
{
	Hermes_ConverterReturn(m_handle);
	delete[] m_data;
}

void CL_LockableSurface_Software::set_colorkey(int colorkey)
{
	m_colorkey = colorkey;
}

int CL_LockableSurface_Software::get_colorkey() const
{
	return m_colorkey;
}

unsigned int CL_LockableSurface_Software::get_no_sprs() const
{
	return m_no_sprs;
}

void CL_LockableSurface_Software::put_screen(
	int x,
	int y,
	int spr_no)
{
	if (m_colorkey == -1 && m_alpha_mask == 0) // opaque surface -> use hermes.
	{
		put_screen(x, y, (int) m_width, (int) m_height, spr_no);
	} 
	else
	{
		// [enter code here - ]
	}
}

void CL_LockableSurface_Software::put_screen(
	int x,
	int y,
	float scale_x,
	float scale_y,
	int spr_no)
{
	put_screen(x, y, (int) (m_width * scale_x), (int) (m_height * scale_y), spr_no);
}

void CL_LockableSurface_Software::put_screen(
	int x,
	int y,
	int size_x,
	int size_y,
	int spr_no)
{
	if (m_colorkey == -1 && m_alpha_mask == 0) // opaque surface -> use hermes.
	{
		m_target = m_card->get_target();

		int res = Hermes_ConverterRequest(m_handle, &m_src, &m_dest);
		cl_assert(res != 0);
		
		m_target->lock();
		res = Hermes_ConverterCopy(
			m_handle,
			m_data,
			0,
			m_height*spr_no,
			m_width,
			m_height,
			m_width*sizeof(int),
			m_target->get_data(),
			x,
			y,
			size_x,
			size_y,
			m_target->get_pitch());

		cl_assert(res != 0);
		m_target->unlock();
	}
	else
	{
		cl_info(info_display, "Cannot blit colorkey/alpha lockable surfaces... yet");
	}
}
