/*
	$Id: lockable_surface_software.h,v 1.5 1999/08/29 18:18:35 vogel Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifndef header_lockable_surface_software
#define header_lockable_surface_software

#include "../API/lockable_surface.h"
extern "C"
{
	#include <Hermes/Hermes.h>
}


class CL_DisplayCard_Generic;
class CL_LockableSurface_Software : public CL_LockableSurface
{
public:
	CL_LockableSurface_Software(
		int width,
		int height,
		int no_sprs,
		int minimum_alpha_mask_size,
		CL_DisplayCard_Generic *card);

	virtual ~CL_LockableSurface_Software();

	// From CL_Target:
	// ---------------
	virtual bool lock() { return true; }
	virtual void unlock() { return; }

	virtual void *get_data() { return m_data; }

	virtual unsigned int get_width() { return m_width; }
	virtual unsigned int get_height() { return m_height; }
	virtual unsigned int get_pitch() { return m_width*sizeof(int); }

	virtual unsigned int get_depth() { return sizeof(int)*8; }

	virtual unsigned int get_red_mask() { return 0xff000000; }
	virtual unsigned int get_green_mask() { return 0x00ff0000; }
	virtual unsigned int get_blue_mask() { return 0x0000ff00; }
	virtual unsigned int get_alpha_mask() { return m_alpha_mask; }

	virtual CL_Palette *get_palette() { return NULL; }

	// From CL_LockableSurface:
	// ------------------------
	virtual void set_colorkey(int colorkey);
	virtual int get_colorkey() const;

	virtual unsigned int get_no_sprs() const;

	virtual void put_screen(int x, int y, int spr_no);
	virtual void put_screen(int x, int y, float scale_x, float scale_y, int spr_no);
	virtual void put_screen(int x, int y, int size_x, int size_y, int spr_no);

private:
	int *m_data;
	int m_colorkey;
	int m_no_sprs;
	unsigned int m_width;
	unsigned int m_height;
	unsigned int m_alpha_mask;

	HermesHandle m_handle;
	HermesFormat m_src, m_dest;

	CL_DisplayCard_Generic *m_card;
	CL_Target *m_target;
};

#endif
