/*
	$Id: surface_generic.h,v 1.4 1999/05/27 21:44:12 starch Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifndef header_surface_generic
#define header_surface_generic

#include "cardsurface.h"

class CL_Surface_Generic : public CL_Surface
{
protected:
	CL_SurfaceProvider *provider;
	bool delete_provider;
	CL_CardSurface **card_surfaces;
	int width, height, no_sprs;
	int translate_x, translate_y;

	inline int ensure_surface_prepared(CL_DisplayCard *card);

public:
	static CL_SurfacePriority *priority_handler;

	CL_Surface_Generic(
		CL_SurfaceProvider *provider,
		bool delete_provider);

	virtual ~CL_Surface_Generic();

	virtual void flush(CL_DisplayCard *card = NULL);
	virtual CL_SurfaceProvider *get_provider() const;

	virtual CL_CardSurface *get_cardsurface(int surface_num);
	virtual CL_CardSurface *get_cardsurface(CL_DisplayCard *card = NULL);

	// Display functions
	virtual void put_screen(
		int x,
		int y,
		int spr_no=0,
		CL_DisplayCard *card=NULL);

	virtual void put_screen(
		int x,
		int y,
		float scale_x,
		float scale_y,
		int spr_no=0,
		CL_DisplayCard *card=NULL);

	virtual void put_screen(
		int x,
		int y,
		int size_x,
		int size_y,
		int spr_no=0,
		CL_DisplayCard *card=NULL);

	virtual int get_width() const { return width; }
	virtual int get_height() const { return height; }
	virtual int get_no_sprs() const { return no_sprs; }

	// is_video() returns true if in videomemory
	// on the specified card (null = current dispcard)
	virtual bool is_video(CL_DisplayCard *card = NULL) const;
	virtual bool is_loaded(CL_DisplayCard *card = NULL) const;

	// returns true if successfully loaded into videomemory, or already there
	virtual bool convert_video(CL_DisplayCard *card = NULL);

	// never fails! (or serious heap usage!)
	virtual bool convert_system(CL_DisplayCard *card = NULL);

	virtual void reload();
};

#endif
