/*
	$Id: surfaceprovider.cpp,v 1.7 1999/09/11 09:33:26 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#include "Layer1/Display/API/surfaceprovider.h"
#include "Layer1/System/API/cl_assert.h"

int CL_SurfaceProvider::get_bytes_pr_pixel() const
{
	switch (get_pixel_format())
	{
	case PAL8:
		return 1;
		break;

	case RGB565:
		return 2;
		break;
	
	case RGBA4444:
		return 2;
		break;

	case RGBA5551:
		return 2;
		break;

	case RGB888:
		return 3;
		break;

	case RGBA8888:
		return 4;
		break;

	default:
		cl_assert(false);
	}
	// Dummy
	return -1;
}

bool CL_SurfaceProvider::pixel_transparent(int x, int y, int sprno) const
{
	int transcol = get_transcol();
	if (transcol == -1) return false;

	cl_assert(x>=0&&y>=0&&x<get_width()&&y<get_height()*get_no_sprs()&&sprno>=0&&sprno<get_no_sprs());

	int bytes_pr_pixel = get_bytes_pr_pixel();
	unsigned char *surface_data = (unsigned char *) get_surface_data();

	unsigned char *pos = &surface_data[
		x*bytes_pr_pixel+
		y*get_pitch()+
		sprno*get_pitch()*get_height()];

	switch (bytes_pr_pixel)
	{
	case 1:
		return *pos == transcol;
	case 2:
		return *((unsigned short *) pos) == transcol;
	case 3:
		{
			// This may not work at all (can't test it!)
			unsigned long val = *((unsigned long *) pos);
			val &= 0xffffff00;
			return val == (unsigned long) transcol;
		}
	case 4:
		return *((unsigned long *) pos) == (unsigned long) transcol;
	default:
		cl_assert(false); // Nutty business!
		return false;
	}
}
