/*
	Class:   CL_SurfaceProvider_32bpp

	Purpose: Convert a PAL8 surface provider to 16 bpp. This is needed since
	         Hermes doesn't support colorkeys (atleast not when this code
	         was written).
*/

#ifndef header_surfaceprovider_32bpp
#define header_surfaceprovider_32bpp

#include "../API/display.h"
#include "Layer1/System/API/cl_assert.h"

class CL_SurfaceProvider_32bpp : public CL_SurfaceProvider
{
public:
	CL_SurfaceProvider_32bpp(CL_SurfaceProvider *src)
	{
		src->lock();
		cl_assert(src->get_pixel_format() == PAL8);

		m_width = src->get_width();
		m_height = src->get_height();
		m_no_sprs = src->get_no_sprs();
		
		m_data = new unsigned int[m_width*m_height*m_no_sprs];
		
		CL_Palette *pal = src->get_palette();
		if (pal == NULL) pal = CL_Display::get_palette();

		int pitch = src->get_pitch();
		int transcol = src->get_transcol();
		unsigned char *ptr = (unsigned char *) src->get_surface_data();
		
		for (int y=0; y<m_height*m_no_sprs; y++)
		{
			for (int x=0; x<m_width; x++)
			{
				int color = ptr[x+y*pitch];
			
				m_data[x+y*m_width] =
					(pal->palette[color*3+0] << 24) +
					(pal->palette[color*3+1] << 16) +
					(pal->palette[color*3+2] << 8);
				
				// Set alphamask according to transparency
				if (transcol == -1 || color != transcol)
				{
					m_data[x+y*m_width] += 255;
				}
			}
		}
		
		src->unlock();
	}

	virtual ~CL_SurfaceProvider_32bpp()
	{
		delete[] m_data;
	}

	virtual int get_pitch() const { return m_width*sizeof(int); }
	virtual int get_width() const { return m_width; }
	virtual int get_height() const { return m_height; }
	virtual int get_no_sprs() const { return m_no_sprs; }
	virtual EPixelFormat get_pixel_format() const { return RGBA8888; }
	virtual CL_Palette *get_palette() const { return NULL; }
	virtual void lock() { return; }
	virtual void unlock() { return; }
	virtual void *get_surface_data() const { return m_data; }
	virtual int get_transcol() const { return -1; }
	
protected:
	unsigned int *m_data;
	int m_width, m_height, m_no_sprs;
};

#endif
