/*
	$Id: displaycard_ptc.cpp,v 1.8 1999/07/05 21:08:10 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifdef USE_PTC

// #include "Layer1/System/Unix/PTC/ptc.h"
#include <ptc/ptc.h>
#include "displaycard_ptc.h"

#ifndef WIN32
#include "Layer1/System/Generic/string_asm.h"
#else
#define asm_memmove memcpy
#define asm_memset memset
#endif

CL_DisplayCard_PTC::CL_DisplayCard_PTC(int card_no)
: CL_XWindow_CompatibleCard(card_no)
{
	m_initialized = false;
	m_target = new CL_Target_PTC(&m_console, &m_palette);
}

CL_DisplayCard_PTC::~CL_DisplayCard_PTC()
{
	if (m_initialized) m_console.close();
	delete m_target;
}

void CL_DisplayCard_PTC::flip_display(bool /*sync*/)
{
	m_console.update();
}

CL_Target *CL_DisplayCard_PTC::get_target()
{
	return m_target;
}

CL_Target *CL_DisplayCard_PTC::get_frontbuffer()
{
	return m_target;
}

/*
	MBN: Not needed anymore - instead make ClanLib call Hermes' clear func.

void CL_DisplayCard_PTC::clear_display(
	float red,
	float green,
	float blue,
	float alpha)
{
	Color color(red, green, blue, alpha);
	m_console.clear(color);
}
*/

void CL_DisplayCard_PTC::set_palette(CL_Palette *cl_pal)
{
	int32 pal_ptr[3*256];

	for (int i=0; i<256; i++)
	pal_ptr[i] = ((cl_pal->palette[i*3]&0xff)<<16)|
	             ((cl_pal->palette[i*3+1]&0xff)<<8)|
	             (cl_pal->palette[i*3+2]&0xff);

	Palette pal(pal_ptr);
	m_console.palette(pal);

	asm_memmove((char *) m_palette.palette, (char *) cl_pal->palette, 3*256);
}

CL_Palette *CL_DisplayCard_PTC::get_palette()
{
	return &m_palette;
}

void CL_DisplayCard_PTC::set_videomode(
	int width,
	int height,
	int bpp)
{
	set_gfxmode(width, height, bpp);
	
//	Format format(16, 0x7c00, 0x3c0, 0x1f);
	Format format;
	m_console.open("", width, height, format);
	
	m_initialized = true;
}

bool CL_DisplayCard_PTC::is_initialized()
{
	return m_initialized;
}

CL_List<CL_VidMode> CL_DisplayCard_PTC::get_videomodes()
{
	// PTC doesn't have a list of videomodes, does it?

	CL_List<CL_VidMode> list;
	return list;
}

#endif /* USE_PTC */
