/*
	$Id: displaycard_ptc.h,v 1.5 1999/07/05 21:08:10 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifndef header_displaycard_ptc
#define header_displaycard_ptc

#include "Layer1/Display/Generic/displaycard_generic.h"
#include "Layer1/Display/API/target.h"
#include "Layer1/Display/X11/display_xwindow.h"
#include "Layer1/System/API/cl_assert.h"

class CL_Target_PTC;

class CL_DisplayCard_PTC
: public CL_XWindow_CompatibleCard
{
public:
	CL_DisplayCard_PTC(int card_no);
	virtual ~CL_DisplayCard_PTC();
	
	// CL_XWindow_CompatibleCard:
	// --------------------------
	
	virtual Window get_window() { return m_console.getX11Window(); }
	virtual Display *get_dpy() { return m_console.getX11Display(); }

	// CL_DisplayCard functions:
	// -------------------------

	virtual void flip_display(bool sync=false);
	virtual void set_palette(CL_Palette *palette);
	virtual CL_Palette *get_palette();
	virtual void set_videomode(int width, int height, int bpp);
	virtual bool is_initialized();
	
	virtual CL_String get_name() { return "OpenPTC Display"; }
	virtual int get_total_memory() { return -1; }
	virtual CL_List<CL_VidMode> get_videomodes();

	// CL_DisplayCard_Generic functions:
	// ---------------------------------
	virtual CL_Target *get_target();
	virtual CL_Target *get_frontbuffer();

private:
	CL_Palette m_palette;
	bool m_initialized;
	
	Console m_console;
	CL_Target_PTC *m_target;
};

class CL_Target_PTC : public CL_Target
{
public:
	CL_Target_PTC(Console *console, CL_Palette *pal)
	{
		m_console = console;
		m_pal = pal;
		m_lock_ref = 0;
	}

	virtual bool lock()
	{
		m_lock_ref++;
		if (m_lock_ref > 1) return true; // already locked
		
		m_lock_ptr = m_console->lock();
		
		return true;
	}

	virtual void unlock()
	{
		m_lock_ref--;
		cl_assert(m_lock_ref >= 0);

		if (m_lock_ref == 0) m_console->unlock();
	}

	virtual void *get_data()
	{
		return m_lock_ptr;
	}

	virtual unsigned int get_width() { return m_console->width(); }
	virtual unsigned int get_height() { return m_console->height(); }
	virtual unsigned int get_pitch() { return m_console->pitch(); }

	virtual unsigned int get_depth()
	{
		Format format = m_console->format();
		return format.bits();
	}
	
	virtual unsigned int get_red_mask()
	{
		Format format = m_console->format();
		return format.r();
	}

	virtual unsigned int get_green_mask()
	{
		Format format = m_console->format();
		return format.g();
	}

	virtual unsigned int get_blue_mask()
	{
		Format format = m_console->format();
		return format.b();
	}

	virtual unsigned int get_alpha_mask()
	{
		Format format = m_console->format();
		return format.a();
	}

	virtual CL_Palette *get_palette()
	{
		return m_pal;
	}

private:
	CL_Palette *m_pal;
	Console *m_console;
	int m_lock_ref;
	void *m_lock_ptr;
};

#endif
