/*
	$Id: display_xwindow.cpp,v 1.21 1999/09/15 21:50:02 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifdef USE_X11

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <iostream.h>

#include "display_xwindow.h"
#include "System/Unix/implementation_xwindow.h"
#include "System/API/cl_assert.h"
#include "System/API/clanapp.h"
#include "System/API/error.h"

CL_XWindow_DisplayCard::CL_XWindow_DisplayCard(
	Display *dpy,
	Window root,
	int card_no)
: CL_XWindow_CompatibleCard(card_no)
{
	m_initialized = false;
	m_dpy = dpy;
	m_root = root;
	m_win = 0;
	m_gc = 0;

	m_scr = DefaultScreen(m_dpy);
//	m_visual_info = DefaultVisual(dpy, m_scr);
	cl_assert(XMatchVisualInfo(dpy, m_scr, DefaultDepth(dpy,m_scr), TrueColor, &m_visual_info));
/*
	m_color_map = XCreateColormap(
		dpy,
		RootWindow(dpy, m_scr),
		m_visual_info.visual,
		AllocNone);
*/
/*
	m_gc = DefaultGC(dpy, m_scr);

	XGCValues gcvalues;
	gcvalues.graphics_exposures = False;
	XChangeGC(
		dpy,
		m_gc,
		GCGraphicsExposures,
		&gcvalues);
*/

	m_palette = new CL_Palette();

	m_target = NULL;
	m_target_front = NULL;

	CL_System_Generic::keep_alives.add(this);
}

CL_XWindow_DisplayCard::~CL_XWindow_DisplayCard()
{
	CL_System_Generic::keep_alives.del(this);

	if (m_initialized)
	{
		// BUGBUG: What about the visual, context and buffer - should they be deleted?

		delete m_target;
		delete m_target_front;
		XFreeGC(m_dpy, m_gc);
		XDestroyWindow(m_dpy, m_win);
	}
	
//	XFreeColormap(m_dpy, m_color_map);
	delete m_palette;
}

bool CL_XWindow_DisplayCard::keep_alive()
{
	XEvent event;
	if (XCheckTypedWindowEvent(m_dpy, m_win, DestroyNotify, &event))
	{
		// Damn, someone closed our window - better ask application to quit.
		return true;
	}

	return false;
}

void CL_XWindow_DisplayCard::flip_display(bool sync)
{
	cl_assert(m_initialized); // card not initialized!

	// New tactic - sync before putscreen
	if (sync) XSync(m_dpy, false);

	m_target->put_image(m_win, m_gc);
	CL_Target_XImage *t = m_target;
	m_target = m_target_front;
	m_target_front = t;
/*
	// Wait until the X server has finished this frame
	if (sync)
	{
		XSync(m_dpy, false);
	}*/
	XFlush(m_dpy);
}

void CL_XWindow_DisplayCard::set_palette(CL_Palette *pal)
{
	m_palette = new CL_Palette(pal->palette);
}

CL_Palette *CL_XWindow_DisplayCard::get_palette()
{
	return m_palette;
}

void CL_XWindow_DisplayCard::set_videomode(
	int width,
	int height,
	int bpp)
{
	if (m_initialized) // clean up
	{
		delete m_target; m_target = NULL;
		delete m_target_front; m_target_front = NULL;
		XFreeGC(m_dpy, m_gc);
		XDestroyWindow(m_dpy, m_win);
	}
	
	set_gfxmode(width, height, bpp);

	m_win = 0;

	/* set window attributes */
	XSetWindowAttributes attr;
//	attr.colormap = m_color_map;
//	attr.event_mask = ExposureMask | StructureNotifyMask;
	attr.border_pixel = BlackPixel(m_dpy, m_scr);
	attr.background_pixel = BlackPixel(m_dpy, m_scr);
	attr.backing_store = Always;
	int attr_flags =
//		CWColormap |
//		CWEventMask |
		CWBorderPixel |
		CWBackPixel |
		CWBackingStore;

	/* Create the window */
	m_win = XCreateWindow(
		m_dpy,
		m_root,
		DisplayWidth(m_dpy, m_scr)/2 - get_width()/2,
		DisplayHeight(m_dpy, m_scr)/2 - get_height()/2,
		get_width(),
		get_height(),
		0,
		m_visual_info.depth,
		CopyFromParent, // m_visual_info.c_class, // InputOutput
		m_visual_info.visual,
		attr_flags,
		&attr);

	cl_assert(m_win); // failed to create window!

	char *title = CL_ClanApplication::app->get_title();

	XTextProperty text_property;
	XStringListToTextProperty(&title, 1, &text_property);
	XSizeHints size_hints;
	size_hints.x = 0;
	size_hints.y = 0;
	size_hints.width = get_width();
	size_hints.height = get_height();
	size_hints.flags = PSize | PMinSize | PMaxSize;
	size_hints.min_width = get_width();
	size_hints.min_height = get_height();
	size_hints.max_width = get_width();
	size_hints.max_height = get_height();
	size_hints.flags |= USPosition | USSize;

	XSetWMProperties(
		m_dpy,
		m_win,
		&text_property,
		&text_property,
		0,
		0,
		&size_hints,
		0,
		0);

//	XMapWindow(m_dpy, m_win);
	XMapRaised(m_dpy, m_win);

/*	while (1)
	{
		XEvent event;
		XNextEvent(m_dpy, &event );
		if (event.type == MapNotify && event.xmap.window == m_win)
		{
			break;
		}
	}*/

	m_target = create_target();


	m_gc = XCreateGC(m_dpy, m_win, 0, 0);

	// Listen for keystrokes...
	XSelectInput(m_dpy, m_win, KeyPressMask|KeyReleaseMask);

	m_initialized = true;
}

CL_Target_XImage *CL_XWindow_DisplayCard::create_target()
{
	try
	{
		m_target = new CL_Target_XImage_Shm(
			m_visual_info,
			m_dpy,
			get_width(),
			get_height());

		m_target_front = new CL_Target_XImage_Shm(
			m_visual_info,
			m_dpy,
			get_width(),
			get_height());
		
		return m_target;
	}
	catch (CL_Error err)
	{
		cout << "X11 Implementation: Fallback to normal XImages." << endl;
		m_target = new CL_Target_XImage_Std(
			m_visual_info,
			m_dpy,
			get_width(),
			get_height());

		m_target_front = new CL_Target_XImage_Std(
			m_visual_info,
			m_dpy,
			get_width(),
			get_height());

		return m_target;
	}
}

bool CL_XWindow_DisplayCard::is_initialized()
{
	return m_initialized;
}

CL_List<CL_VidMode> CL_XWindow_DisplayCard::get_videomodes()
{
	cl_assert(false); // not implemented yet!
	CL_List<CL_VidMode> ret;
	return ret;
}

#endif
