/*
	$Id: display_xwindow.h,v 1.15 1999/07/05 21:08:12 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifndef header_display_xwindow
#define header_display_xwindow

#ifdef USE_X11

#include "Display/Generic/displaycard_generic.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "target_ximage_std.h"
#include "target_ximage_shm.h"
#include "target_ximage_dga.h"

#include "System/Unix/init_linux.h"

class CL_XWindow_CompatibleCard : public CL_DisplayCard_Generic
// needed by the PTC implementation to get x11 input support.
{
public:
	CL_XWindow_CompatibleCard(int card_no)
	: CL_DisplayCard_Generic(card_no)
	{
	}

	virtual Window get_window()=0;
	virtual Display *get_dpy()=0;
};

class CL_XWindow_DisplayCard
: public CL_XWindow_CompatibleCard, public CL_System_KeepAlive
{
public:
	CL_XWindow_DisplayCard(Display *dpy, Window root, int card_no);
	virtual ~CL_XWindow_DisplayCard();
	
	// CL_XWindow_CompatibleCard:
	// --------------------------

	virtual Window get_window() { return m_win; }
	virtual Display *get_dpy() { return m_dpy; }

	// CL_DisplayCard functions:
	// -------------------------

	virtual void flip_display(bool sync=false);
	virtual void set_palette(CL_Palette *palette);
	virtual CL_Palette *get_palette();
	virtual void set_videomode(int width, int height, int bpp);
	virtual bool is_initialized();

	virtual CL_String get_name() { return "X11 Display"; }
	virtual int get_total_memory() { return -1; }
	virtual CL_List<CL_VidMode> get_videomodes();

	// CL_DisplayCard_Generic functions:
	// ---------------------------------

	virtual CL_Target *get_target() { return m_target; }
	virtual CL_Target *get_frontbuffer() { return m_target_front; }
	
	// CL_System_KeepAlive functions:
	// ------------------------------

	virtual bool keep_alive();
	
protected: // inherited by mesa impl.
	virtual CL_Target_XImage *create_target();
	virtual XVisualInfo *get_visual_info() { return &m_visual_info; }

private:
	bool m_initialized;
	CL_Palette *m_palette;
	CL_Target_XImage *m_target;
	CL_Target_XImage *m_target_front;

	XVisualInfo m_visual_info;
	Display *m_dpy;
	Window m_win;
	Window m_root;
	GC m_gc;
	int m_scr;
	Colormap m_color_map;
};

#endif /* USE_X11 */

#endif
