/*
	$Id: target_ximage_std.cpp,v 1.3 1999/06/04 02:58:39 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#include "System/API/cl_assert.h"
#include "target_ximage_std.h"

#include <iostream.h>
#include <string.h>

CL_Target_XImage_Std::CL_Target_XImage_Std(
	XVisualInfo visual_info,
	Display *dpy,
	unsigned int width,
	unsigned int height)
{
	m_visual_info = visual_info;
	m_dpy = dpy;
	m_width = width;
	m_height = height;
	
	int image_size = get_pitch()*height;
	char *image_data = new char[image_size];
	
	m_std_image = XCreateImage(
		dpy,
		visual_info.visual,
		visual_info.depth, // PTC does this: DefaultDepth(dpy,0),
		ZPixmap,
		0,
		image_data,
		width,
		height,
		8, // bitmap_pad
		0);
	
	cl_assert(m_std_image != NULL);
	
	m_delete_ximage = true;
}

CL_Target_XImage_Std::CL_Target_XImage_Std(
	XVisualInfo visual_info,
	Display *dpy,
	unsigned int width,
	unsigned int height,
	XImage *image,
	bool delete_ximage)
{
	m_visual_info = visual_info;
	m_dpy = dpy;
	m_width = width;
	m_height = height;
	
	m_std_image = image;
	m_delete_ximage = delete_ximage;
}
		
CL_Target_XImage_Std::~CL_Target_XImage_Std()
{
	if (m_delete_ximage)
		XDestroyImage(m_std_image);
}

void CL_Target_XImage_Std::put_image(Drawable win, GC gc)
{
	XPutImage(
		m_dpy,
		win,
		gc,
		m_std_image,
		0, 0,
		0, 0,
		m_width,
		m_height);
}

bool CL_Target_XImage_Std::lock()
{
	return true;
}
	
void CL_Target_XImage_Std::unlock()
{
}

void *CL_Target_XImage_Std::get_data()
{
	return m_std_image->data;
}

unsigned int CL_Target_XImage_Std::get_width()
{
	return m_width;
}

unsigned int CL_Target_XImage_Std::get_height()
{
	return m_height;
}

unsigned int CL_Target_XImage_Std::get_pitch()
{
	int bytes_per_pixel = (get_depth()+7)/8;

	return m_width*bytes_per_pixel;
}
	
unsigned int CL_Target_XImage_Std::get_depth()
{
	int hack = (m_visual_info.depth+15)/16;
	return hack*16;

//	return m_visual_info.depth;

//	return BitmapUnit(m_dpy); // <-- always returns 32 bpp!!!
}

unsigned int CL_Target_XImage_Std::get_red_mask()
{
	return m_visual_info.red_mask;
}

unsigned int CL_Target_XImage_Std::get_green_mask()
{
	return m_visual_info.green_mask;
}

unsigned int CL_Target_XImage_Std::get_blue_mask()
{
	return m_visual_info.blue_mask;
}

unsigned int CL_Target_XImage_Std::get_alpha_mask()
{
//	return m_visual_info.alpha_mask; // no alpha mask in visualinfo struct!?
	return 0;
}

CL_Palette *CL_Target_XImage_Std::get_palette()
{
	return NULL;
}
