/*
	$Id: datafile_inputprovider.h,v 1.6 1999/07/03 02:25:09 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Datafile input source provider, part of the i/o data component.

*/

#ifndef header_datafile_inputprovider
#define header_datafile_inputprovider

#ifdef __BEOS__
#include <gnu/zlib.h>
#else
#include <zlib.h>
#endif

#include "Layer1/System/API/cl_assert.h"
#include "Layer1/IOData/API/inputsource.h"

class CL_Zipped_Position
{
public:
	gzFile gzfile;
	int datafile_pos;
	int seek_pos;
};

class CL_InputSource_Datafile : public CL_InputSource
{
protected:
	CL_String filename;
	CL_String datafile;
	int datafile_handle;

	int objsize;

	gzFile gzfile;
	int index_open;
	int datafile_open;

	CL_List<CL_Zipped_Position> index_stack;
	
	int seek_pos;

	void open_index();
	void close_index();

public:
	CL_InputSource_Datafile(const char *index, const char *datafile);
	CL_InputSource_Datafile(const CL_InputSource_Datafile *source);
	virtual ~CL_InputSource_Datafile();

	// Change input data endianess (defaults to little endian)
	virtual void set_system_mode();
	virtual void set_big_endian_mode();
	virtual void set_little_endian_mode();

	// read 32 bit integer:
	virtual int read_int32();
	virtual unsigned int read_uint32();

	// read 16 bit short:
	virtual short read_short16();
	virtual unsigned short read_ushort16();

	// read 8 bit char:
	virtual char read_char8();
	virtual unsigned char read_uchar8();

	virtual float read_float32();

	// read larger amounts of data (no endian and 64 bit conversion):
	// returns num bytes actually read
	virtual int read(void *data, int size);

	virtual void open();
	virtual void close();

	// Make a copy of the current InputSource, standing at the same position.
	virtual CL_InputSource *clone() const;

	// Returns current position in input source
	virtual int tell() const;
	virtual void seek(int pos, SeekEnum seek_type);

	// Returns the size of the input source
	virtual int size() const;

	virtual char *read_string();	// remember to delete string after usage!!!
	virtual CL_String read_clanstring();

	virtual void push_position();
	virtual void pop_position();
};

class CL_InputSourceProvider_Datafile : public CL_InputSourceProvider
{
protected:
	CL_String datafile;

public:
	CL_InputSourceProvider_Datafile(const char *datafile);
	virtual CL_InputSource *open_source(const char *filename);
	virtual CL_InputSourceProvider *clone();
};

#endif
