/*
	$Id: keyboard_ggi.cpp,v 1.6 1999/08/24 21:24:25 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/



#ifdef USE_GGI

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <iostream.h>
#include <unistd.h>

#include <ggi/ggi.h>

#include "keyboard_ggi.h"

#include "Display/API/display.h"
#include "System/Unix/implementation_ggi.h"
#include "Display/GGI/display_ggi.h"
#include "System/API/cl_assert.h"
#include "System/API/error.h"

#include "Layer1/System/Generic/string_asm.h"

/******************
  Keyboard
******************/

CL_GGIKeyboard::CL_GGIKeyboard(CL_GGI_DisplayCard* card)
{
	memset( &keymap, 0, sizeof(keymap) );
	
	CL_System_Generic::keep_alives.add(this);

	m_vis = card->vis;
}

CL_GGIKeyboard::~CL_GGIKeyboard()
{
	CL_System_Generic::keep_alives.del(this);
	
	int num = buttons.get_num_items();
	for (int i=0; i<num; i++) delete buttons[i];
}

bool CL_GGIKeyboard::keep_alive()
{
	ggi_event_mask	mask;
	ggi_event		event;
	struct timeval	tv = {0,0};

	mask = ggiEventPoll( m_vis, emKeyboard, &tv );
	
	while (mask)
	{
		ggiEventRead( m_vis, &event, emKeyboard );
		handle_event( &event );
		
		mask = ggiEventPoll( m_vis, emKeyboard, &tv );
	}

	return false;
}

void CL_GGIKeyboard::handle_event(gii_event* event)
{
	bool keydown;

	switch (event->any.type)
	{
		case evKeyPress:
		case evKeyRepeat:
			keydown = true;
			break;
		case evKeyRelease:
			keydown = false;
			break;
		default:
			return;
	}

	CL_Key key;
	key.id = translate(event->key.label);
	key.ascii = -1;

	if (keydown)
	{
		key.state = CL_Key::Pressed;
		buffer.add_key( key );
	}
	else
	{
		key.state = CL_Key::Released;
		buffer.add_key( key );
	}
	keymap[translate(event->key.label)] = keydown;
}

char CL_GGIKeyboard::translate(int kb_value)
{
	switch (kb_value)
	{
	case GIIK_F1:	return KEY_F1;
	case GIIK_F2:	return KEY_F2;
	case GIIK_F3:	return KEY_F3;
	case GIIK_F4:	return KEY_F4;
	case GIIK_F5:	return KEY_F5;
	case GIIK_F6:	return KEY_F6;
	case GIIK_F7:	return KEY_F7;
	case GIIK_F8:	return KEY_F8;
	case GIIK_F9:	return KEY_F9;
	case GIIK_F10:	return KEY_F10;
	case GIIK_F11:	return KEY_F11;
	case GIIK_F12:	return KEY_F12;
	
	case GIIUC_A:	return KEY_A;
	case GIIUC_B:	return KEY_B;
	case GIIUC_C:	return KEY_C;
	case GIIUC_D:	return KEY_D;
	case GIIUC_E:	return KEY_E;
	case GIIUC_F:	return KEY_F;
	case GIIUC_G:	return KEY_G;
	case GIIUC_H:	return KEY_H;
	case GIIUC_I:	return KEY_I;
	case GIIUC_J:	return KEY_J;
	case GIIUC_K:	return KEY_K;
	case GIIUC_L:	return KEY_L;
	case GIIUC_M:	return KEY_M;
	case GIIUC_N:	return KEY_N;
	case GIIUC_O:	return KEY_O;
	case GIIUC_P:	return KEY_P;
	case GIIUC_Q:	return KEY_Q;
	case GIIUC_R:	return KEY_R;
	case GIIUC_S:	return KEY_S;
	case GIIUC_T:	return KEY_T;
	case GIIUC_U:	return KEY_U;
	case GIIUC_V:	return KEY_V;
	case GIIUC_W:	return KEY_W;
	case GIIUC_X:	return KEY_X;
	case GIIUC_Y:	return KEY_Y;
	case GIIUC_Z:	return KEY_Z;
	
	case GIIUC_0:	return KEY_0;
	case GIIUC_1:	return KEY_1;
	case GIIUC_2:	return KEY_2;
	case GIIUC_3:	return KEY_3;
	case GIIUC_4:	return KEY_4;
	case GIIUC_5:	return KEY_5;
	case GIIUC_6:	return KEY_6;
	case GIIUC_7:	return KEY_7;
	case GIIUC_8:	return KEY_8;
	case GIIUC_9:	return KEY_9;
	
	case GIIUC_Escape:	return KEY_ESCAPE;
	case GIIK_Left:	return KEY_LEFT;
	case GIIK_Right:	return KEY_RIGHT;
	case GIIK_Up:		return KEY_UP;
	case GIIK_Down:	return KEY_DOWN;
	case GIIK_Enter:	return KEY_ENTER;

	case GIIK_CtrlL:	return KEY_LCTRL;
	case GIIK_CtrlR:	return KEY_RCTRL;
	case GIIK_ShiftL:	return KEY_LSHIFT;
	case GIIK_ShiftR:	return KEY_RSHIFT;
	case GIIK_AltL:	return KEY_ALT;
	case GIIK_AltR:	return KEY_ALT;
	case GIIK_AltGr:	return KEY_ALTGR;
	case GIIUC_Tab:	return KEY_TAB;
	case GIIUC_Space:	return KEY_SPACE;
	case GIIUC_BackSpace:	return KEY_BACKSPACE;
	case GIIK_Insert:	return KEY_INSERT;
	case GIIK_Delete:	return KEY_DELETE;
	case GIIK_Home:	return KEY_HOME;
	case GIIK_End:		return KEY_END;
	case GIIK_PageUp:	return KEY_PAGEUP;
	case GIIK_PageDown:	return KEY_PAGEDOWN;
	case GIIK_Caps:	return KEY_CAPSLOCK;
	case GIIK_Num:		return KEY_NUMLOCK;
	case GIIK_Scroll:	return KEY_SCRLOCK;
	case GIIK_PrintScreen:	return KEY_PRINT;
	case GIIK_Pause:	return KEY_PAUSE;
	case GIIK_PSlash:	return KEY_KP_DIV;
	case GIIK_PAsterisk:	return KEY_KP_MULT;
	case GIIK_PMinus:	return KEY_KP_MINUS;
	case GIIK_PPlus:	return KEY_KP_PLUS;
	case GIIK_PEnter:	return KEY_KP_ENTER;
	}
	
	return KEY_NONE_OF_THE_ABOVE;
}

char *CL_GGIKeyboard::get_name() const
{
	return "GGI keyboard";
}

int CL_GGIKeyboard::get_num_buttons() const
{
	return NUM_KEYS;
}

CL_InputButton *CL_GGIKeyboard::get_button(int button_num)
{
	if (buttons[button_num] != NULL) return buttons[button_num];
	
	buttons.add( new CL_InputButton_GGIKeyboard(button_num, keymap), button_num );

	return buttons[button_num];
}

int CL_GGIKeyboard::get_num_axes() const
{
	return 0;
}

CL_InputAxis *CL_GGIKeyboard::get_axis(int /*axis_num*/)
{
	return NULL;
}

int CL_GGIKeyboard::get_num_hats() const
{
	return 0;
}

CL_InputHat *CL_GGIKeyboard::get_hat(int /*hat_num*/)
{
	return NULL;
}

int CL_GGIKeyboard::get_num_buffers() const
{
	return 1;
}

CL_InputBuffer *CL_GGIKeyboard::get_buffer(int /*buffer_num*/)
{
	return &buffer;
}

int CL_GGIKeyboard::get_num_cursors() const
{
	return 0;
}

CL_InputCursor *CL_GGIKeyboard::get_cursor(int /*cursor_num*/)
{
	return NULL;
}

/***************************
  CL_InputBuffer_GGIKeyboard
***************************/

CL_InputBuffer_GGIKeyboard::CL_InputBuffer_GGIKeyboard()
{
	m_begin = 0;
	m_end = 0;
	m_buffer_size = 1024;
	m_buffer = new CL_Key[m_buffer_size];
}

CL_InputBuffer_GGIKeyboard::~CL_InputBuffer_GGIKeyboard()
{
	delete[] m_buffer;
}

CL_Key CL_InputBuffer_GGIKeyboard::peek_key() const
{
	if (m_begin == m_end)
	{
		CL_Key k;
		k.state = CL_Key::NoKey;
		return k;
	}

	return m_buffer[m_begin];
}

CL_Key CL_InputBuffer_GGIKeyboard::get_key()
{
	if (m_begin == m_end)
	{
		CL_Key k;
		k.state = CL_Key::NoKey;
		return k;
	}

	return (m_buffer[m_begin++]);
}

int CL_InputBuffer_GGIKeyboard::keys_left()
{
	return m_end - m_begin;
}

void CL_InputBuffer_GGIKeyboard::clear()
{
	m_begin = 0;
	m_end = 0;
}

void CL_InputBuffer_GGIKeyboard::add_key(CL_Key key)
{
	if (m_end == m_buffer_size) // reached end of buffer
	{
		if (m_begin == 0) return; // totally full! - discard

		// Move contents to beginning of buffer...
		asm_memmove((char *) m_buffer, (char *) (m_buffer+m_begin), (m_end-m_begin)*sizeof(CL_Key));
		m_end = m_end-m_begin;
		m_begin = 0;
	}
	m_buffer[m_end++] = key;
}

/***************************
  CL_InputButton_GGIKeyboard
***************************/

CL_InputButton_GGIKeyboard::CL_InputButton_GGIKeyboard(
	int _key, char *_keymap)
{
	key = _key;
	keymap = _keymap;
}

bool CL_InputButton_GGIKeyboard::is_pressed()
{
	return keymap[key];
}

#endif
