/*
	$Id: keyboard_ggi.h,v 1.6 1999/08/31 05:49:26 sphair Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifndef header_keyboard_ggi
#define header_keyboard_ggi


#ifdef USE_GGI

#include "Input/API/inputbutton.h"
#include "Input/API/keyboard.h"
#include "System/Unix/init_linux.h"
#include "Layer1/Display/GGI/display_ggi.h"

#include <ggi/ggi.h>

class CL_InputButton_GGIKeyboard : public CL_InputButton
{
public:
	CL_InputButton_GGIKeyboard(int key, char *keymap);
	bool is_pressed();

private:
	int key;
	char *keymap;
};

class CL_InputBuffer_GGIKeyboard : public CL_InputBuffer
{
public:
	CL_InputBuffer_GGIKeyboard();
	virtual ~CL_InputBuffer_GGIKeyboard();

	CL_Key peek_key() const;
	CL_Key get_key();
	int keys_left();
	void clear();

	void add_key(CL_Key key);
private:
	int m_begin;
	int m_end;
	CL_Key *m_buffer;
	int m_buffer_size;
};

class CL_GGIKeyboard : public CL_Keyboard, CL_System_KeepAlive
{
public:
	CL_GGIKeyboard(CL_GGI_DisplayCard* card);
	virtual ~CL_GGIKeyboard();

	virtual char *get_name() const;

	virtual int get_num_buttons() const;
	virtual CL_InputButton *get_button(int button_num);

	virtual int get_num_axes() const;
	virtual CL_InputAxis *get_axis(int axis_num);

	virtual int get_num_hats() const;
	virtual CL_InputHat *get_hat(int hat_num);

	virtual int get_num_buffers() const;
	virtual CL_InputBuffer *get_buffer(int buffer_num);

	virtual int get_num_cursors() const;
	virtual CL_InputCursor *get_cursor(int cursor_num);

	virtual bool keep_alive();

private:
	char keymap[128];
	ggi_visual_t m_vis;
	
	void handle_event(gii_event* event);
	char translate(int kb_value);
	char translate_ascii(int kb_value);
	
	CL_Array<CL_InputButton_GGIKeyboard> buttons;
	CL_InputBuffer_GGIKeyboard buffer;
};

#endif /* USE_GGI */

#endif
