/*
	$Id: keyboard_svgalib.cpp,v 1.3 1999/08/24 22:57:38 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifdef USE_SVGALIB

#include <vgakeyboard.h>
#include "keyboard_svgalib.h"

#include "Layer1/System/API/cl_assert.h"

CL_SVGAKeyboard::CL_SVGAKeyboard()
{
	int err = keyboard_init();
	cl_assert(err != -1); // Failed to init svgalib keyboard handler

	keyboard_translatekeys(TRANSLATE_CURSORKEYS);
}

CL_SVGAKeyboard::~CL_SVGAKeyboard()
{
	keyboard_close();
	
	int num = buttons.get_num_items();
	for (int i=0; i<num; i++) delete buttons[i];
}

char *CL_SVGAKeyboard::get_name()
{
	return "Standard keyboard";
}

int CL_SVGAKeyboard::get_num_buttons()
{
	return NUM_KEYS;
}

CL_InputButton *CL_SVGAKeyboard::get_button(int button_num)
{
	keyboard_update();
	if (buttons[button_num] != NULL) return buttons[button_num];

	int translate_key = -1;
	switch (button_num)
	{
	case KEY_A: translate_key = SCANCODE_A; break;
	case KEY_B: translate_key = SCANCODE_B; break;
	case KEY_C: translate_key = SCANCODE_C; break;
	case KEY_D: translate_key = SCANCODE_D; break;
	case KEY_E: translate_key = SCANCODE_E; break;
	case KEY_F: translate_key = SCANCODE_F; break;
	case KEY_G: translate_key = SCANCODE_G; break;
	case KEY_H: translate_key = SCANCODE_H; break;
	case KEY_I: translate_key = SCANCODE_I; break;
	case KEY_J: translate_key = SCANCODE_J; break;
	case KEY_K: translate_key = SCANCODE_K; break;
	case KEY_L: translate_key = SCANCODE_L; break;
	case KEY_M: translate_key = SCANCODE_M; break;
	case KEY_N: translate_key = SCANCODE_N; break;
	case KEY_O: translate_key = SCANCODE_O; break;
	case KEY_P: translate_key = SCANCODE_P; break;
	case KEY_Q: translate_key = SCANCODE_Q; break;
	case KEY_R: translate_key = SCANCODE_R; break;
	case KEY_S: translate_key = SCANCODE_S; break;
	case KEY_T: translate_key = SCANCODE_T; break;
	case KEY_U: translate_key = SCANCODE_U; break;
	case KEY_V: translate_key = SCANCODE_V; break;
	case KEY_W: translate_key = SCANCODE_W; break;
	case KEY_X: translate_key = SCANCODE_X; break;
	case KEY_Y: translate_key = SCANCODE_Y; break;
	case KEY_Z: translate_key = SCANCODE_Z; break;

	case KEY_0: translate_key = SCANCODE_0; break;
	case KEY_1: translate_key = SCANCODE_1; break;
	case KEY_2: translate_key = SCANCODE_2; break;
	case KEY_3: translate_key = SCANCODE_3; break;
	case KEY_4: translate_key = SCANCODE_4; break;
	case KEY_5: translate_key = SCANCODE_5; break;
	case KEY_6: translate_key = SCANCODE_6; break;
	case KEY_7: translate_key = SCANCODE_7; break;
	case KEY_8: translate_key = SCANCODE_8; break;
	case KEY_9: translate_key = SCANCODE_9; break;

	case KEY_F1: translate_key = SCANCODE_F1; break;
	case KEY_F2: translate_key = SCANCODE_F2; break;
	case KEY_F3: translate_key = SCANCODE_F3; break;
	case KEY_F4: translate_key = SCANCODE_F4; break;
	case KEY_F5: translate_key = SCANCODE_F5; break;
	case KEY_F6: translate_key = SCANCODE_F6; break;
	case KEY_F7: translate_key = SCANCODE_F7; break;
	case KEY_F8: translate_key = SCANCODE_F8; break;
	case KEY_F9: translate_key = SCANCODE_F9; break;
	case KEY_F10: translate_key = SCANCODE_F10; break;
	case KEY_F11: translate_key = SCANCODE_F11; break;
	case KEY_F12: translate_key = SCANCODE_F12; break;

	case KEY_ESCAPE: translate_key = SCANCODE_ESCAPE; break;
	case KEY_LEFT: translate_key = SCANCODE_CURSORLEFT; break;
	case KEY_RIGHT: translate_key = SCANCODE_CURSORRIGHT; break;
	case KEY_UP: translate_key = SCANCODE_CURSORUP; break;
	case KEY_DOWN: translate_key = SCANCODE_CURSORDOWN; break;
	case KEY_LCTRL: translate_key = SCANCODE_LEFTCONTROL; break;
	case KEY_RCTRL: translate_key = SCANCODE_RIGHTCONTROL; break;
	case KEY_LSHIFT: translate_key = SCANCODE_LEFTSHIFT; break;
	case KEY_RSHIFT: translate_key = SCANCODE_RIGHTSHIFT; break;
	case KEY_ALT: translate_key = SCANCODE_LEFTALT; break;
	case KEY_ALTGR: translate_key = SCANCODE_RIGHTALT; break;
	case KEY_TAB: translate_key = SCANCODE_TAB; break;
	case KEY_ENTER: translate_key = SCANCODE_ENTER; break;
	case KEY_SPACE: translate_key = SCANCODE_SPACE; break;
	case KEY_BACKSPACE: translate_key = SCANCODE_BACKSPACE; break;
	case KEY_INSERT: translate_key = SCANCODE_INSERT; break;
	case KEY_DELETE: translate_key = SCANCODE_REMOVE; break;
	case KEY_HOME: translate_key = SCANCODE_HOME; break;
	case KEY_END: translate_key = SCANCODE_END; break;
	case KEY_PAGEUP: translate_key = SCANCODE_PAGEUP; break;
	case KEY_PAGEDOWN: translate_key = SCANCODE_PAGEDOWN; break;
	case KEY_CAPSLOCK: translate_key = SCANCODE_CAPSLOCK; break;
	case KEY_NUMLOCK: translate_key = SCANCODE_NUMLOCK; break;
	case KEY_SCRLOCK: translate_key = SCANCODE_SCROLLLOCK; break;
	case KEY_PRINT: translate_key = SCANCODE_PRINTSCREEN; break;
	case KEY_PAUSE: translate_key = SCANCODE_BREAK; break;
	case KEY_KP_DIV: translate_key = SCANCODE_KEYPADDIVIDE; break;
	case KEY_KP_MULT: translate_key = SCANCODE_KEYPADMULTIPLY; break;
	case KEY_KP_MINUS: translate_key = SCANCODE_KEYPADMINUS; break;
	case KEY_KP_PLUS: translate_key = SCANCODE_KEYPADPLUS; break;
	case KEY_KP_ENTER: translate_key = SCANCODE_KEYPADENTER; break;
	}

	if (translate_key == -1) return NULL;

	buttons.add(
		new CL_InputButton_SVGAKeyboard(translate_key),
		button_num);

	return buttons[button_num];
}

int CL_SVGAKeyboard::get_num_axes()
{
	return 0;
}

CL_InputAxis *CL_SVGAKeyboard::get_axis(int /*axis_num*/)
{
	return NULL;
}

int CL_SVGAKeyboard::get_num_hats()
{
	return 0;
}

CL_InputHat *CL_SVGAKeyboard::get_hat(int /*hat_num*/)
{
	return NULL;
}

int CL_SVGAKeyboard::get_num_buffers()
{
	return 1;
}

CL_InputBuffer *CL_SVGAKeyboard::get_buffer(int /*buffer_num*/)
{
	return &buffer;
}

int CL_SVGAKeyboard::get_num_cursors()
{
	return 0;
}

CL_InputCursor *CL_SVGAKeyboard::get_cursor(int /*cursor_num*/)
{
	return NULL;
}

/***********************
  InputButton SVGA
***********************/

CL_InputButton_SVGAKeyboard::CL_InputButton_SVGAKeyboard(int _key)
{
	key = _key;
}

bool CL_InputButton_SVGAKeyboard::is_pressed()
{
	if (keyboard_keypressed(key)) return true;
	return false;
}

/***********************
  InputBuffer SVGA
***********************/

int CL_InputBuffer_SVGAKeyboard::peek_key()
{
	return -1;
}

int CL_InputBuffer_SVGAKeyboard::get_key()
{
	return -1;
}

int CL_InputBuffer_SVGAKeyboard::keys_left()
{
	return 0;
}

void CL_InputBuffer_SVGAKeyboard::clear()
{
}

#endif
