#include "Layer1/System/Win32/app_windows.h"
#include "Layer1/System/API/cl_assert.h"
#include "input_dx.h"
#include "input_keyboard.h"
#include "input_mouse.h"
#include "input_joystick.h"

LPDIRECTINPUT CL_Input_DX::dinput = NULL;

CL_Input_DX::CL_Input_DX()
{
	HRESULT err = DirectInputCreate(CL_WindowsSystem::hInstance, 0x0300, &dinput, NULL);
	cl_assert(err == DI_OK);

	CL_Input::keyboards.add(new CL_Keyboard_Win32);
	CL_Input::pointers.add(new CL_Mouse_Win32);

	int num_devs = joyGetNumDevs();
	for (int i=0; i<num_devs; i++)
	{
		JOYCAPS joycaps;
		MMRESULT err = joyGetDevCaps(i, &joycaps, sizeof(joycaps));
		if (err == JOYERR_NOERROR) CL_Input::joysticks.add(new CL_Joystick_Win32(i));
	}
}

CL_Input_DX::~CL_Input_DX()
{
	dinput->Release();

	int i;

	int num_keyboards = CL_Input::keyboards.get_num_items();
	for (i=0; i<num_keyboards; i++)
	{
		delete CL_Input::keyboards[i];
		CL_Input::keyboards.del(i);
	}

	int num_pointers = CL_Input::pointers.get_num_items();
	for (i=0; i<num_pointers; i++)
	{
		delete CL_Input::pointers[i];
		CL_Input::pointers.del(i);
	}

	int num_joysticks = CL_Input::joysticks.get_num_items();
	for (i=0; i<num_joysticks; i++)
	{
		delete CL_Input::joysticks[i];
		CL_Input::joysticks.del(i);
	}
}
