/*
	$Id: input_keyboard.cpp,v 1.8 1999/09/04 14:35:18 starch Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#include <windows.h>
#include "Layer1/System/API/cl_assert.h"
#include "input_keyboard.h"

/*************************
		CL_Keyboard_Win32
*************************/

CL_Keyboard_Win32::CL_Keyboard_Win32()
{
	buffer = new CL_InputBuffer_Keyboard_Win32();
	memset(keys_down, 0, 256);

	CL_WindowsSystem::add_keepalive(this);
	CL_WindowsSystem::add_listener(this);
}

CL_Keyboard_Win32::~CL_Keyboard_Win32()
{
	CL_WindowsSystem::remove_keepalive(this);
	CL_WindowsSystem::remove_listener(this);

	delete buffer;

	int num_buttons = buttons.get_num_items();
	for (int i=0; i<num_buttons; i++) delete buttons[i];
}

bool CL_Keyboard_Win32::keep_alive()
{
	GetKeyboardState(keys_down);

	return false;
}

bool CL_Keyboard_Win32::received_event(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	bool keydown = false;
	switch (uMsg)
	{
		case WM_KEYDOWN:
		case WM_SYSKEYDOWN:
			keydown = true;
			break;
		
		case WM_KEYUP:
		case WM_SYSKEYUP:
			keydown = false;
			break;

		default:
			return false;
	}

	int key = -1;
	switch ((int) wParam)
	{
	case 'A': key = KEY_A; break;
	case 'B': key = KEY_B; break;
	case 'C': key = KEY_C; break;
	case 'D': key = KEY_D; break;
	case 'E': key = KEY_E; break;
	case 'F': key = KEY_F; break;
	case 'G': key = KEY_G; break;
	case 'H': key = KEY_H; break;
	case 'I': key = KEY_I; break;
	case 'J': key = KEY_J; break;
	case 'K': key = KEY_K; break;
	case 'L': key = KEY_L; break;
	case 'M': key = KEY_M; break;
	case 'N': key = KEY_N; break;
	case 'O': key = KEY_O; break;
	case 'P': key = KEY_P; break;
	case 'Q': key = KEY_Q; break;
	case 'R': key = KEY_R; break;
	case 'S': key = KEY_S; break;
	case 'T': key = KEY_T; break;
	case 'U': key = KEY_U; break;
	case 'V': key = KEY_V; break;
	case 'W': key = KEY_W; break;
	case 'X': key = KEY_X; break;
	case 'Y': key = KEY_Y; break;
	case 'Z': key = KEY_Z; break;
	case '0': key = KEY_0; break;
	case '1': key = KEY_1; break;
	case '2': key = KEY_2; break;
	case '3': key = KEY_3; break;
	case '4': key = KEY_4; break;
	case '5': key = KEY_5; break;
	case '6': key = KEY_6; break;
	case '7': key = KEY_7; break;
	case '8': key = KEY_8; break;
	case '9': key = KEY_9; break;
	case VK_F1: key = KEY_F1; break;
	case VK_F2: key = KEY_F2; break;
	case VK_F3: key = KEY_F3; break;
	case VK_F4: key = KEY_F4; break;
	case VK_F5: key = KEY_F5; break;
	case VK_F6: key = KEY_F6; break;
	case VK_F7: key = KEY_F7; break;
	case VK_F8: key = KEY_F8; break;
	case VK_F9: key = KEY_F9; break;
	case VK_F10: key = KEY_F10; break;
	case VK_F11: key = KEY_F11; break;
	case VK_F12: key = KEY_F12; break;
	case VK_ESCAPE: key = KEY_ESCAPE; break;
	case VK_LEFT: key = KEY_LEFT; break;
	case VK_RIGHT: key = KEY_RIGHT; break;
	case VK_UP: key = KEY_UP; break;
	case VK_DOWN: key = KEY_DOWN; break;
	case VK_LCONTROL: key = KEY_LCTRL; break;
	case VK_RCONTROL: key = KEY_RCTRL; break;
	case VK_LSHIFT: key = KEY_LSHIFT; break;
	case VK_RSHIFT: key = KEY_RSHIFT; break;
	case VK_LMENU: key = KEY_ALT; break;
	case VK_RMENU: key = KEY_ALTGR; break;
	case VK_TAB: key = KEY_TAB; break;
	case VK_RETURN: key = KEY_ENTER; break;
	case VK_SPACE: key = KEY_SPACE; break;
	case VK_BACK: key = KEY_BACKSPACE; break;
	case VK_INSERT: key = KEY_INSERT; break;
	case VK_DELETE: key = KEY_DELETE; break;
	case VK_HOME: key = KEY_HOME; break;
	case VK_END: key = KEY_END; break;
	case VK_PRIOR: key = KEY_PAGEUP; break;
	case VK_NEXT: key = KEY_PAGEDOWN; break;
	case VK_CAPITAL: key = KEY_CAPSLOCK; break;
	case VK_NUMLOCK: key = KEY_NUMLOCK; break;
	case VK_SCROLL: key = KEY_SCRLOCK; break;
	case VK_PRINT: key = KEY_PRINT; break;
	case VK_PAUSE: key = KEY_PAUSE; break;
	case VK_DIVIDE: key = KEY_KP_DIV; break;
	case VK_MULTIPLY: key = KEY_KP_MULT; break;
	case VK_SEPARATOR: key = KEY_KP_MINUS; break;
	case VK_ADD: key = KEY_KP_PLUS; break;
	default:
		return true;
	}
	
	CL_Key k;
	k.id = key;
	k.ascii = -1;
	k.state = keydown ? CL_Key::Pressed : CL_Key::Released;

	buffer->add_to_buffer(k);
	return true;
}

CL_InputButton *CL_Keyboard_Win32::map_button(int button_num)
{
	int key = -1;
	switch (button_num)
	{
	case KEY_A: key = 'A'; break;
	case KEY_B: key = 'B'; break;
	case KEY_C: key = 'C'; break;
	case KEY_D: key = 'D'; break;
	case KEY_E: key = 'E'; break;
	case KEY_F: key = 'F'; break;
	case KEY_G: key = 'G'; break;
	case KEY_H: key = 'H'; break;
	case KEY_I: key = 'I'; break;
	case KEY_J: key = 'J'; break;
	case KEY_K: key = 'K'; break;
	case KEY_L: key = 'L'; break;
	case KEY_M: key = 'M'; break;
	case KEY_N: key = 'N'; break;
	case KEY_O: key = 'O'; break;
	case KEY_P: key = 'P'; break;
	case KEY_Q: key = 'Q'; break;
	case KEY_R: key = 'R'; break;
	case KEY_S: key = 'S'; break;
	case KEY_T: key = 'T'; break;
	case KEY_U: key = 'U'; break;
	case KEY_V: key = 'V'; break;
	case KEY_W: key = 'W'; break;
	case KEY_X: key = 'X'; break;
	case KEY_Y: key = 'Y'; break;
	case KEY_Z: key = 'Z'; break;
	case KEY_0: key = '0'; break;
	case KEY_1: key = '1'; break;
	case KEY_2: key = '2'; break;
	case KEY_3: key = '3'; break;
	case KEY_4: key = '4'; break;
	case KEY_5: key = '5'; break;
	case KEY_6: key = '6'; break;
	case KEY_7: key = '7'; break;
	case KEY_8: key = '8'; break;
	case KEY_9: key = '9'; break;
	case KEY_F1: key = VK_F1; break;
	case KEY_F2: key = VK_F2; break;
	case KEY_F3: key = VK_F3; break;
	case KEY_F4: key = VK_F4; break;
	case KEY_F5: key = VK_F5; break;
	case KEY_F6: key = VK_F6; break;
	case KEY_F7: key = VK_F7; break;
	case KEY_F8: key = VK_F8; break;
	case KEY_F9: key = VK_F9; break;
	case KEY_F10: key = VK_F10; break;
	case KEY_F11: key = VK_F11; break;
	case KEY_F12: key = VK_F12; break;
	case KEY_ESCAPE: key = VK_ESCAPE; break;
	case KEY_LEFT: key = VK_LEFT; break;
	case KEY_RIGHT: key = VK_RIGHT; break;
	case KEY_UP: key = VK_UP; break;
	case KEY_DOWN: key = VK_DOWN; break;
	case KEY_LCTRL: key = VK_LCONTROL; break;
	case KEY_RCTRL: key = VK_RCONTROL; break;
	case KEY_LSHIFT: key = VK_LSHIFT; break;
	case KEY_RSHIFT: key = VK_RSHIFT; break;
	case KEY_ALT: key = VK_LMENU; break;
	case KEY_ALTGR: key = VK_RMENU; break;
	case KEY_TAB: key = VK_TAB; break;
	case KEY_ENTER: key = VK_RETURN; break;
	case KEY_SPACE: key = VK_SPACE; break;
	case KEY_BACKSPACE: key = VK_BACK; break;
	case KEY_INSERT: key = VK_INSERT; break;
	case KEY_DELETE: key = VK_DELETE; break;
	case KEY_HOME: key = VK_HOME; break;
	case KEY_END: key = VK_END; break;
	case KEY_PAGEUP: key = VK_PRIOR; break;
	case KEY_PAGEDOWN: key = VK_NEXT; break;
	case KEY_CAPSLOCK: key = VK_CAPITAL; break;
	case KEY_NUMLOCK: key = VK_NUMLOCK; break;
	case KEY_SCRLOCK: key = VK_SCROLL; break;
	case KEY_PRINT: key = VK_PRINT; break;
	case KEY_PAUSE: key = VK_PAUSE; break;
	case KEY_KP_DIV: key = VK_DIVIDE; break;
	case KEY_KP_MULT: key = VK_MULTIPLY; break;
	case KEY_KP_MINUS: key = VK_SEPARATOR; break;
	case KEY_KP_PLUS: key = VK_ADD; break;
	case KEY_KP_ENTER: key = VK_RETURN; break;
	default:
		cl_assert(false);
	}

	if (key != -1)
		buttons.add(
			new CL_InputButton_Keyboard_Win32(key, keys_down),
			button_num);

	return buttons[button_num];
}

char *CL_Keyboard_Win32::get_name() const
{
	return "Standard keyboard";
}

int CL_Keyboard_Win32::get_num_buttons() const
{
	return NUM_KEYS;
}

CL_InputButton *CL_Keyboard_Win32::get_button(int button_num)
{
	if (buttons[button_num] != NULL) return buttons[button_num];

	return map_button(button_num);
}

int CL_Keyboard_Win32::get_num_axes() const
{
	return 0;
}

CL_InputAxis *CL_Keyboard_Win32::get_axis(int axis_num)
{
	return NULL;
}

int CL_Keyboard_Win32::get_num_hats() const
{
	return 0;
}

CL_InputHat *CL_Keyboard_Win32::get_hat(int hat_num)
{
	return NULL;
}

int CL_Keyboard_Win32::get_num_cursors() const
{
	return 0;
}

CL_InputCursor *CL_Keyboard_Win32::get_cursor(int cursor_num)
{
	return NULL;
}

int CL_Keyboard_Win32::get_num_buffers() const
{
	return 1;
}

CL_InputBuffer *CL_Keyboard_Win32::get_buffer(int buffer_num)
{
	return buffer;
}

/************************************
		CL_InputButton_Keyboard_Win32
************************************/

CL_InputButton_Keyboard_Win32::CL_InputButton_Keyboard_Win32(
	int _key, unsigned char *_keys_down)
{
	key = _key;
	keys_down = _keys_down;
}

CL_InputButton_Keyboard_Win32::~CL_InputButton_Keyboard_Win32()
{
}

bool CL_InputButton_Keyboard_Win32::is_pressed()
{
	if (keys_down[key]&128) return true;
	return false;
}

/************************************
		CL_InputBuffer_Keyboard_Win32
************************************/

CL_InputBuffer_Keyboard_Win32::CL_InputBuffer_Keyboard_Win32()
{
	buffer_length = 0;
	buffer_start = 0;
	buffer_end = 0;
}

CL_InputBuffer_Keyboard_Win32::~CL_InputBuffer_Keyboard_Win32()
{
}

CL_Key CL_InputBuffer_Keyboard_Win32::peek_key() const
{
	if (buffer_length==0)
	{
		CL_Key k;
		k.state = CL_Key::NoKey;
		return k;
	}

	return buffer[buffer_start];
}

CL_Key CL_InputBuffer_Keyboard_Win32::get_key()
{
	if (buffer_length==0)
	{
		CL_Key k;
		k.state = CL_Key::NoKey;
		return k;
	}
	
	CL_Key retval=buffer[buffer_start++];
	if (buffer_start==256) buffer_start=0;
	buffer_length--;

	return retval;
}

int CL_InputBuffer_Keyboard_Win32::keys_left()
{
	return buffer_length;
}

void CL_InputBuffer_Keyboard_Win32::clear()
{
	buffer_length = 0;
	buffer_start = 0;
	buffer_end = 0;
}

void CL_InputBuffer_Keyboard_Win32::add_to_buffer(CL_Key keycode)
{
	if (buffer_length==256) return;

	buffer[buffer_end++]=keycode;
	if (buffer_end==256) buffer_end=0;

	buffer_length++;
}
