/*
	$Id: input_keyboard.h,v 1.6 1999/08/24 21:24:26 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifndef header_input_keyboard
#define header_input_keyboard

#include "Layer1/Input/API/input.h"
#include "Layer1/System/Win32/app_windows.h"
#include "Layer1/Input/API/keyboard.h"

class CL_InputButton_Keyboard_Win32;
class CL_InputBuffer_Keyboard_Win32;

class CL_Keyboard_Win32
: public CL_Keyboard,
	public CL_Win32KeepAlive,
	public CL_Win32EventListener
{
protected:
	unsigned char keys_down[256];
	CL_Array<CL_InputButton_Keyboard_Win32> buttons;
	CL_InputBuffer_Keyboard_Win32 *buffer;

	CL_InputButton *map_button(int key);

public:
	CL_Keyboard_Win32();
	virtual ~CL_Keyboard_Win32();

	virtual char *get_name() const;

	virtual int get_num_buttons() const;
	virtual CL_InputButton *get_button(int button_num);

	virtual int get_num_axes() const;
	virtual CL_InputAxis *get_axis(int axis_num);

	virtual int get_num_hats() const;
	virtual CL_InputHat *get_hat(int hat_num);

	virtual int get_num_cursors() const;
	virtual CL_InputCursor *get_cursor(int cursor_num);

	virtual int get_num_buffers() const;
	virtual CL_InputBuffer *get_buffer(int buffer_num);

	CL_InputBuffer_Keyboard_Win32 *get_kb_buffer() { return buffer; }

	virtual bool keep_alive();
	virtual bool received_event(UINT uMsg, WPARAM wParam, LPARAM lParam);
};

class CL_InputButton_Keyboard_Win32 : public CL_InputButton
{
protected:
	int key;
	unsigned char *keys_down;

public:
	CL_InputButton_Keyboard_Win32(int key, unsigned char *keys_down);
	virtual ~CL_InputButton_Keyboard_Win32();

	virtual bool is_pressed();
};

class CL_InputBuffer_Keyboard_Win32 : public CL_InputBuffer
{
protected:
	CL_Key buffer[256];
	int buffer_length;
	int buffer_start;
	int buffer_end;

public:
	CL_InputBuffer_Keyboard_Win32();
	virtual ~CL_InputBuffer_Keyboard_Win32();

	virtual CL_Key peek_key() const;
	virtual CL_Key get_key();
	virtual int keys_left();
	virtual void clear();

	void add_to_buffer(CL_Key keycode);
};

#endif
