/*
	$Id: input_mouse.cpp,v 1.6 1999/08/10 18:19:23 starch Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#include "Layer1/Display/API/display.h"
#include "input_mouse.h"
#include "Layer1/System/API/cl_assert.h"

/******************************
				CL_Mouse_Win32
******************************/

CL_Mouse_Win32::CL_Mouse_Win32()
{
	cursor = new CL_InputCursor_Mouse_Win32();
	axes = new CL_InputAxis_Mouse_Win32[2];

	CL_WindowsSystem::add_keepalive(this);
	CL_WindowsSystem::add_listener(this);
}

CL_Mouse_Win32::~CL_Mouse_Win32()
{
	CL_WindowsSystem::remove_keepalive(this);
	CL_WindowsSystem::remove_listener(this);

	delete cursor;

	int num_buttons = buttons.get_num_items();
	for (int i=0; i<num_buttons; i++) delete buttons[i];
}

char *CL_Mouse_Win32::get_name() const
{
	return "Standard mouse";
}

int CL_Mouse_Win32::get_num_buttons() const
{
	return 3;
}

CL_InputButton *CL_Mouse_Win32::get_button(int button_num)
{
	if (buttons[button_num] == NULL)
		buttons.add(new CL_InputButton_Mouse_Win32(button_num), button_num);

	return buttons[button_num];
}

int CL_Mouse_Win32::get_num_axes() const
{
	return 2;
}

CL_InputAxis *CL_Mouse_Win32::get_axis(int axis_num)
{
	cl_assert(((unsigned int) axis_num)<2);
	return &axes[axis_num];
}

int CL_Mouse_Win32::get_num_hats() const
{
	return 0;
}

CL_InputHat *CL_Mouse_Win32::get_hat(int hat_num)
{
	return NULL;
}

int CL_Mouse_Win32::get_num_buffers() const
{
	return 0;
}

CL_InputBuffer *CL_Mouse_Win32::get_buffer(int buffer_num)
{
	return NULL;
}

int CL_Mouse_Win32::get_num_cursors() const
{
	return 1;
}

CL_InputCursor *CL_Mouse_Win32::get_cursor(int cursor_num)
{
	return cursor;
}

bool CL_Mouse_Win32::keep_alive()
{
	POINT cursor_pos;

	GetCursorPos(&cursor_pos);

	cursor->x = cursor_pos.x;
	cursor->y = cursor_pos.y;

	axes[0].center = float(CL_Display::get_width())/2.0f;
	axes[1].center = float(CL_Display::get_height())/2.0f;
	
	axes[0].pos = cursor_pos.x;
	axes[1].pos = cursor_pos.y;

	return false;
}


bool CL_Mouse_Win32::received_event(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_LBUTTONDOWN:
	case WM_LBUTTONUP:
	case WM_MBUTTONDOWN:
	case WM_MBUTTONUP:
	case WM_RBUTTONDOWN:
	case WM_RBUTTONUP:
		break;

	default:
		return false;
	}

	int button_states = wParam;

	int num_buttons = buttons.get_num_items();
	for (int i=0; i<num_buttons; i++)
	{
		if (buttons[i] == NULL) continue;

		buttons[i]->button_state = (button_states&(1<<i)) ? true : false;
	}
	
	return true;
}

/*******************************
	 CL_InputButton_Mouse_Win32
*******************************/

CL_InputButton_Mouse_Win32::CL_InputButton_Mouse_Win32(int _key)
{
	key = (1<<_key);
	button_state = false;
}

CL_InputButton_Mouse_Win32::~CL_InputButton_Mouse_Win32()
{
}

bool CL_InputButton_Mouse_Win32::is_pressed()
{
	return button_state;
}

/*******************************
	 CL_InputCursor_Mouse_Win32
*******************************/

CL_InputCursor_Mouse_Win32::CL_InputCursor_Mouse_Win32()
{
	x = 0;
	y = 0;
}

CL_InputCursor_Mouse_Win32::~CL_InputCursor_Mouse_Win32()
{
}

float CL_InputCursor_Mouse_Win32::get_x()
{
	return x;
}

float CL_InputCursor_Mouse_Win32::get_y()
{
	return y;
}

float CL_InputCursor_Mouse_Win32::get_max_x()
{
	return CL_Display::get_width();
}

float CL_InputCursor_Mouse_Win32::get_max_y()
{
	return CL_Display::get_height();
}

/*******************************
	 CL_InputAxis_Mouse_Win32
*******************************/

CL_InputAxis_Mouse_Win32::CL_InputAxis_Mouse_Win32()
{
	pos = 0;
	center = 1;
}

CL_InputAxis_Mouse_Win32::~CL_InputAxis_Mouse_Win32()
{
}

float CL_InputAxis_Mouse_Win32::get_pos()
{
	return (pos - center) / center;
}