/*
	$Id: keyboard_x11.cpp,v 1.16 1999/08/30 21:30:36 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifdef USE_X11

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <iostream.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>

#include "keyboard_x11.h"

#include "Display/API/display.h"
#include "System/Unix/implementation_xwindow.h"
#include "Display/X11/display_xwindow.h"
#include "System/API/cl_assert.h"

#include "Layer1/System/Generic/string_asm.h"

/******************
  Keyboard
******************/

CL_XWindowKeyboard::CL_XWindowKeyboard(
	CL_XWindow_CompatibleCard *_card)
{
	card = _card;
	
	for (int j=0; j<X_KEYBOARD_BUFFERSIZE; j++) keys_return[j]=0;

	CL_System_Generic::keep_alives.add(this);
}

CL_XWindowKeyboard::~CL_XWindowKeyboard()
{
	CL_System_Generic::keep_alives.del(this);
	CL_Implementation_XWindow::clean_up();

	int num = buttons.get_num_items();
	for (int i=0; i<num; i++)
		delete buttons[i];
}

bool CL_XWindowKeyboard::keep_alive()
{
	XEvent e;
	while (XCheckMaskEvent(card->get_dpy(), KeyPressMask |KeyReleaseMask, &e))
	{
		KeySym sym = XLookupKeysym(&e.xkey,0);
		
		CL_Key key;
		key.id = map_keysym_to_id(sym);
		key.ascii = -1; // todo: map to ascii here.
		
		// Figure out the ascii value from symkey:
		char buf[11];
		buf[10]=0;
		XLookupString(&e.xkey, buf, 10, NULL, NULL);
		if (strlen(buf) == 1) key.ascii = buf[0];
		
		if (e.type == KeyRelease)
		{
			key.state = CL_Key::Released;
		}
		else
		{
			key.state = CL_Key::Pressed;
		}

		buffer.add_key(key);
	}

	// Check if window has focus right now:
	Window focus_win;
	int focus_state;
	XGetInputFocus(
		card->get_dpy(),
		&focus_win,
		&focus_state);
		
	// get_window() returns NULL if in DGA mode.
	if (card->get_window() == 0 ||
		card->get_window() == focus_win)
	{
		XQueryKeymap(card->get_dpy(), keys_return);
	}
	else
	{
		// no keys are down when not in focus:
		for (int j=0; j<X_KEYBOARD_BUFFERSIZE; j++) keys_return[j]=0;
	}

	return false;
}

char *CL_XWindowKeyboard::get_name() const
{
	return "XWindow keyboard";
}

int CL_XWindowKeyboard::get_num_buttons() const
{
	return NUM_KEYS;
}

CL_InputButton *CL_XWindowKeyboard::get_button(int button_num)
{
	if (buttons[button_num] != NULL) return buttons[button_num];

	int translate_key = map_id_to_keysym(button_num);
	if (translate_key == -1) return NULL;

	buttons.add(
		new CL_InputButton_XKeyboard(translate_key, keys_return),
		button_num);

	return buttons[button_num];
}

int CL_XWindowKeyboard::get_num_axes() const
{
	return 0;
}

CL_InputAxis *CL_XWindowKeyboard::get_axis(int /*axis_num*/)
{
	return NULL;
}

int CL_XWindowKeyboard::get_num_hats() const
{
	return 0;
}

CL_InputHat *CL_XWindowKeyboard::get_hat(int /*hat_num*/)
{
	return NULL;
}

int CL_XWindowKeyboard::get_num_buffers() const
{
	return 1;
}

CL_InputBuffer *CL_XWindowKeyboard::get_buffer(int /*buffer_num*/)
{
	return &buffer;
}

int CL_XWindowKeyboard::get_num_cursors() const
{
	return 0;
}

CL_InputCursor *CL_XWindowKeyboard::get_cursor(int /*cursor_num*/)
{
	return NULL;
}

int CL_XWindowKeyboard::map_keysym_to_id(int keysym)
{
	switch (keysym)
	{
	case XK_BackSpace: return CL_Keyboard::KEY_BACKSPACE;
	case XK_Tab: return CL_Keyboard::KEY_TAB;
//	case XK_Clear: return CL_Keyboard::KEY_CLEAR;
	case XK_Return: return CL_Keyboard::KEY_ENTER;
	case XK_Pause: return CL_Keyboard::KEY_PAUSE;
//	case XK_Scroll_Lock: return CL_Keyboard::KEY_SCROLL;
	case XK_Escape: return CL_Keyboard::KEY_ESCAPE;
	case XK_Delete: return CL_Keyboard::KEY_DELETE;
//	case XK_Kanji: return CL_Keyboard::KEY_KANJI;
	case XK_Home: return CL_Keyboard::KEY_HOME;
	case XK_Left: return CL_Keyboard::KEY_LEFT;
	case XK_Up: return CL_Keyboard::KEY_UP;
	case XK_Right: return CL_Keyboard::KEY_RIGHT;
	case XK_Down: return CL_Keyboard::KEY_DOWN;
	case XK_Page_Up: return CL_Keyboard::KEY_PAGEUP;
	case XK_Page_Down: return CL_Keyboard::KEY_PAGEDOWN;
	case XK_End: return CL_Keyboard::KEY_END;
	case XK_Print: return CL_Keyboard::KEY_PRINT;
	case XK_Insert: return CL_Keyboard::KEY_INSERT;
	case XK_Num_Lock: return CL_Keyboard::KEY_NUMLOCK;
//	case XK_KP_0: return CL_Keyboard::KEY_KP_0;
//	case XK_KP_1: return CL_Keyboard::KEY_KP_1;
//	case XK_KP_2: return CL_Keyboard::KEY_KP_2;
//	case XK_KP_3: return CL_Keyboard::KEY_KP_3;
//	case XK_KP_4: return CL_Keyboard::KEY_KP_4;
//	case XK_KP_5: return CL_Keyboard::KEY_KP_5;
//	case XK_KP_6: return CL_Keyboard::KEY_KP_6;
//	case XK_KP_7: return CL_Keyboard::KEY_KP_7;
//	case XK_KP_8: return CL_Keyboard::KEY_KP_8;
//	case XK_KP_9: return CL_Keyboard::KEY_KP_9;
	case XK_F1: return CL_Keyboard::KEY_F1;
	case XK_F2: return CL_Keyboard::KEY_F2;
	case XK_F3: return CL_Keyboard::KEY_F3;
	case XK_F4: return CL_Keyboard::KEY_F4;
	case XK_F5: return CL_Keyboard::KEY_F5;
	case XK_F6: return CL_Keyboard::KEY_F6;
	case XK_F7: return CL_Keyboard::KEY_F7;
	case XK_F8: return CL_Keyboard::KEY_F8;
	case XK_F9: return CL_Keyboard::KEY_F9;
	case XK_F10: return CL_Keyboard::KEY_F10;
	case XK_F11: return CL_Keyboard::KEY_F11;
	case XK_F12: return CL_Keyboard::KEY_F12;
	case XK_Shift_L: return CL_Keyboard::KEY_LSHIFT;
	case XK_Shift_R: return CL_Keyboard::KEY_RSHIFT;
	case XK_Control_L: return CL_Keyboard::KEY_LCTRL;
	case XK_Control_R: return CL_Keyboard::KEY_RCTRL;
	case XK_Caps_Lock: return CL_Keyboard::KEY_CAPSLOCK;
	case XK_Meta_L: return CL_Keyboard::KEY_ALT;
	case XK_Meta_R: return CL_Keyboard::KEY_ALTGR;
	case XK_Alt_L: return CL_Keyboard::KEY_ALT;
	case XK_Alt_R: return CL_Keyboard::KEY_ALTGR;

	case XK_space: return CL_Keyboard::KEY_SPACE;
//	case XK_comma: return CL_Keyboard::KEY_COMMA;
//	case XK_minus: return CL_Keyboard::KEY_MINUS;
//	case XK_period: return CL_Keyboard::KEY_PERIOD;
//	case XK_slash: return CL_Keyboard::KEY_SLASH;
	case XK_0: return CL_Keyboard::KEY_0;
	case XK_1: return CL_Keyboard::KEY_1;
	case XK_2: return CL_Keyboard::KEY_2;
	case XK_3: return CL_Keyboard::KEY_3;
	case XK_4: return CL_Keyboard::KEY_4;
	case XK_5: return CL_Keyboard::KEY_5;
	case XK_6: return CL_Keyboard::KEY_6;
	case XK_7: return CL_Keyboard::KEY_7;
	case XK_8: return CL_Keyboard::KEY_8;
	case XK_9: return CL_Keyboard::KEY_9;
//	case XK_semicolon: return CL_Keyboard::KEY_PUNKTUM;
//	case XK_equal: return CL_Keyboard::KEY_EQUAL;
//	case XK_bracketleft: return CL_Keyboard::KEY_BRACKETLEFT;
//	case XK_backslash: return CL_Keyboard::KEY_BACKSLASH;
//	case XK_bracketright: return CL_Keyboard::KEY_BRACKETRIGHT;
	case XK_a: return CL_Keyboard::KEY_A;
	case XK_b: return CL_Keyboard::KEY_B;
	case XK_c: return CL_Keyboard::KEY_C;
	case XK_d: return CL_Keyboard::KEY_D;
	case XK_e: return CL_Keyboard::KEY_E;
	case XK_f: return CL_Keyboard::KEY_F;
	case XK_g: return CL_Keyboard::KEY_G;
	case XK_h: return CL_Keyboard::KEY_H;
	case XK_i: return CL_Keyboard::KEY_I;
	case XK_j: return CL_Keyboard::KEY_J;
	case XK_k: return CL_Keyboard::KEY_K;
	case XK_l: return CL_Keyboard::KEY_L;
	case XK_m: return CL_Keyboard::KEY_M;
	case XK_n: return CL_Keyboard::KEY_N;
	case XK_o: return CL_Keyboard::KEY_O;
	case XK_p: return CL_Keyboard::KEY_P;
	case XK_q: return CL_Keyboard::KEY_Q;
	case XK_r: return CL_Keyboard::KEY_R;
	case XK_s: return CL_Keyboard::KEY_S;
	case XK_t: return CL_Keyboard::KEY_T;
	case XK_u: return CL_Keyboard::KEY_U;
	case XK_v: return CL_Keyboard::KEY_V;
	case XK_w: return CL_Keyboard::KEY_W;
	case XK_x: return CL_Keyboard::KEY_X;
	case XK_y: return CL_Keyboard::KEY_Y;
	case XK_z: return CL_Keyboard::KEY_Z;
	}

	return CL_Keyboard::KEY_NONE_OF_THE_ABOVE;
}

int CL_XWindowKeyboard::map_id_to_keysym(int id)
{
	switch (id)
	{
	case KEY_A: return 38;
	case KEY_B: return 56;
	case KEY_C: return 54;
	case KEY_D: return 40;
	case KEY_E: return 26;
	case KEY_F: return 41;
	case KEY_G: return 42;
	case KEY_H: return 43;
	case KEY_I: return 31;
	case KEY_J: return 44;
	case KEY_K: return 45;
	case KEY_L: return 46;
	case KEY_M: return 58;
	case KEY_N: return 57;
	case KEY_O: return 32;
	case KEY_P: return 33;
	case KEY_Q: return 24;
	case KEY_R: return 27;
	case KEY_S: return 39;
	case KEY_T: return 28;
	case KEY_U: return 30;
	case KEY_V: return 55;
	case KEY_W: return 25;
	case KEY_X: return 53;
	case KEY_Y: return 29;
	case KEY_Z: return 52;

	case KEY_0: return 19;
	case KEY_1: return 10;
	case KEY_2: return 11;
	case KEY_3: return 12;
	case KEY_4: return 13;
	case KEY_5: return 14;
	case KEY_6: return 15;
	case KEY_7: return 16;
	case KEY_8: return 17;
	case KEY_9: return 18;

	case KEY_F1: return 67;
	case KEY_F2: return 68;
	case KEY_F3: return 69;
	case KEY_F4: return 70;
	case KEY_F5: return 71;
	case KEY_F6: return 72;
	case KEY_F7: return 73;
	case KEY_F8: return 74;
	case KEY_F9: return 75;
	case KEY_F10: return 76;
	case KEY_F11: return 95;
	case KEY_F12: return 96;

	case KEY_ESCAPE: return 9; 
	case KEY_LEFT: return 100;
	case KEY_RIGHT: return 102;
	case KEY_UP: return 98;
	case KEY_DOWN: return 104;
	case KEY_LCTRL: return 37;
	case KEY_RCTRL: return 109;
	case KEY_LSHIFT: return 50;
	case KEY_RSHIFT: return 62;
	case KEY_ALT: return 64;
	case KEY_ALTGR: return 113;
	case KEY_TAB: return 23;
	case KEY_ENTER: return 36;
	case KEY_SPACE: return 65;
	case KEY_BACKSPACE: return 22;
	case KEY_INSERT: return 106;
	case KEY_DELETE: return 107;
	case KEY_HOME: return 97;
	case KEY_END: return 98;
	case KEY_PAGEUP: return 99;
	case KEY_PAGEDOWN: return 105;
	case KEY_CAPSLOCK: return 66;
	case KEY_NUMLOCK: return 77;
	case KEY_SCRLOCK: return 78;
	case KEY_PRINT: return 111;
	case KEY_PAUSE: return 110;
	case KEY_KP_DIV: return 112;
	case KEY_KP_MULT: return 63;
	case KEY_KP_MINUS: return 82;
	case KEY_KP_PLUS: return 86;
	case KEY_KP_ENTER: return 108;
	}
	
	return -1;
}

/***************************
  CL_InputBuffer_XKeyboard
***************************/

CL_InputBuffer_XKeyboard::CL_InputBuffer_XKeyboard()
{
	m_begin = 0;
	m_end = 0;
	m_buffer_size = 1024;
	m_buffer = new CL_Key[m_buffer_size];
}

CL_InputBuffer_XKeyboard::~CL_InputBuffer_XKeyboard()
{
	delete[] m_buffer;
}

CL_Key CL_InputBuffer_XKeyboard::peek_key() const
{
	if (m_begin == m_end)
	{
		CL_Key key;
		key.state = CL_Key::NoKey;
		return key;
	}

	return m_buffer[m_begin];
}

CL_Key CL_InputBuffer_XKeyboard::get_key()
{
	if (m_begin == m_end)
	{
		CL_Key key;
		key.state = CL_Key::NoKey;
		return key;
	}

	return m_buffer[m_begin++];
}

int CL_InputBuffer_XKeyboard::keys_left()
{
	return m_end - m_begin;
}

void CL_InputBuffer_XKeyboard::clear()
{
	m_begin = 0;
	m_end = 0;
}

void CL_InputBuffer_XKeyboard::add_key(CL_Key key)
{
	if (m_end == m_buffer_size) // reached end of buffer
	{
		if (m_begin == 0) return; // totally full! - discard

		// Move contents to beginning of buffer...
		asm_memmove(
			(char *) m_buffer,
			(char *) (m_buffer+m_begin),
			(m_end-m_begin)*sizeof(CL_Key));

		m_end = m_end-m_begin;
		m_begin = 0;
	}
	m_buffer[m_end++] = key;
}

/***************************
  CL_InputButton_XKeyboard
***************************/

CL_InputButton_XKeyboard::CL_InputButton_XKeyboard(
	int _key, char *_keys_return)
{
	key = _key;
	keys_return = _keys_return;
}

bool CL_InputButton_XKeyboard::is_pressed()
{
	int byte = key/8;
	int bit = key%8;

	return (keys_return[byte] & (1<<bit)) ? true : false;
}

#endif
