/*
	$Id: keyboard_x11.h,v 1.10 1999/08/29 23:21:34 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifndef header_keyboard_x11
#define header_keyboard_x11

#ifdef USE_X11

#include "Input/API/inputbutton.h"
#include "Input/API/keyboard.h"
#include "System/Unix/init_linux.h"

// Don't change this; hardcoded in X
#define X_KEYBOARD_BUFFERSIZE 32

class CL_InputButton_XKeyboard : public CL_InputButton
{
public:
	CL_InputButton_XKeyboard(int key, char *keys_return);
	virtual bool is_pressed();

private:
	int key;
	char *keys_return;
};

class CL_InputBuffer_XKeyboard : public CL_InputBuffer
{
public:
	CL_InputBuffer_XKeyboard();
	virtual ~CL_InputBuffer_XKeyboard();

	virtual CL_Key peek_key() const;
	virtual CL_Key get_key();
	virtual int keys_left();
	virtual void clear();

	void add_key(CL_Key key);
private:
	int m_begin;
	int m_end;
	CL_Key *m_buffer;
	int m_buffer_size;
};

class CL_XWindow_CompatibleCard;
class CL_XWindowKeyboard : public CL_Keyboard, CL_System_KeepAlive
{
public:
	CL_XWindowKeyboard(CL_XWindow_CompatibleCard *card);
	virtual ~CL_XWindowKeyboard();

	virtual char *get_name() const;

	virtual int get_num_buttons() const;
	virtual CL_InputButton *get_button(int button_num);

	virtual int get_num_axes() const;
	virtual CL_InputAxis *get_axis(int axis_num);

	virtual int get_num_hats() const;
	virtual CL_InputHat *get_hat(int hat_num);

	virtual int get_num_buffers() const;
	virtual CL_InputBuffer *get_buffer(int buffer_num);

	virtual int get_num_cursors() const;
	virtual CL_InputCursor *get_cursor(int cursor_num);

	virtual bool keep_alive();

private:
	static int map_keysym_to_id(int keysym);
	static int map_id_to_keysym(int id);

	char keys_return[X_KEYBOARD_BUFFERSIZE];
	
	CL_Array<CL_InputButton_XKeyboard> buttons;
	CL_InputBuffer_XKeyboard buffer;
	CL_XWindow_CompatibleCard *card;
};

#endif /* USE_X11 */

#endif
