/*
	$Id: mouse_x11.h,v 1.5 1999/06/22 14:08:57 vogel Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifndef header_mouse_x11
#define header_mouse_x11

#ifdef USE_X11

#include "Input/API/inputdevice.h"
#include "Input/API/inputbutton.h"
#include "Input/API/inputcursor.h"
#include "Display/X11/display_xwindow.h"
#include "System/Unix/init_linux.h"

class CL_InputButton_Mouse_XWin : public CL_InputButton
{
public:
	bool button_state;

	CL_InputButton_Mouse_XWin();
	virtual ~CL_InputButton_Mouse_XWin();

	virtual bool is_pressed();
};

class CL_InputCursor_Mouse_XWin : public CL_InputCursor
{
friend class CL_Mouse_XWin;

protected:
	CL_XWindow_CompatibleCard *card;
	float x, y;

public:

	CL_InputCursor_Mouse_XWin(CL_XWindow_CompatibleCard *card);
	virtual ~CL_InputCursor_Mouse_XWin();

	virtual float get_x();
	virtual float get_y();
	virtual float get_max_x();
	virtual float get_max_y();
};

class CL_InputAxis_Mouse_XWin : public CL_InputAxis
{
friend class CL_Mouse_XWin;

protected:
	float pos, center;

public:
	CL_InputAxis_Mouse_XWin();
	virtual ~CL_InputAxis_Mouse_XWin();

	virtual float get_pos();
};

class CL_Mouse_XWin : public CL_InputDevice, CL_System_KeepAlive
{
protected:
	CL_XWindow_CompatibleCard *card;
	CL_Array<CL_InputButton_Mouse_XWin> buttons;
	CL_InputCursor_Mouse_XWin *cursor;
	CL_InputAxis_Mouse_XWin *axes;

public:
	CL_Mouse_XWin(CL_XWindow_CompatibleCard *card);
	virtual ~CL_Mouse_XWin();

	virtual char *get_name() const { return "XMouse"; };

	virtual int get_num_buttons() const;
	virtual CL_InputButton *get_button(int button_num);

	virtual int get_num_axes() const;
	virtual CL_InputAxis *get_axis(int axis_num);

	virtual int get_num_hats() const;
	virtual CL_InputHat *get_hat(int hat_num);

	virtual int get_num_buffers() const;
	virtual CL_InputBuffer *get_buffer(int buffer_num);

	virtual int get_num_cursors() const;
	virtual CL_InputCursor *get_cursor(int cursor_num);

	virtual bool keep_alive();
};

#endif /* USE_X11 */

#endif
