
#ifdef USE_NETWORK

#include "network_generic.h"
#include "netgame_client.h"
#include "netgame_server.h"

#ifndef WIN32
	#include <sys/socket.h>
	#include <netinet/in.h>
	#include <arpa/inet.h>
	#include <netdb.h>
#endif

CL_Network_Generic *CL_Network_Generic::self = NULL;

CL_Network_Generic::CL_Network_Generic(CL_ConnectionProvider *provider)
{
	this->provider = provider;
	self = this;
}

CL_Network_Generic::~CL_Network_Generic()
{
	delete provider;
	self = NULL;
}

void CL_Network_Generic::find_games_broadcast(
	CL_String game_id,
	int port)
{
	throw CL_Error("Not implemented yet!");
}

void CL_Network_Generic::find_game_at(
		CL_String game_id,
		const char *host_address,
		int port)
{
	// lookup dns name, if needed.
	unsigned int addr = 0;
	if (host_address[0] >= '0' && host_address[0] <= '9') // a.b.c.d
	{
		addr = inet_addr(host_address);
	}
	else
	{
		hostent *host = gethostbyname(host_address);
		if (host == NULL) throw CL_Error("Could not lookup DNS name");

		addr = *((unsigned int*) host->h_addr_list[0]);
	}

	queue.put(
		new CL_NetGame_Client(
			addr,
			port,
			game_id,
			this));
}

bool CL_Network_Generic::peek_game_found()
{
	return !queue.is_empty();
}

CL_NetGame *CL_Network_Generic::receive_game_found(int timeout_millis)
{
	if (queue.is_empty()) throw CL_Error("Game not found");
	return queue.get();
}

void CL_Network_Generic::clear_games_found()
{
	while (queue.is_empty() == false) delete queue.get();
}

CL_NetGame *CL_Network_Generic::create_game(CL_String game_id, int port)
{
	return new CL_NetGame_Server(
		this,
		game_id,
		port);
}

void CL_Network_Generic::keep_alive()
{
/*
	CL_Iterator<CL_NetGame_Generic> it(netgames);
	while (it.next() != NULL)
	{
		it()->keep_alive();
	}
*/
}

/*
	CL_Network API static function calls:
	-------------------------------------
*/

void CL_Network::find_games_broadcast(
	CL_String game_id,
	int port)
{
	if (CL_Network_Generic::self == NULL)
		throw CL_Error("Network not initialized!");

	CL_Network_Generic::self->find_games_broadcast(
		game_id,
		port);
}
	
void CL_Network::find_game_at(
	CL_String game_id,
	const char *host_address,
	int port)
{
	if (CL_Network_Generic::self == NULL)
		throw CL_Error("Network not initialized!");

	CL_Network_Generic::self->find_game_at(
		game_id,
		host_address,
		port);
}
	
bool CL_Network::peek_game_found()
{
	if (CL_Network_Generic::self == NULL)
		throw CL_Error("Network not initialized!");

	return CL_Network_Generic::self->peek_game_found();
}

CL_NetGame *CL_Network::receive_game_found(int timeout)
{
	if (CL_Network_Generic::self == NULL)
		throw CL_Error("Network not initialized!");

	return CL_Network_Generic::self->receive_game_found(timeout);
}

void CL_Network::clear_games_found()
{
	if (CL_Network_Generic::self == NULL)
		throw CL_Error("Network not initialized!");

	CL_Network_Generic::self->clear_games_found();
}

CL_NetGame *CL_Network::create_game(
	CL_String game_id,
	int port)
{
	if (CL_Network_Generic::self == NULL)
		throw CL_Error("Network not initialized!");

	return CL_Network_Generic::self->create_game(
		game_id,
		port);
}

#endif
