/*
	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Network delivery implementation - Unix sockets version.
*/

#ifdef USE_NETWORK

#ifndef header_network_delivery_winsocket
#define header_network_delivery_winsocket

#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

#include "Layer1/Network/Generic/network_delivery_impl.h"
#include "Common/queue.h"

class CL_Connections_Unix;
class CL_UDPConnection;

class CL_UnixSocket : public CL_Connection
{
protected:
	int sock;
	bool is_connection_lost;

	char *cur_message;
	int cur_message_size;

	unsigned long read_int();
	void read_data(void *dest, int size_data);
	void write_int(unsigned long value);
	inline void write_data(void *data, unsigned int size);

	// read from socket:
	enum
	{
		expect_magic,
		expect_packet_size,
		expect_packet_data,
		packet_finished
	} recv_state;

	bool read_avail();
	bool require_avail(int size);
	void get_avail(void *buf, int size);

	CL_Connections_Unix *provider;
/*extremely*/ private:
	CL_List<char> avail_buffers;
	int avail_buffer_pos;
	int avail_buffer_end;

public:
	virtual int get_socket() { return sock; }

	CL_UnixSocket(CL_Connections_Unix *provider);
	virtual ~CL_UnixSocket();

	// CL_UnixSocket defined virtual function
	virtual bool try_connect(unsigned long remote_ip_network_format, int port);
	virtual bool init_socket(int init_socket=-1);

	// CL_Connection inherited functions
	virtual bool peek();
	virtual CL_ConnectionPacket receive();
	virtual void send(CL_ConnectionPacket message);
	virtual bool connection_lost();
};

class CL_UnixAcceptSocket : public CL_UnixSocket
{
	bool is_inited;
	int port;
public:
	CL_UnixAcceptSocket(CL_Connections_Unix *provider);
	virtual ~CL_UnixAcceptSocket();

	bool bind(int port);

	CL_UnixSocket *accept();
	int get_port() { return port; }

	// CL_UnixSocket inherited functions
	virtual bool try_connect(unsigned long remote_ip_network_format, int port);

	// CL_Connection inherited functions
	virtual bool peek();
	virtual CL_ConnectionPacket receive();
	virtual void send(CL_ConnectionPacket message);
	virtual bool connection_lost();
};


class CL_UDPConnection_Unix : public CL_UDPConnection
{
protected:
	int sock;
	int port;

public:
	virtual int get_socket() { return sock; }

	CL_UDPConnection_Unix();
	virtual ~CL_UDPConnection_Unix();

	bool bind(unsigned int port);

	// CL_UDPConnection inherited functions
	virtual bool peek();
	virtual CL_UDPConnectionPacket receive();
	virtual void send(CL_UDPConnectionPacket message);
	virtual void broadcast(CL_UDPConnectionPacket message);
	virtual unsigned int get_port();
};


class CL_UnixPipeConnection : public CL_Connection
{
	int read, write;
	bool is_connection_lost;

	unsigned long read_int();
	void read_data(void *dest, int size_data);
	void write_int(unsigned long value);
	inline void write_data(void *data, unsigned int size);

public:
	CL_UnixPipeConnection(int read, int write);
	virtual ~CL_UnixPipeConnection();

	int get_socket() { return read; }

	// CL_Connection inherited functions
	virtual bool peek();
	virtual CL_ConnectionPacket receive();
	virtual void send(CL_ConnectionPacket message);
	virtual bool connection_lost();
};


#endif

#endif // USE_NETWORK
