/*
	$Id: network_delivery_unix.cpp,v 1.11 1999/08/20 17:25:44 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifdef USE_NETWORK

#include "Layer1/System/API/cl_assert.h" 
#include "Layer1/System/API/error.h" 
#include "network_delivery_socket.h"
#include "network_delivery_unix.h"

/*********************************************************************
				CL_Connections_Unix Implementation
*********************************************************************/

CL_Connections_Unix::CL_Connections_Unix(CL_UnixPipeConnection *_client_connection)
{
	client_connection = _client_connection;
}

CL_Connections_Unix::~CL_Connections_Unix()
{
}

CL_UDPConnection *CL_Connections_Unix::create_udp_connection(unsigned int port)
{
	CL_UDPConnection_Unix *ret = new CL_UDPConnection_Unix;
	bool res = ret->bind(port);

	if (res) 
	{
		udp_connections.add(ret);
		return ret;
	}
	else
	{
		delete ret;
		return NULL;
	}
}

CL_Connection *CL_Connections_Unix::get_client()
{
	return client_connection;
}

CL_Connection *CL_Connections_Unix::create_tcp_connection(
	int ip_addr,
	int port)
{
	CL_UnixSocket *socket = new CL_UnixSocket(this);
	socket->init_socket();
	bool res = socket->try_connect(ip_addr, port);

	if (res) 
	{
		connections.add(socket);
		return socket;
	}
	else
	{
		delete socket;
		return NULL;
	}
}

CL_Connection *CL_Connections_Unix::accept()
{
	CL_Iterator<CL_UnixAcceptSocket> counter(accepting_connections);
	while (counter.next() != NULL)
	{
		CL_UnixSocket *ret = counter()->accept();
		if (ret != NULL) 
		{
			connections.add(ret);
			return ret;
		}
	}
	return NULL;
}

void CL_Connections_Unix::start_accept_on_port(int port)
{
	CL_UnixAcceptSocket *new_accept_socket = new CL_UnixAcceptSocket(this);
	bool res = new_accept_socket->bind(port);

	// Argh - we need some robustness bad!
	if (res == false) throw CL_Error("Port already in use an other application!");

	accepting_connections.add(new_accept_socket);
}

void CL_Connections_Unix::stop_accept_on_port(int port)
{
	CL_Iterator<CL_UnixAcceptSocket> counter(accepting_connections);
	while (counter.next() != NULL)
	{
		if (counter()->get_port() == port)
		{
			delete counter();
			counter.remove();
			return;
		}
	}
}

void CL_Connections_Unix::wait_for_connection_data(CL_Mutex *mutex)
{
	mutex->enter();

	fd_set fdset;
	FD_ZERO(&fdset);

	int highest_fd = -1;
	if (client_connection != NULL)
	{
		highest_fd = client_connection->get_socket();
		FD_SET(client_connection->get_socket(), &fdset);
	}

	CL_Iterator<CL_UnixSocket> counter(connections);
	while (counter.next() != NULL)
	{
		if (counter()->get_socket() > highest_fd)
			highest_fd = counter()->get_socket();

		FD_SET(counter()->get_socket(), &fdset);
	}

	CL_Iterator<CL_UDPConnection_Unix> counter2(udp_connections);
	while (counter2.next() != NULL)
	{
		if (counter2()->get_socket() > highest_fd)
			highest_fd = counter2()->get_socket();

		FD_SET(counter2()->get_socket(), &fdset);
	}

	CL_Iterator<CL_UnixAcceptSocket> counter3(accepting_connections);
	while (counter3.next() != NULL)
	{
		if (counter3()->get_socket() > highest_fd)
			highest_fd = counter3()->get_socket();

		FD_SET(counter3()->get_socket(), &fdset);
	}
	
	mutex->leave();

	/*int ret =*/ ::select(highest_fd+1, &fdset, NULL, NULL, NULL);

// Don't assert if select failed. It _can_ happen because the game thread may
// close a socket. This is possible because we select _outside_ the mutex.

//	cl_assert(ret != -1);
}

void CL_Connections_Unix::remove_connection(CL_Connection *removed_connection)
{
	CL_Iterator<CL_UnixSocket> counter(connections);
	while (counter.next() != NULL)
	{
		if (counter() == removed_connection) 
		{
			counter.remove();
			return;
		}
	}
}

#endif // USE_NETWORK
