/*
	$Id: network_delivery_win32.cpp,v 1.10 1999/08/24 17:27:16 starch Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	$Log: network_delivery_win32.cpp,v $
	Revision 1.10  1999/08/24 17:27:16  starch
	Modified network impl under windows.
	
	Revision 1.8  1999/08/14 22:48:24  mbn
	Multithreaded network code.
	
	Revision 1.7  1999/01/03 06:26:39  starch
	We would really really like to tell what we have done with this code - but then
	again, we would have to shoot you! :-)
	Anyways, we made the network code WORK! Now that's a new cool concept!
	
	Revision 1.3  1998/12/25 01:36:11  starch
	Made lots of changes. Fixed a gazillion include errors, rewrote the win32 part,
	plus various hacks.


	------------------------------------------------------------------------
*/

#ifdef USE_NETWORK

#include "Layer1/System/API/cl_assert.h" 
#include "network_delivery_winsocket.h"
#include "network_delivery_win32.h"

/*********************************************************************
				CL_Connections_Win32 Implementation
*********************************************************************/

CL_Connections_Win32::CL_Connections_Win32()
{
}

CL_Connections_Win32::~CL_Connections_Win32()
{
}

CL_UDPConnection *CL_Connections_Win32::create_udp_connection(unsigned int port)
{
	CL_UDPConnection_Win32 *ret = new CL_UDPConnection_Win32;
	bool res = ret->bind(port);

	if (res) 
	{
		udp_connections.add(ret);
		return ret;
	}
	else
	{
		delete ret;
		return NULL;
	}
}

CL_Connection *CL_Connections_Win32::create_tcp_connection(
	int ip_addr,
	int port)
{
	CL_Win32Socket *socket = new CL_Win32Socket;
	socket->init_socket();
	bool res = socket->try_connect(ip_addr, port);

	if (res) 
	{
		connections.add(socket);
		return socket;
	}
	else
	{
		delete socket;
		return NULL;
	}
}

CL_Connection *CL_Connections_Win32::accept()
{
	CL_Iterator<CL_Win32AcceptSocket> counter(accepting_connections);
	while (counter.next() != NULL)
	{
		CL_Win32Socket *ret = counter()->accept();
		if (ret != NULL) 
		{
			connections.add(ret);
			return ret;
		}
	}
	return NULL;
}

void CL_Connections_Win32::start_accept_on_port(int port)
{
	CL_Win32AcceptSocket *new_accept_socket = new CL_Win32AcceptSocket;
	bool res = new_accept_socket->bind(port);

	// Argh - we need some robustness bad!
	cl_assert(res);

	accepting_connections.add(new_accept_socket);
}

void CL_Connections_Win32::stop_accept_on_port(int port)
{
	CL_Iterator<CL_Win32AcceptSocket> counter(accepting_connections);
	while (counter.next() != NULL)
	{
		if (counter()->get_port() == port)
		{
			delete counter();
			counter.remove();
			return;
		}
	}
}

void CL_Connections_Win32::wait_for_connection_data(CL_Mutex *mutex)
{
	fd_set fdset;
	FD_ZERO(&fdset);

	CL_Iterator<CL_Win32Socket> counter(connections);
	while (counter.next() != NULL)
	{
		FD_SET(counter()->get_socket(), &fdset);
	}

	CL_Iterator<CL_UDPConnection_Win32> counter2(udp_connections);
	while (counter2.next() != NULL)
	{
		FD_SET(counter2()->get_socket(), &fdset);
	}

	CL_Iterator<CL_Win32AcceptSocket> counter3(accepting_connections);
	while (counter3.next() != NULL)
	{
		FD_SET(counter3()->get_socket(), &fdset);
	}

	int ret = ::select(0, &fdset, NULL, NULL, NULL);
	cl_assert(ret != SOCKET_ERROR);
}

void CL_Connections_Win32::remove_connection(CL_Connection *removed_connection)
{
	CL_Iterator<CL_Win32Socket> counter(connections);
	while (counter.next() != NULL)
	{
		if (counter() == removed_connection) 
		{
			counter.remove();
			return;
		}
	}
}

#endif // USE_NETWORK