/*
	$Id: array.h,v 1.1 1998/12/21 23:24:50 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Expandable array class template.


	$Log: array.h,v $
	Revision 1.1  1998/12/21 23:24:50  mbn
	Added ClanSound library to ClanLib repository.
	
	Revision 1.1  1998/10/26 13:46:48  mbn
	Added session management.

	Revision 1.4  1998/10/14 23:13:32  starch
	Fixed off-by-one error in get_item()

	Revision 1.3  1998/09/17 12:05:20  mbn
	Converted all spaces to tabs by using unexpand.

	Revision 1.2	1998/02/17 22:08:48	 mbn
	Changed class prefix to 'CL'.

	Revision 1.1.1.1	1998/02/07 16:51:17	 mbn
	Finally uploaded ClanLib-0.1.3 to CVS.

*/

#ifndef header_array
#define header_array

template<class TYPE>
class CL_Array
{
private:
	TYPE **items;
	int max_items;
	int highest_available;
	int lowest_available;
	
public:
	CL_Array()
	{
		highest_available = 0;
		lowest_available = 0;
		max_items = 50;
		items = new TYPE*[max_items];
		
		for (int i=0; i<max_items; i++) items[i] = NULL;
	}
	
	CL_Array(int start_entries)
	{
		highest_available = 0;
		lowest_available = 0;
		max_items = start_entries;
		items = new TYPE*[max_items];
		
		for (int i=0; i<max_items; i++) items[i] = NULL;
	}
	
	~CL_Array()
	{
		delete[] items;
	}
	
	void empty_array()
	{
		for (int i=0; i<max_items; i++) items[i] = NULL;
		lowest_available = 0;
		highest_available = 0;
	}
	
	int add(TYPE *item)
	{
		int retval;

		if (lowest_available == max_items) expand_array();
		items[lowest_available] = item;
		retval = lowest_available;

		find_lowest_available();

		return retval;
	}
	
	int add(TYPE *item, int position)
	{
		if (position >= max_items) expand_array(position+50);
		items[position] = item;
		
		if (position == lowest_available) find_lowest_available();

		return position;
	}
	
	bool del(int number)
	{
		if (number < 0) return false;
		if (number >= max_items) return false;

		if (items[number] == NULL) return false;
		items[number] = NULL;
		
		if (number < lowest_available) lowest_available = number;

		while (highest_available > lowest_available)
		{
			if (items[highest_available] != NULL) break;
			highest_available--;
		}
		
		return true;
	}
	
	bool del(TYPE *item)
	{
		return del(get_num(item));
	}
	
	int get_num(TYPE *find_item)
	{
		for (int i=0; i<highest_available; i++)
		{
			if (items[i] == find_item) return i;
		}
		
		return -1;
	}
	
	TYPE *get_item(int number)
	{
		if (number < 0) return NULL;
		if (number >= max_items) return NULL;

		return items[number];
	}
	
	TYPE *operator[] (int number)
	{
		return get_item(number);
	}

	int get_num_items()
	{
		return highest_available;
	}
	
	int get_array_size()
	{
		return max_items;
	}
	
private:
	void expand_array(int index_size = -1)
	{
		if (index_size == -1) index_size = max_items*2;

		TYPE **old_array = items;
		items = new TYPE*[index_size];
		
		for (int i=0; i<max_items; i++)
		{
			items[i] = old_array[i];
		}
		
		for (int j=max_items; j<index_size; j++)
		{
			items[j] = NULL;
		}
		
		max_items = index_size;
	}
	
	void find_lowest_available()
	{
		while (lowest_available < max_items)
		{
			if (items[lowest_available] == NULL) break;
			lowest_available++;
		}

		if (lowest_available > highest_available) highest_available = lowest_available;
	}
};

#endif

