/*
	$Id: pipeconnection.cpp,v 1.2 1999/06/14 17:43:00 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifdef USE_CLANSOUND

#include "soundserver.h"
#include "pipeconnection.h"
#include <string.h>
#include <errno.h>
#include <sys/time.h>

CS_PipeConnection::CS_PipeConnection(int _read, int _write)
{
	read = _read;
	write = _write;
	is_connection_lost = false;
	
	queue_size = 0;
	queue_data = NULL;
}

CS_PipeConnection::~CS_PipeConnection()
{
	close(read);
	close(write);
	
	delete[] queue_data;
}

bool CS_PipeConnection::peek()
{
	fd_set rfds;
	FD_ZERO(&rfds);
	FD_SET(read, &rfds);

	timeval timeout;
	memset(&timeout, 0, sizeof(timeval));

	int retval = select(read+1, &rfds, NULL, NULL, &timeout);
	return retval > 0;
}

CS_ConnectionPacket CS_PipeConnection::receive()
{
	unsigned long size_data = read_int();

	unsigned char *data = new unsigned char[size_data];
	read_data(data, size_data);

	CS_ConnectionPacket ret;
	ret.size = size_data;
	ret.data = data;

	return ret;
}

void CS_PipeConnection::send(CS_ConnectionPacket message)
{
	write_int(message.size);
	write_data(message.data, message.size);
}

bool CS_PipeConnection::connection_lost()
{
	return is_connection_lost;
}

unsigned long CS_PipeConnection::read_int()
{
	unsigned long ret;
	read_data(&ret, sizeof(unsigned long));

	return ret;
}

void CS_PipeConnection::read_data(void *dest, int size_data)
{
	int total_read = 0;
	while (total_read < size_data)
	{
		int res = ::read(
			read, 
			((unsigned char *) dest)+total_read,
			size_data-total_read);

		if (res == 0)
		{
			cout << "ClanSound server: Lost connection to client" << endl;
			is_connection_lost = true;
			return;
		}
			
		total_read += res;
	}
}

void CS_PipeConnection::write_int(unsigned long value)
{
	write_data(&value, sizeof(unsigned long));
}

inline void CS_PipeConnection::write_data(void *data, unsigned int size)
{
	// there is a bug in this code... :-(

	int written = 0;
	if (queue_data != NULL) written = ::write(write, queue_data, queue_size);

	if (written == queue_size)
	{
		delete[] queue_data; queue_data = NULL; queue_size = 0;

		written = ::write(write, data, size);
		if (written < (int) size)
		{
			unsigned char *_data = (unsigned char *) data;

			queue_data = new unsigned char[size-written];
			memcpy(queue_data, _data+written, size-written);
			queue_size = size-written;
		}
	}
	else
	{
		int left = queue_size-written;
		unsigned char *temp = new unsigned char[left+size];

		if (queue_data != NULL)
			memcpy(temp, queue_data+written, left);

		memcpy(temp+left, data, size);
		
		delete[] queue_data;
		queue_data = temp;
		queue_size = left+size;
	}
}

#endif // USE_CLANSOUND
