/*
	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Network delivery implementation - Windows sockets version.
*/

#ifdef USE_CLANSOUND

#ifndef header_pipeconnection
#define header_pipeconnection

#include <unistd.h>
#include <errno.h>
#include <string.h>

#include "Common/queue.h"
#include "Common/formatted_data.h"

class CS_ConnectionPacket
//: A packet sent through a CS_Connection.
//!desc: The basic packet type sent 
{
public:
	int size;
//: Size of the packet data.

	void *data;
//: The packet data.

	CS_ConnectionPacket() { data = NULL; }
//: Default constructor

	CS_ConnectionPacket(CL_DataWriter *writer)
	{
		data = writer->get_data();
		size = writer->get_size();
	}
//: Constructor creating a packet from a datawriter
};

class CS_PipeConnection
{
	int read, write;
	bool is_connection_lost;

	unsigned long read_int();
	void read_data(void *dest, int size_data);
	void write_int(unsigned long value);
	inline void write_data(void *data, unsigned int size);
	
	int queue_size;
	unsigned char *queue_data;

public:
	CS_PipeConnection(int read, int write);
	virtual ~CS_PipeConnection();

	int get_socket() { return read; }

	virtual bool peek();
	virtual CS_ConnectionPacket receive();
	virtual void send(CS_ConnectionPacket message);
	virtual bool connection_lost();
};

#endif

#endif // USE_CLANSOUND
