/*
	ClanSound, sound mixing library written for ClanLib.
	Copyright (c)1998 Magnus Norddahl / ClanSoft.
*/

#ifndef header_session
#define header_session

#include "sample.h"

class CS_Session
{
public:
	virtual void play()=0;
	virtual void stop()=0;
	virtual void flush()=0;
	virtual void set_volume(int vol)=0;
	virtual void set_pan(int pan)=0;
	virtual void set_frequency(int freq)=0;
	virtual void set_position(int position)=0;
	
	virtual bool is_playing()=0;
	virtual void get_playback_data(int *data, int num_samples)=0;
	virtual void mix_to(int *data, int num_samples)=0;
	
	// functions only used by streamed sessions:
	virtual void eof() { ; }
	virtual int ask_update() { return 0; }
	virtual int update(void * /*data*/, int /*data_size*/) { return 0; }
};

class CS_Session_Sample : public CS_Session
{
protected:
	CS_Sample *sample;
	int vol, pan, freq;
	double pos;
	bool playing;

public:
	CS_Session_Sample(CS_Sample *sample);
	virtual ~CS_Session_Sample();

	virtual void play();
	virtual void stop();
	virtual void flush() {}; //FIXME: anything for flush on static buffers? -- dok
	virtual void set_volume(int vol);
	virtual void set_pan(int pan);
	virtual void set_frequency(int freq);
	virtual void set_position(int pos);

	virtual bool is_playing();
	virtual void get_playback_data(int *data, int num_samples);
	virtual void mix_to(int *data, int num_samples);
};

class CS_Session_Stream : public CS_Session
{
protected:
	int format;
	int vol, pan, freq;
	double pos;
	bool playing;
	int filled_pos;
	bool first_time;
	
	short *ring_buffer;
	int buffer_size;
	bool stream_eof;

	void update_format(int pos, int len, void *data);
	int get_samples_available();

public:
	CS_Session_Stream(int format);
	virtual ~CS_Session_Stream();

	virtual void play();
	virtual void stop();
	virtual void flush();
	virtual void set_volume(int vol);
	virtual void set_pan(int pan);
	virtual void set_frequency(int freq);
	virtual void set_position(int position) {}; //FIXME: implement it -- dok

	virtual bool is_playing();
	virtual void get_playback_data(int *data, int num_samples);
	virtual void mix_to(int *data, int num_samples);

	virtual void eof();
	virtual int ask_update();
	virtual int update(void *data, int data_size);
};

#endif
