/*
	ClanSound, sound mixing library written for ClanLib.
	Copyright (c)1998 Magnus Norddahl / ClanSoft.
*/

#ifndef header_soundserver
#define header_soundserver

#define cs_assert(a) if ((a) == false) { cout << "ClanSound assert in " << __FILE__ << ":" << __LINE__ << ", pid: " << getpid() << endl; *((char*) NULL)=0; }

#include "array.h"
#include "clansound.h"
#include "session.h"
#include "pipeconnection.h"

// Sound commands:
// ----------------------------

enum CS_COMMANDS
{
	GOODBYE,

	// static:
	CREATE_SAMPLE,
	RELEASE_SAMPLE,
	CREATE_SESSION,
	RELEASE_SESSION,

	// shared:
	PLAY_SESSION,
	STOP_SESSION,
	FLUSH_SESSION,
	SET_VOLUME,
	SET_PAN,
	SET_FREQUENCY,
	SET_POSITION,
	IS_PLAYING,

	// streamed:
	CREATE_STREAM,
	ASK_UPDATE_STREAM,
	UPDATE_STREAM,
	EOF_STREAM
};

bool cs_server_init();
void cs_server_main(int client_read, int client_write);
int cs_server_read_command(CS_PipeConnection *pipe);

int cs_server_create_sample(int format, void *data, int data_size);
void cs_server_release_sample(int sample_id);

int cs_server_create_session(int sample_id);
int cs_server_create_session_streamed(int format);
void cs_server_play(int session_id);
void cs_server_stop(int session_id);
void cs_server_flush(int session_id);
void cs_server_set_volume(int session_id, int volume); // volume in db
void cs_server_set_pan(int session_id, int pan); // -100 to 100
void cs_server_set_frequency(int session_id, int frequency);
void cs_server_set_position(int session_id, int position); // position in samples
void cs_server_play(int session_id);
void cs_server_stop(int session_id);
bool cs_server_is_playing(int session_id);
int cs_server_release_session(int session_id);
int cs_server_update_stream(int session_id, void *data, int data_size);
int cs_server_ask_update_stream(int session_id);
void cs_server_eof_stream(int session_id);

#ifdef file_soundserver
	CL_Array<CS_Sample> samples;
	CL_Array<CS_Session> sessions;
#else
	extern CL_Array<CS_Sample> samples;
	extern CL_Array<CS_Session> sessions;
#endif

#endif
