/*
	$Id: cardplayback_clan.cpp,v 1.6 1999/09/04 18:26:44 fischlustig Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifdef USE_CLANSOUND

#include <math.h>

#include "cardplayback_clan.h"

CL_CardPlayback_ClanSound::CL_CardPlayback_ClanSound(int sample_id)
{
	if (sample_id != -1) session_id = cs_create_session(sample_id);
	
	volume = 1.0f;
	pan = 0.0f;
	frequency = -1;
}

CL_CardPlayback_ClanSound::~CL_CardPlayback_ClanSound()
{
	cs_release_session(session_id);
}

// Functions inherited from Generic:
// ---------------------------------

int CL_CardPlayback_ClanSound::get_position()
{
	cl_info(info_sound, "get_position not implemented");
	return -1;
}

float CL_CardPlayback_ClanSound::get_position_relative()
{
	cl_info(info_sound, "get_position_relative not implemented");
	return 0;
}

bool CL_CardPlayback_ClanSound::set_position(int new_pos)
{
	cs_set_position(session_id, new_pos);
	return false;
}

bool CL_CardPlayback_ClanSound::set_position_relative(float new_pos)
{
	cl_info(info_sound, "set_position_relative not implemented");
	return false;
}

int CL_CardPlayback_ClanSound::get_length()
{
	cl_info(info_sound, "get_length not implemented");
	return 0;
}

int CL_CardPlayback_ClanSound::get_frequency()
{
	return frequency;
}

bool CL_CardPlayback_ClanSound::set_frequency(int new_freq)
{
	frequency = new_freq;
	cs_set_frequency(session_id, new_freq);
	return true;
}

float CL_CardPlayback_ClanSound::get_volume()
{
	return volume;
}

bool CL_CardPlayback_ClanSound::set_volume(float new_volume)
{
	volume = new_volume;
	cs_set_volume(session_id, (int) (new_volume*128));
	return true;
}

float CL_CardPlayback_ClanSound::get_pan()
{
	return pan;
}

bool CL_CardPlayback_ClanSound::set_pan(float new_pan)
{
	pan = new_pan;

	int pal_int = (int) (new_pan*128);
	if (pal_int < -128) pal_int = -128;
	if (pal_int > 128) pal_int = 128;

	cs_set_pan(session_id, pal_int);
	return 0;
}

void CL_CardPlayback_ClanSound::set_looping(bool looping)
{
	static bool warning = true;
	if (warning)
	{
		cl_info(info_sound, "Not implemented.");
		warning = false;
	}
}

bool CL_CardPlayback_ClanSound::get_looping()
{
	return false;
}

void CL_CardPlayback_ClanSound::play()
{
	cs_play(session_id);
}

void CL_CardPlayback_ClanSound::stop()
{
	cs_stop(session_id);
}

bool CL_CardPlayback_ClanSound::is_playing()
{
	return cs_is_playing(session_id);
}

#endif
