/*
	$Id: soundbuffer_stream_clan.cpp,v 1.4 1999/05/31 12:18:54 fischlustig Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Sound card impl. that uses the ClanSound library.

*/

#ifdef USE_CLANSOUND

#include "soundbuffer_stream_clan.h"
#include "Layer1/System/API/cl_assert.h"

CL_CardPlayback_ClanSound_Stream::CL_CardPlayback_ClanSound_Stream(CL_StreamSoundProvider *_provider)
: CL_CardPlayback_ClanSound(-1)
{
	provider = _provider;
	provider_session = provider->begin_session();

	int format;
	
	switch (provider_session->get_format())
	{
	case sf_8bit_signed:
		format = CS_8BIT_MONO;
		break;
		
	case sf_8bit_signed_stereo:
		format = CS_8BIT_STEREO;
		break;
		
	case sf_16bit_signed:
		format = CS_16BIT_MONO;
		break;
		
	case sf_16bit_signed_stereo:
		format = CS_16BIT_STEREO;
		break;
		
	default:
		format = -1;
		cl_assert(false);
	}
	
	session_id = cs_create_session_streamed(format);
	cs_set_frequency(session_id, provider_session->get_frequency());
	
	CL_System_Generic::keep_alives.add(this);
}

CL_CardPlayback_ClanSound_Stream::~CL_CardPlayback_ClanSound_Stream()
{
	CL_System_Generic::keep_alives.del(this);

	provider->end_session(provider_session);
}

bool CL_CardPlayback_ClanSound_Stream::keep_alive()
{
	int bytes_wanted = cs_ask_update_stream(session_id);
	if (bytes_wanted == 0) return false;

	char *copy_buffer = new char[bytes_wanted];
	int bytes_given = provider_session->get_data(copy_buffer, bytes_wanted);

//	cout << "request data: " << bytes_wanted <<
//		", give data: " << bytes_given << endl;

	if (bytes_given > 0) cs_update_stream(session_id, copy_buffer, bytes_given);
	delete[] copy_buffer;
	
	if (provider_session->eof()) cs_eof_stream(session_id);
	
	return false;
}

bool CL_CardPlayback_ClanSound_Stream::set_position(int new_pos)
{
	if (provider_session->set_position(new_pos))
	{
		cs_flush(session_id);
		return true;
	}
	
	return false;
}

void CL_CardPlayback_ClanSound_Stream::stop()
{
//	provider_session->stop();
	cs_stop(session_id);
}

void CL_CardPlayback_ClanSound_Stream::play()
{
//	provider_session->play();
	cs_play(session_id);
}

#endif
