/*
	$Id: cardsession_manager.cpp,v 1.1.1.1 1998/12/20 23:33:13 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#include "cardsession_manager.h"

CL_CardSessionManager::CL_CardSessionManager()
{
}

CL_CardSessionManager::~CL_CardSessionManager()
{
	int num = playbacks.get_num_items();
	for (int i=0; i<num; i++) delete playbacks[i];
}

int CL_CardSessionManager::add_playback(
	CL_CardSoundBuffer_Playback *playback_buffer)
{
	garbage_collect();

	return playbacks.add(new SPlayBackStruct(playback_buffer));
}

void CL_CardSessionManager::add_reference(int handle)
{
	playbacks[handle]->reference_count++;
}

void CL_CardSessionManager::remove_reference(int handle)
{
	cl_assert(playbacks[handle] != NULL);

	playbacks[handle]->reference_count--;
	if (playbacks[handle]->reference_count == 0 && !playbacks[handle]->playback->is_playing())
	{
		playbacks.del(handle);
		delete playbacks[handle];
	}
}

void CL_CardSessionManager::garbage_collect()
{
	for (int i=0;i<playbacks.get_num_items();i++)
	{
		if (playbacks[i] != NULL &&
			playbacks[i]->reference_count == 0 &&
			!playbacks[i]->playback->is_playing())
		{
			playbacks.del(i);
			delete playbacks[i];
		}
	}
}

CL_CardSessionManager::SPlayBackStruct::SPlayBackStruct(
	CL_CardSoundBuffer_Playback *_playback)
{
	playback = _playback;
	reference_count = 1;
}

CL_CardSessionManager::SPlayBackStruct::~SPlayBackStruct()
{ 
	delete playback; 
}
