/*
	$Id: soundbuffer_static.cpp,v 1.5 1999/08/24 17:25:40 starch Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#include "Layer1/Sound/API/static_soundprovider.h"
#include "Layer1/Sound/API/sound.h"
#include "Layer1/Sound/Generic/soundcard_generic.h"
#include "Layer1/Sound/Generic/cardsession_manager.h"
#include "Layer1/System/API/cl_assert.h"

#include "soundbuffer_static.h"

CL_SoundBuffer *CL_SoundBuffer::create(
	CL_StaticSoundProvider *provider, bool delete_provider)
{
	return new CL_SoundBuffer_Generic_Static(provider, delete_provider);
}

CL_SoundBuffer_Generic_Static::CL_SoundBuffer_Generic_Static(
	CL_StaticSoundProvider *_provider, bool _delete_provider)
{
	provider = _provider;
	delete_provider = _delete_provider;

	volume = -1; 
	pan = -1; 
	frequency = -1; 
}

CL_SoundBuffer_Generic_Static::~CL_SoundBuffer_Generic_Static()
{
	int num = card_buffers.get_num_items();
	for (int i=0; i<num; i++) delete card_buffers[i];

	if (delete_provider) delete provider;
}

CL_StaticSoundProvider *CL_SoundBuffer_Generic_Static::get_static_provider() const
{
	return provider;
}

int CL_SoundBuffer_Generic_Static::get_length() const
{
	return provider->data_size();
}

int CL_SoundBuffer_Generic_Static::get_num_samples() const 
{
	switch (provider->get_format())
	{
	case sf_8bit_signed:
		return provider->data_size();
	case sf_8bit_signed_stereo:
	case sf_16bit_signed:
		return provider->data_size()/2;

	case sf_16bit_signed_stereo:
		return provider->data_size()/4;
	}
	
	cl_assert(false); // invalid sample format.

	return -1;
}

int CL_SoundBuffer_Generic_Static::get_frequency() const
{
	return frequency;
}

bool CL_SoundBuffer_Generic_Static::set_frequency(int new_freq)
{
	frequency = new_freq;
	return true;
}

float CL_SoundBuffer_Generic_Static::get_volume() const
{
	return volume;
}

bool CL_SoundBuffer_Generic_Static::set_volume(float new_volume)
{
	volume = new_volume;
	return true;
}

float CL_SoundBuffer_Generic_Static::get_pan() const
{
	return pan;
}

bool CL_SoundBuffer_Generic_Static::set_pan(float new_pan)
{
	pan = new_pan;
	return true;
}

bool CL_SoundBuffer_Generic_Static::is_playing(CL_SoundBuffer_Session **session, CL_SoundCard *card) const
{
	throw "CL_SoundBuffer_Generic_Static::is_playing() not implemented";
	return true;
}

void CL_SoundBuffer_Generic_Static::stop(CL_SoundCard *card)
{
	throw "CL_SoundBuffer_Generic_Static::stop() not implemented";
}

CL_SoundBuffer_Session CL_SoundBuffer_Generic_Static::play(bool looping, CL_SoundCard *card)
{
	if (card == NULL) card = CL_Sound::get_current_card();

	CL_SoundBuffer_Session ret = prepare(looping, card);
	ret.play();

	return ret;
}

CL_SoundBuffer_Session CL_SoundBuffer_Generic_Static::prepare(bool looping, CL_SoundCard *card)
{
	if (card == NULL) card = CL_Sound::get_current_card();

	if (card_buffers[card->card_no] == NULL)
	{
		provider->lock();

		volume = 1;
		pan = 0; 
		frequency = provider->get_frequency(); 

		card_buffers.add(
			((CL_SoundCard_Generic *) card)->create_soundbuffer_static(provider),
			card->card_no);

		provider->unlock();
	}

	CL_SoundBuffer_Session ret(
		CL_SoundBuffer_Session(
			((CL_SoundCard_Generic *) card)->manager->add_playback(
				card_buffers[card->card_no]->prepare()),
			card));

	ret.set_volume(volume);
	ret.set_frequency(frequency);
	ret.set_pan(pan);

	return ret;
}
