/*
	$Id: soundbuffer_stream.cpp,v 1.5 1999/08/24 17:25:40 starch Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#include "Layer1/Sound/API/stream_soundprovider.h"
#include "Layer1/Sound/API/sound.h"
#include "soundbuffer_stream.h"
#include "soundcard_generic.h"
#include "cardsession_manager.h"

CL_SoundBuffer *CL_SoundBuffer::create(
	CL_StreamSoundProvider *provider, bool delete_provider)
{
	return new CL_SoundBuffer_Generic_Stream(provider, delete_provider);
}

CL_SoundBuffer_Generic_Stream::CL_SoundBuffer_Generic_Stream(
	CL_StreamSoundProvider *_provider, bool _delete_provider)
{
	provider = _provider;
	delete_provider = _delete_provider;

	volume = -1; 
	pan = -1; 
	frequency = -1; 
}

CL_SoundBuffer_Generic_Stream::~CL_SoundBuffer_Generic_Stream()
{
	if (delete_provider) delete provider;
}

CL_StreamSoundProvider *CL_SoundBuffer_Generic_Stream::get_stream_provider() const
{
	return provider;
}

int CL_SoundBuffer_Generic_Stream::get_length() const
{
	return -1;
}

int CL_SoundBuffer_Generic_Stream::get_num_samples() const
{
	return -1;
}

int CL_SoundBuffer_Generic_Stream::get_frequency() const
{
	return frequency;
}

bool CL_SoundBuffer_Generic_Stream::set_frequency(int new_freq)
{
	frequency = new_freq;
	return true;
}

float CL_SoundBuffer_Generic_Stream::get_volume() const
{
	return volume;
}

bool CL_SoundBuffer_Generic_Stream::set_volume(float new_volume)
{
	volume = new_volume;
	return true;
}

float CL_SoundBuffer_Generic_Stream::get_pan() const
{
	return pan;
}

bool CL_SoundBuffer_Generic_Stream::set_pan(float new_pan)
{
	pan = new_pan;
	return true;
}

bool CL_SoundBuffer_Generic_Stream::is_playing(CL_SoundBuffer_Session **session, CL_SoundCard *card) const
{
	throw "CL_SoundBuffer_Generic_Stream::is_playing() not implemented";
	return true;
}

void CL_SoundBuffer_Generic_Stream::stop(CL_SoundCard *card)
{
	throw "CL_SoundBuffer_Generic_Stream::stop() not implemented";
}

CL_SoundBuffer_Session CL_SoundBuffer_Generic_Stream::play(bool looping, CL_SoundCard *card)
{
	if (card == NULL) card = CL_Sound::get_current_card();

	CL_SoundBuffer_Session ret(prepare(looping, card));
	ret.play();

	return ret;
}

CL_SoundBuffer_Session CL_SoundBuffer_Generic_Stream::prepare(bool looping, CL_SoundCard *card)
{
	if (card == NULL) card = CL_Sound::get_current_card();

	CL_CardSoundBuffer_Playback *playback =
		((CL_SoundCard_Generic *) card)->
			create_cardsoundbuffer_playback_streamed(this);

	CL_SoundBuffer_Session ret(
		CL_SoundBuffer_Session(
			((CL_SoundCard_Generic *) card)->manager->add_playback(playback), card));

	if (pan == -1) pan = 0;
	if (volume == -1) volume = 1;

	ret.set_volume(volume);
	ret.set_pan(pan);
	ret.set_looping(looping);
	if (frequency != -1) ret.set_frequency(frequency);

	return ret;
}
