/*
	$Id: implementation_ggi.cpp,v 1.10 1999/08/17 18:44:22 fischlustig Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Initialization functions for GGI implementation.

*/

#ifdef USE_GGI

#include "implementation_ggi.h"
#include "Display/API/display.h"
#include "Display/GGI/display_ggi.h"
#include "Input/API/input.h"
#include "Input/GGI/mouse_ggi.h"
#include "Input/GGI/keyboard_ggi.h"
#include "Input/X11/joystick_linux.h"
#include "implementation.h"

bool CL_Implementation_GGI::initialized = false;
int CL_Implementation_GGI::display_counter=0;

extern "C"
{
        DL_PREFIX char* clan_module_identify() {
	  return "GGI target";
	}

        DL_PREFIX char* clan_module_abbreviation() {
	  return "ggi";
	}

	DL_PREFIX void clan_module_init()
	{
		CL_Implementation_GGI::add_display();
	}

}

char *ggi_identify() {
  return clan_module_identify();
}

char *ggi_abbreviation() {
  return clan_module_abbreviation();
}

void ggi_init() {
  clan_module_init();
}


bool CL_Implementation_GGI::init()
{
	if (initialized) return true;

	return true;
}

void CL_Implementation_GGI::clean_up()
{
	if (display_counter == 0) return; // nothing to clean up...

	display_counter--;
	if (display_counter>0) return;
}

void CL_Implementation_GGI::add_display()
{
	if (init() == false)
	{
		cout << "ClanLib: Failed to open display. No GGI display available." << endl;
	}

	CL_GGI_DisplayCard *card = new CL_GGI_DisplayCard(display_counter);

	CL_Display::cards.add(card, display_counter);

	CL_Input::keyboards.add(new CL_GGIKeyboard(card));
	CL_Input::pointers.add(new CL_Mouse_GGI(card));

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)

        for (int i=0; i<8; i++)
        {
                CL_LinuxJoystick *joy = new CL_LinuxJoystick();
                if (joy->init(i))
                        CL_Input::joysticks.add(joy);
                else
                        delete joy;
        }

#endif

	display_counter+=2;
}

#endif
