/*
	$Id: thread_pthread.cpp,v 1.2 1999/08/20 17:25:44 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#include <pthread.h>
#include "../API/cl_assert.h"
#include "thread_pthread.h"

CL_Thread *CL_Thread::create(void(*func)(void *))
{
	return new CL_Thread_Posix(func);
}

CL_Thread_Posix::CL_Thread_Posix(void(*func)(void *))
{
	this->func = func;
	running = false;
}

CL_Thread_Posix::~CL_Thread_Posix()
{
	terminate();
}

void CL_Thread_Posix::start(void *arg)
{
	if (running) return;

	this->arg = arg;

	cl_assert(
		pthread_create(
			&thread,
			NULL,
			run_init,
			this)==0);

	running = true;
}

void *CL_Thread_Posix::run_init(void *_self)
{
	CL_Thread_Posix *self = (CL_Thread_Posix *) _self;

	// kill thread immidiately - no cancelation point...
	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);

	self->func(self->arg);
	return NULL;
}

void CL_Thread_Posix::terminate()
{
	if (running) pthread_cancel(thread);
	running = false;
}
