/*
	$Id: app_windows.cpp,v 1.4 1999/03/12 03:48:53 starch Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		WinMain. Initialization of the Win32 version of ClanLib.

*/
#include <windows.h>
#include <mmsystem.h>
#include <iostream.h>
#include <crtdbg.h>

#include "Common/error.h"
#include "Common/clanstring.h"
#include "Layer1/System/Win32/implementation_directx.h"

#include "app_windows.h"
#include "timer_manager_win32.h"

HWND CL_WindowsSystem::hWnd = NULL;
HINSTANCE CL_WindowsSystem::hInstance;
HDC CL_WindowsSystem::hDC = NULL;
HGLRC CL_WindowsSystem::hRC = NULL;

CL_WindowsSystem *CL_WindowsSystem::system = NULL;

LONG WINAPI MainMessageHandler(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

bool windows_inited = false;

CL_WindowsSystem::CL_WindowsSystem()
{
	system = this;
}

CL_WindowsSystem::~CL_WindowsSystem()
{
	system = NULL;
}

void CL_WindowsSystem::add_listener(CL_Win32EventListener *listener)
{
	system->listeners.add(listener);
}

void CL_WindowsSystem::remove_listener(CL_Win32EventListener *listener)
{
	system->listeners.del(listener);
}

void CL_WindowsSystem::add_keepalive(CL_Win32KeepAlive *keep_alive)
{
	system->keep_alives.add(keep_alive);
}

void CL_WindowsSystem::remove_keepalive(CL_Win32KeepAlive *keep_alive)
{
	system->keep_alives.del(keep_alive);
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	CL_WindowsSystem system;

	int ret=255;

	#ifdef ENABLE_MEMORYDEBUG
		CL_MemManager::enable();
	#endif

	if (CL_ClanApplication::app == NULL)
	{
		MessageBox(NULL, "No program instance found", "ClanLib/Win32", 32);
		return 0;
	}

	CL_WindowsSystem::hInstance = hInstance;

	WNDCLASS wndclass;

	wndclass.style = 0;
	wndclass.lpfnWndProc = (WNDPROC) MainMessageHandler;
	wndclass.cbClsExtra = 0;
	wndclass.cbWndExtra = 0;
	wndclass.hInstance = hInstance;
	wndclass.hIcon = NULL;
	wndclass.hCursor = LoadCursor (NULL,IDC_ARROW);
	wndclass.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	wndclass.lpszMenuName = CL_ClanApplication::app->get_title();
	wndclass.lpszClassName = CL_ClanApplication::app->get_title();

	if (RegisterClass(&wndclass) == 0) return 0;

	cout << "Creating window" << endl;
	CL_WindowsSystem::hWnd = CreateWindow(
					CL_ClanApplication::app->get_title(),
					CL_ClanApplication::app->get_title(),
					WS_CAPTION|WS_SYSMENU,
					0,
					0,
					64, // GetSystemMetrics(SM_CXSCREEN),
					64, //GetSystemMetrics(SM_CYSCREEN),
					NULL,
					NULL,
					hInstance,
					NULL);

	if (CL_WindowsSystem::hWnd == NULL)
	{
		cout << "Somethings rotten" << endl;
		return 0;
	}

	ShowWindow(CL_WindowsSystem::hWnd, nCmdShow);
	UpdateWindow(CL_WindowsSystem::hWnd);

	new CL_TimerManager_Win32;

	CL_Implementation_DirectX::add_input();

	// Get commandline arguments:
	CL_String command_line = GetCommandLine();
	int num_words = 0;
	while (command_line.get_word(num_words)!="") num_words++;

	char **words = new char*[num_words];
	CL_String *string_words = new CL_String[num_words];

	int i;
	for (i=0; i<num_words; i++)
	{
		string_words[i] = command_line.get_word(i);
		words[i] = string_words[i].get_string();
	}

	ret=CL_ClanApplication::app->main(num_words, words);

	delete[] words;
	delete[] string_words;

	CL_Implementation_DirectX::remove_display();
	CL_Implementation_DirectX::remove_input();
	CL_Implementation_DirectX::remove_sound();
	CL_Implementation_DirectX::remove_network();

	delete CL_TimerManager_Win32::timermanager;

	cout << "Dumping memory info" << endl;
	try
	{
		#ifdef ENABLE_MEMORYDEBUG
		CL_MemManager::dump_info();
		#endif
	}
	catch (...)
	{
		cout << "Dump failed" << endl;
	}

	cout << "Now it is up to windows...." << endl;

	_CrtDumpMemoryLeaks();

	return ret;
}

LONG WINAPI MainMessageHandler(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	bool handled = false;

	CL_Iterator<CL_Win32EventListener> counter(CL_WindowsSystem::system->listeners);
	while (counter.next() != NULL)
	{
		if (counter()->received_event(uMsg, wParam, lParam)) handled = true;
	}

	if (handled) return 1;

	switch (uMsg)
	{
	case WM_CREATE:
		break;

	case WM_CLOSE:
		PostQuitMessage(0);
		break;

	case WM_DESTROY:
		break;

	case WM_PAINT:
		{
			PAINTSTRUCT ps;

			BeginPaint(hWnd, &ps);
			EndPaint(hWnd, &ps);
		}
		break;

	default:
		return DefWindowProc(hWnd, uMsg, wParam, lParam);
	}

	return 1;
}

/*
	Class CL_WindowsSystem
*/

bool CL_System::keep_alive()
{
	MSG msg;

	while (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE) == TRUE)
	{
		if (GetMessage(&msg, NULL, 0, 0))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else
		{
			return true;
		}
	}

	CL_Iterator<CL_Win32KeepAlive> counter(CL_WindowsSystem::system->keep_alives);
	while (counter.next() != NULL)
	{
		if (counter()->keep_alive()) return true;
	}

	return false;
}

int CL_System::get_time()
{
	return timeGetTime();
}

void CL_System::sleep(int millis)
{
	Sleep(millis);
}

void CL_System::init_display(int options)
{
	CL_Implementation_DirectX::add_display(options);
}

void CL_System::init_sound()
{
	CL_Implementation_DirectX::add_sound();
}

void CL_System::init_network()
{
	CL_Implementation_DirectX::add_network();
}

void CL_System::deinit_display()
{
	CL_Implementation_DirectX::remove_display();
}

void CL_System::deinit_sound()
{
	CL_Implementation_DirectX::remove_sound();
}

void CL_System::deinit_network()
{
	CL_Implementation_DirectX::remove_network();
}
