/*
	$Id: timer_manager_win32.cpp,v 1.2 1998/12/22 00:59:07 starch Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Provide ClanTimer support through Win32's multimedia timers.


	$Log: timer_manager_win32.cpp,v $
	Revision 1.2  1998/12/22 00:59:07  starch
	Fixed a gazillion include errors.
	
	Revision 1.1.1.1  1998/12/20 23:33:14  mbn
	ClanLib 0.1.10 development version importet.

	Revision 1.3  1998/09/17 12:06:38  mbn
	Converted all spaces to tabs by using unexpand.

	Revision 1.2	1998/02/17 22:09:14	 mbn
	Changed class prefix to 'CL'.

	Revision 1.1.1.1	1998/02/07 16:51:18	 mbn
	Finally uploaded ClanLib-0.1.3 to CVS.

*/

#include <windows.h>
#include <mmsystem.h>

#include "timer_manager_win32.h"

CL_List<STimerInfo> CL_TimerManager_Win32::timer_info;
MMRESULT CL_TimerManager_Win32::timer_handle = NULL;
int CL_TimerManager_Win32::timer_idcount = 0;
int CL_TimerManager_Win32::granularity = 0;
CL_TimerManager_Win32 *CL_TimerManager_Win32::timermanager = NULL;

void CALLBACK clantimercallback(UINT uID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2);

CL_TimerManager_Win32::CL_TimerManager_Win32()
{
	if (CL_TimerManager_Win32::timermanager != NULL)
	{
		throw 42;
	}

	CL_TimerManager_Win32::timermanager = this;
}

CL_TimerManager_Win32::~CL_TimerManager_Win32()
{
	CL_TimerManager_Win32::timermanager = NULL;

	CL_TimerManager_Win32::kill_timer();

	CL_Iterator<STimerInfo> counter(CL_TimerManager_Win32::timer_info);
	while (counter.next() != NULL)
	{
		STimerInfo *del_info = counter();
		counter.remove();
		delete del_info;
	}
}

void CL_TimerManager_Win32::create_timer(int granularity, int tolerance)
{
	TIMECAPS tc;
	unsigned int timer_res;

	MMRESULT err = timeGetDevCaps(&tc, sizeof(TIMECAPS));
	if (err != TIMERR_NOERROR)
	{
		throw 42;
	}

	timer_res = min(max(tc.wPeriodMin, (unsigned int) granularity), tc.wPeriodMax);
	err = timeBeginPeriod(timer_res);
	if (err != TIMERR_NOERROR)
	{
		throw 42;
	}

	CL_TimerManager_Win32::granularity = granularity;

	CL_TimerManager_Win32::timer_handle = timeSetEvent(granularity, tolerance, clantimercallback, 0, TIME_PERIODIC);
	if (CL_TimerManager_Win32::timer_handle == NULL)
	{
		throw 42;
	}

	err = timeEndPeriod(timer_res);
	if (err != TIMERR_NOERROR)
	{
		throw 42;
	}
}

void CL_TimerManager_Win32::kill_timer()
{
	if (CL_TimerManager_Win32::timer_handle != NULL)
	{
		timeKillEvent(CL_TimerManager_Win32::timer_handle);
	}
}

int CL_TimerManager_Win32::greatest_common_div(int int1, int int2)
{
	if (int2<int1) return CL_TimerManager_Win32::greatest_common_div(int2, int1);
	else if (int1==0) return int2;
	else return CL_TimerManager_Win32::greatest_common_div(int2%int1, int1);
}

bool CL_TimerManager::subscribe(int milliseconds, CL_TimerCallback *callback_class, float tolerance, bool periodic, bool delete_pointer)
{
	if (callback_class == NULL) return false;
	if (milliseconds <= 0) return false;
	
	CL_TimerManager_Win32::timer_info.add(new STimerInfo(milliseconds, tolerance, callback_class, timeGetTime(), periodic, delete_pointer));

	if (CL_TimerManager_Win32::timer_handle == NULL)
	{
		CL_TimerManager_Win32::create_timer(milliseconds, int(tolerance*milliseconds));
	}
	else if (milliseconds % CL_TimerManager_Win32::granularity != 0)
	{
		CL_Iterator<STimerInfo> counter(CL_TimerManager_Win32::timer_info);
		int new_granularity = counter.next()->callback_interval;
		float least_tolerance = counter()->tolerance;
		while (counter.next() != NULL)
		{
			new_granularity = CL_TimerManager_Win32::greatest_common_div(new_granularity, counter()->callback_interval);
			if (counter()->tolerance < least_tolerance) least_tolerance = counter()->tolerance;
		}

		CL_TimerManager_Win32::kill_timer();
		CL_TimerManager_Win32::create_timer(new_granularity, int(new_granularity*least_tolerance));
	}
	return true;
}

void CL_TimerManager::unsubscribe(CL_TimerCallback *callback_class)
{
	CL_Iterator<STimerInfo> counter(CL_TimerManager_Win32::timer_info);
	while (counter.next() != NULL)
	{
		if (counter()->callback_class == callback_class) break;
	}
	if (counter() != NULL)
	{
		STimerInfo *del_info = counter();
		counter.remove();
		delete del_info;
	}
}

void CALLBACK clantimercallback(UINT uID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2)
{
	CL_Iterator<STimerInfo> counter(CL_TimerManager_Win32::timer_info);

	counter.next();
	while (counter()!= NULL)
	{
		int cur_time = timeGetTime();
		STimerInfo *cur = counter();
		counter.next();
		if (cur_time - cur->latest_timestamp >= cur->callback_interval)
		{
			int new_time = cur->callback_class->callback(cur_time - cur->latest_timestamp);
			if (cur->periodic) cur->latest_timestamp = cur_time;
			else
			{
				CL_TimerManager_Win32::timer_info.del(cur);
				delete cur;
			}

			if (new_time > 0)
			{
				int milliseconds = new_time;
				CL_TimerCallback *callback_class = cur->callback_class;
				float tolerance = cur->tolerance;
				bool periodic = cur->periodic;
				bool delete_pointer = cur->delete_pointer;

				cur->delete_pointer = false;

				CL_TimerManager::unsubscribe(callback_class);
				CL_TimerManager::subscribe(milliseconds, callback_class, tolerance, periodic, delete_pointer);
			}
		}
	}
}
