/*
	$Id: inputbuffer_simple.cpp,v 1.1 1999/08/27 20:27:31 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#include "../../clanlib.h"
#include "inputbuffer_simple.h"

CL_InputBuffer_Simple::CL_InputBuffer_Simple(CL_InputBuffer *buffer)
{
	input_buffer = buffer;
}

CL_InputBuffer_Simple::~CL_InputBuffer_Simple()
{
}

CL_Key CL_InputBuffer_Simple::peek_key()
{
	read_buffer();

	if (keys.is_empty())
	{
		CL_Key k;
		k.state = CL_Key::NoKey;
		return k;
	}
	
	return keys.peek();
}

CL_Key CL_InputBuffer_Simple::get_key()
{
	read_buffer();

	if (keys.is_empty())
	{
		CL_Key k;
		k.state = CL_Key::NoKey;
		return k;
	}
	
	return keys.get();
}

int CL_InputBuffer_Simple::get_key_as_id()
{
	CL_Key k = get_key();
	if (k.state == CL_Key::NoKey) return -1;
	return k.id;
}

int CL_InputBuffer_Simple::get_key_as_ascii()
{
	CL_Key k = get_key();
	if (k.state == CL_Key::NoKey) return -1;
	return k.ascii;
}

int CL_InputBuffer_Simple::keys_left()
{
	read_buffer();
	return keys.get_num_items();
}

void CL_InputBuffer_Simple::clear()
{
	read_buffer();
	keys.clear();
}

void CL_InputBuffer_Simple::read_buffer()
{
	while (true)
	{
		CL_Key key = input_buffer->get_key();
		switch (key.state)
		{
		case CL_Key::Pressed:
			keys.put(key);
			break;
			
		case CL_Key::NoKey:
			return;
		
		default:
			break;
		}
	}
}
