/*
	$Id: font.cpp,v 1.10 1999/07/15 22:33:05 vogel Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Simple Font class.

*/

#include "font.h"
#include "font_description.h"

/*******************
   CL_Font class
*******************/
/*
CL_Font::CL_Font(
	CL_SurfaceProvider *source,
	const char *letters,
	int space_len,
	int subtract_width)
{
	this->space_len = space_len;
	this->subtract_width = subtract_width;
	
	int len = strlen(letters);
	
	for (int j=0; j<256; j++) chars[j] = NULL;
	
	CL_Font_Description cutter(
		source,
		0.05f,
		space_len,
		subtract_width,
		letters);

	for (int i=0; i<len; i++)
	{
		if (i == 0) height = cutter.letter_providers[i]->get_height();
	
		chars[(unsigned char) letters[i]] = CL_Surface::create(
			cutter.letter_providers[i],
			false);
	}
}
*/
CL_Font::CL_Font(
	CL_Font_Description *font_desc)
{
	space_len = font_desc->space_len;
	subtract_width = font_desc->subtract_width;

	int len = strlen(font_desc->letters);
	for (int j=0; j<256; j++) chars[j] = NULL;

	for (int i=0; i<len; i++)
	{
		if (i == 0) height = font_desc->letter_providers[i]->get_height();
	
		chars[(unsigned char) font_desc->letters[i]] = CL_Surface::create(
			font_desc->letter_providers[i],
			false);
	}
}

CL_Font::~CL_Font()
{
	for (int i=0; i<256; i++)
	{
		if (chars[i]!=NULL) delete chars[i];
	}
}

int CL_Font::get_text_width(const char *_text)
{
	int len = 0;

	unsigned char *text = (unsigned char *) _text;

	while (*text != 0)
	{
		if (chars[*text] != NULL)
		{
			len += chars[*text]->get_width()-subtract_width;
		}
		else
		{
			len += space_len;
		}

		text++;
	}

	return len;
}

void CL_Font::print_left(int x, int y, const char *_text)
{
	int pos_x = x;

	unsigned char *text = (unsigned char *) _text;

	while (*text != 0)
	{
		if (chars[*text] != NULL)
		{
			int width = chars[*text]->get_width()-subtract_width;

			chars[*text]->put_screen(pos_x-4, y);

			pos_x += width+1;
		}
		else
		{
			pos_x += space_len;
		}

		text++;
	}
}

void CL_Font::print_left(int x, int y, float sx, float sy, const char *_text)
{
	int pos_x = x;

	unsigned char *text = (unsigned char *) _text;

	while (*text != 0)
	{
		if (chars[*text] != NULL)
		{
			int width = (int) ((chars[*text]->get_width()-subtract_width)*sx);

			chars[*text]->put_screen(pos_x-4, y, sx, sy);

			pos_x += width+1;
		}
		else
		{
			pos_x += (int) (space_len*sx);
		}

		text++;
	}
}

void CL_Font::print_center(int x, int y, const char *text)
{
	print_left(x - get_text_width(text)/2, y, text);
}

void CL_Font::print_right(int x, int y, const char *text)
{
	print_left(x - get_text_width(text), y, text);
}
