/*
	$Id: font.h,v 1.10 1999/07/07 14:27:59 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Simple font support.
*/

//! l2_component="Misc"

#ifndef header_font
#define header_font

class CL_Surface;
class CL_Font_Description;

class CL_Font
//: ClanLib's layer2 font class.
{
protected:
	CL_Surface *chars[256];
	int space_len;
	int subtract_width;
	int height;

public:
/*
	CL_Font(
		CL_SurfaceProvider *provider,
		const char *letters,
		int space_width,
		int subtract_width);
*/	
	CL_Font(CL_Font_Description *font_desc);

	virtual ~CL_Font();

	int get_height() { return height; }
	//: Returns the height of the font.
	//!retval: Width of the font.

	int get_text_width(const char *text);
	//: Return the width of 'text' if drawn with this font.
	//!param: text - Text string the width should be calculated from.
	//!retval: Width of the text (in pixels).
	
	void print_left(int x, int y, const char *text);
	//: Prints 'text' left aligned at coordinates ('x','y').
	//!param: x - X coordinate (in pixels) the text is drawn at.
	//!param: y - Y coordiante (in pixels) the text is drawn at.
	//!param: text - The text to be drawn with this font.

	void print_left(int x, int y, float scale_x, float scale_y, const char *text);
	//: Prints 'text' left aligned at coordinates ('x','y').
	//!param: x - X coordinate (in pixels) the text is drawn at.
	//!param: y - Y coordiante (in pixels) the text is drawn at.
	//!param: scale_x - Scale font width scale_x times.
	//!param: scale_y - Scale font height scale_y times.
	//!param: text - The text to be drawn with this font.
	
	void print_center(int x, int y, const char *text);
	//: Prints 'text' center aligned at coordinates ('x','y').
	//!param: x - X coordinate (in pixels) the text is centered at.
	//!param: y - Y coordiante (in pixels) the text is centered at.
	//!param: text - The text to be drawn with this font.

	void print_right(int x, int y, const char *text);
	//: Prints 'text' right aligned at coordinates ('x','y').
	//!param: x - X coordinate (in pixels) the text is ends at.
	//!param: y - Y coordiante (in pixels) the text is drawn at.
	//!param: text - The text to be drawn with this font.
};

#endif
