#include <stdio.h>
#include "netchannel_filetransfer.h"
#include "netchannel_portnumbers.h"

#ifndef min
#define min(a, b) ((a<b) ? (a) : (b))
#endif

void CL_NetChannel_FileTransfer::transfer_file(
	CL_NetGame *netgame, 
	CL_NetGroup *receiver_group, 
	CL_String filename_id, 
	CL_InputSourceProvider *provider)
{
	cl_assert(netgame != NULL);
	cl_assert(receiver_group != NULL);

	CL_InputSourceProvider *p = provider != NULL ? provider : CL_InputSourceProvider::create_file_provider(".");

	CL_InputSource *source = p->open_source(filename_id);
	cl_assert(source != NULL);

	CL_DataWriter writer(source->size()+filename_id.get_length()+8);
	writer.write_string(filename_id);
	writer.write_int32(source->size());

	int data_left = source->size();
	unsigned char buffer[1024];
	while (data_left > 0)
	{
		int to_read = min(1024, data_left);
		source->read(buffer, to_read);
		writer.write_data(buffer, to_read);
		data_left -= to_read;
	}

	CL_NetMessage msg(writer.get_data(), writer.get_size());
	netgame->send(receiver_group, this, &msg);

	delete source;

	if (provider == NULL) delete p;
}

void CL_NetChannel_FileTransfer::transfer_file(
	CL_NetGame *netgame, 
	CL_NetComputer *receiver_computer, 
	CL_String filename_id, 
	CL_InputSourceProvider *provider)
{
	cl_assert(netgame != NULL);
	cl_assert(receiver_computer != NULL);

	CL_InputSourceProvider *p = provider != NULL ? provider : CL_InputSourceProvider::create_file_provider(".");

	CL_InputSource *source = p->open_source(filename_id);
	cl_assert(source != NULL);

	CL_DataWriter writer(source->size()+filename_id.get_length()+8);
	writer.write_string(filename_id);
	writer.write_int32(source->size());

	int data_left = source->size();
	unsigned char buffer[1024];
	while (data_left > 0)
	{
		int to_read = min(1024, data_left);
		source->read(buffer, to_read);
		writer.write_data(buffer, to_read);
		data_left -= to_read;
	}

	CL_NetMessage msg(writer.get_data(), writer.get_size());
	netgame->send(receiver_computer, this, &msg);

	delete source;

	if (provider == NULL) delete p;
}

int CL_NetChannel_FileTransfer::get_netchannel()
{
	return CL_NETPORT_FILETRANSFER;
}

void CL_NetChannel_FileTransfer::receive(CL_NetComputer *from, CL_NetMessage *message)
{
	CL_DataReader reader((unsigned char *) message->data, message->size);

	CL_String filename = reader.read_string();
	int file_size = reader.read_int32();

	cout << "Received file " << filename << " of length " << file_size << endl;

	FILE *out = fopen(filename, "wb");

	fwrite(reader.get_data_ptr(), file_size, 1, out);

	fclose(out);
}

bool CL_NetChannel_FileTransfer::is_secure()
{
	return true;
}
