/*
	$Id: netdata_controller.h,v 1.2 1999/09/02 14:45:43 sphair Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

//! l2_component="NetChannels"

#ifndef header_netdata_controller
#define header_netdata_controller

#define MAX_PACKET_SIZE 1024

#include "netobject.h"
#include "netobject_creator.h"

#include "../Misc/inputsource_memory.h"
#include "../Misc/outputsource_memory.h"

/*****************************************************************************/

class CL_NetDataInfo
{
public:
	CL_NetDataInfo(const CL_NetComputer *dest, CL_NetDataObject *object, int id, int type, int priority);
	CL_NetDataInfo(CL_NetDataObject *object, int id, int type, int size);

public:
	const CL_NetComputer *dest;
	CL_NetDataObject *object;

	int id;
	int type;
	int priority;

	int data_size;
	int data_left;
};

/*****************************************************************************/

class CL_NetDataController
{
public:
	CL_NetDataController();
	virtual ~CL_NetDataController() {};

	virtual void init(CL_NetDataCreator *creator, CL_NetGame *game, int channel);
	
	virtual void send(const CL_NetComputer *dest, CL_NetDataObject *object, int type, int priority = 0);
	virtual void update_client();
	virtual bool update_server(int priority=0, const CL_NetComputer *dest=NULL);

private:
	enum TransferMode
	{
		InitTransfer,
		Transfer,
		EndTransfer
	};

protected:
	CL_Array<CL_NetDataInfo> serverTransfers;
	CL_Array<CL_NetDataInfo> clientTransfers;

	CL_NetGame *game;
	int channel;

	CL_NetDataCreator *creator;
	int next_object_id;
};

/*****************************************************************************/

class CL_NetFileController : public CL_NetDataController
{
public:
	CL_NetFileController() {};
	virtual ~CL_NetFileController() {};

	void send(const CL_NetComputer *dest, char *filename, int type, int priority = 0);
};

#endif
