/*
	$Id: netobject.h,v 1.12 1999/09/02 14:45:43 sphair Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

//! l2_component="NetChannels"

#ifndef header_netobject
#define header_netobject

#include "../../clanlib.h"

/*****************************************************************************/

class CL_NetObject
//: NetObject interface.
// Please read the overview network documentation for an explanation of this
// class.
{
public:
	CL_NetObject() {};
	virtual ~CL_NetObject() {};

	virtual void serialize_save(CL_OutputSource *output)=0;
	virtual void serialize_load(CL_InputSource *input)=0;
};

/*****************************************************************************/

class CL_NetDataObject
//: NetDataObject interface.
// These objects are used with the CL_NetDataController.
//!also: CL_NetDataController
{
public:
	CL_NetDataObject() {};
	virtual ~CL_NetDataObject() {};

	virtual int get_size()=0;
	//: Return the size of the object to be sent.
	
	virtual void serialize_save(CL_OutputSource *output, int pos, int length)=0;
	//: Writes the objectdata starting at position 'pos' and with a length 'length'.
	// If pos==0 and size!=0, then the first packet is to be written.
	// If pos!=0 and size==0, then the sending of the object is complete.
	
	virtual void serialize_load(CL_InputSource *input, int pos, int length)=0;
	//: Reads the object starting at position 'pos' and with a length 'length'.
	// If pos==0 and size!=0, then the first packet is to be read.
	// If pos!=0 and size==0, then the receiving of the object is complete.
};

/*****************************************************************************/

class CL_NetFileObject : public CL_NetDataObject
{
public:
	CL_NetFileObject(char *filename);
	virtual ~CL_NetFileObject() {};

	virtual int get_size();
	virtual void serialize_save(CL_OutputSource *output, int pos, int length);
	virtual void serialize_load(CL_InputSource *input, int pos, int length);

private:
	CL_InputSource *input_file;
	CL_OutputSource *output_file;
};

/*****************************************************************************/

class CL_NetData;
class CL_NetObject_Simple : public CL_NetObject
//: Simplified serialization NetObject.
// CL_NetObject_Simple is a netobject using a more static approach to its
// serialization. In the constructor of your object, you should add those
// member variables you want to be duplicated on the slave side.
//
// Use the add_int32(), add_short16(), etc. functions to do this.
//
// For more information: Read the network overview document.
{
public:
	CL_NetObject_Simple();
	virtual ~CL_NetObject_Simple();

	virtual void serialize_save(CL_OutputSource *output);
	virtual void serialize_load(CL_InputSource *input);

protected:
	void add_int32(int *ptr, int num=1);
	void add_uint32(unsigned int *ptr, int num=1);
	void add_short16(short *ptr, int num=1);
	void add_ushort16(unsigned short *ptr, int num=1);
	void add_float32(float *ptr, int num=1);
	void add_bool(bool *ptr, int num=1);
	void add_data(void *ptr, int size, int num=1);
	void add_child(CL_NetObject *child, int num=1);
	void add_netdata(CL_NetData *netdata_child);

private:
	CL_List<CL_NetData> serialize_data;
};

class CL_NetData
{
public:
	CL_NetData(int num);
	virtual ~CL_NetData() { return; }
	virtual void save(CL_OutputSource *output)=0;
	virtual void load(CL_InputSource *input)=0;
protected:
	int m_num;
};

class CL_NetData_Int32 : public CL_NetData
{
public:
	CL_NetData_Int32(int *ptr, int num);
	virtual void save(CL_OutputSource *output);
	virtual void load(CL_InputSource *input);
private:
	int *m_ptr;
};

class CL_NetData_Bool : public CL_NetData
{
public:
	CL_NetData_Bool(bool *ptr, int num);
	virtual void save(CL_OutputSource *output);
	virtual void load(CL_InputSource *input);
private:
	bool *m_ptr;
};

class CL_NetData_Short16 : public CL_NetData
{
public:
	CL_NetData_Short16(short *ptr, int num);
	virtual void save(CL_OutputSource *output);
	virtual void load(CL_InputSource *input);
private:
	short *m_ptr;
};

class CL_NetData_Float32 : public CL_NetData
{
public:
	CL_NetData_Float32(float *ptr, int num);
	virtual void save(CL_OutputSource *output);
	virtual void load(CL_InputSource *input);
private:
	float *m_ptr;
};

class CL_NetData_Data : public CL_NetData
{
public:
	CL_NetData_Data(void *data, int size, int num);
	virtual void save(CL_OutputSource *output);
	virtual void load(CL_InputSource *input);
private:
	void *m_data;
	int m_size;
};

class CL_NetData_Child : public CL_NetData
{
public:
	CL_NetData_Child(CL_NetObject *child, int num);
	virtual void save(CL_OutputSource *output);
	virtual void load(CL_InputSource *input);
private:
	CL_NetObject *m_child;
};

#endif
