/*
	$Id: netobject_controller_basic.cpp,v 1.2 1999/08/31 22:42:18 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#include "netobject_controller_basic.h"
#include "netobject.h"

#include "../Misc/inputsource_memory.h"
#include "../Misc/outputsource_memory.h"

CL_NetObjectController_Basic::CL_NetObjectController_Basic()
{
	// bug bug: obj_id must be unique across computers as well.
	next_obj_id = 0;
	
	creator = NULL;
}

CL_NetObjectController_Basic::~CL_NetObjectController_Basic()
{
	// hmm... maybe we should delete netobjects here?
	// maybe it'll be better if handled by the game? Not sure yet.
}

void CL_NetObjectController_Basic::add(CL_NetObject *netobj, int type)
{
	CL_NetObjectInfo *info = new CL_NetObjectInfo;
	info->obj = netobj;
	info->id = next_obj_id++;
	info->type = type;

	netobjects.add(info);
}

void CL_NetObjectController_Basic::remove(CL_NetObject *netobj)
{
	CL_Iterator<CL_NetObjectInfo> counter(netobjects);
	while (counter.next() != NULL)
	{
		if (counter()->obj == netobj)
		{
			delete counter();
			counter.remove();
			break;
		}
	}
}

int CL_NetObjectController_Basic::get_netobj_id(CL_NetObject *netobj)
{
	CL_Iterator<CL_NetObjectInfo> counter(netobjects);
	while (counter.next() != NULL)
	{
		if (counter()->obj == netobj) return counter()->id;
	}

	return -1;
}

void CL_NetObjectController_Basic::set_creator(CL_NetObjectCreator *creator)
{
	this->creator = creator;
}

void CL_NetObjectController_Basic::update(CL_NetGame *game, int netchannel)
{
	while (game->peek(netchannel))
	{
		CL_NetMessage message = game->receive(netchannel);
		CL_InputSource_Memory input(message.data, message.size, true);
		
		cl_assert(input.size() > 0);

		int netobj_id = input.read_int32();
		int netobj_type = input.read_int32();

		// Performance bug: List of netobjects should be a hash list.

		CL_Iterator<CL_NetObjectInfo> it_netobjs(netobjects);
		while (it_netobjs.next() != NULL)
		{
			if (it_netobjs()->id == netobj_id)
			{
				it_netobjs()->obj->serialize_load(&input);
				break;
			}
		}

		if (it_netobjs() == NULL) // didn't find netobj in list.
		{
			if (creator != NULL)
			{
				CL_NetObject *obj = creator->create(
					netobj_type,
					&input);
				
				if (obj != NULL)
				{
					CL_NetObjectInfo *info = new CL_NetObjectInfo;
					info->obj = obj;
					info->id = netobj_id++;
					info->type = netobj_type;

					netobjects.add(info);
				}
			}
		}
	}
}

void CL_NetObjectController_Basic::send(
	CL_NetGame *game,
	int channel,
	const CL_NetGroup *dest)
{
	cl_assert_debug(game != NULL);
	cl_assert_debug(obj != NULL);
	if (dest == NULL) dest = game->get_all();

	// Performance bug: List of netobjects should be a hash list.

	CL_Iterator<CL_NetObjectInfo> it_netobjs(netobjects);
	while (it_netobjs.next() != NULL)
	{
		CL_OutputSource_Memory output;
		output.write_int32(it_netobjs()->id);
		output.write_int32(it_netobjs()->type);
		it_netobjs()->obj->serialize_save(&output);

		game->send(
			channel,
			dest,
			CL_NetMessage(output.get_data(), output.size()));
	}
}

void CL_NetObjectController_Basic::send(
	CL_NetGame *game,
	int channel,
	CL_NetObject *obj,
	const CL_NetGroup *dest)
{
	cl_assert_debug(game != NULL);
	cl_assert_debug(obj != NULL);
	if (dest == NULL) dest = game->get_all();

	// Performance bug: List of netobjects should be a hash list.

	CL_Iterator<CL_NetObjectInfo> it_netobjs(netobjects);
	while (it_netobjs.next() != NULL)
	{
		if (it_netobjs()->obj == obj)
		{
			CL_OutputSource_Memory output;
			output.write_int32(it_netobjs()->id);
			output.write_int32(it_netobjs()->type);
			obj->serialize_save(&output);

			game->send(
				channel,
				dest,
				CL_NetMessage(output.get_data(), output.size()));

			return;
		}
	}
	
	// if this assertion occours in your application, you tried to send an
	// object not added to this controller.
	cl_assert(false);
}
