/*
	$Id: resourceoptions.cpp,v 1.1 1999/05/27 21:42:14 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#include "resourceoptions.h"
#include "resourceoption.h"
#include "resource_manager.h"

CL_ResourceOptions::CL_ResourceOptions(CL_ResourceManager *_parent)
{
	parent = _parent;
}

CL_ResourceOptions::~CL_ResourceOptions()
{
	CL_Iterator<CL_ResourceOption> counter(options);
	while (counter.next() != NULL)
	{
		delete counter();
	}
}

bool CL_ResourceOptions::read_options(const char *resource_name)
{
	CL_String token = parent->get_next_token();
	if (token != "(" && token != ";")
	{
		CL_String err;
		err << "Missing '(' following declaration of resource '"
			<< resource_name << "'";

		parent->write_error(err);

		return false;
	}
	if (token == ";") 
	{
		// Dirty anti-get
		parent->cur_pos--;
		return true;
	}

	token = "";
	while (token != ")")
	{
		CL_String option_name = parent->get_next_token();
//		option_name.to_lower();

		CL_String option_value = "";
		bool has_value = false;

		token = parent->get_next_token();
		if (token == "=")
		{
			token = parent->get_next_token();

			if (token == "(")
			{
				CL_List<CL_String> *value_list = new CL_List<CL_String>;

				while (true)
				{
					value_list->add(new CL_String(parent->get_next_token()));
					token = parent->get_next_token();

					if (token == ")")
					{
						token = parent->get_next_token();
						options.add(new CL_ResourceOption(option_name, value_list));
						break;
					}
					else if (token != ",")
					{
						CL_String err;
						err << "Missing ',' or ')' following declaration of multiple option values in option '" <<
							   option_name << "' in resource '" << resource_name << "' (found '" << token << "')";
						parent->write_error(err);

						CL_Iterator<CL_String> cleanup(value_list);
						while (cleanup.next()!=NULL) delete cleanup();
						delete value_list;

						return false;
					}
				}
			}
			else
			{
				option_value = token;
				has_value = true;
				token = parent->get_next_token();
	
				options.add(new CL_ResourceOption(option_name, option_value));
			}
		}

		if (token != "," && token != ")")
		{
			CL_String err;
			err << "Missing ',' or ')' following declaration of option '" <<
				   option_name << "' in resource '" << resource_name << "' (found '" << token << "')";
			parent->write_error(err);
			return false;
		}

		if (!has_value)
		{
			options.add(new CL_ResourceOption(option_name));
		}
	}
	return true;
}

bool CL_ResourceOptions::exists(CL_String option)
{
//	option.to_lower();

	CL_Iterator<CL_ResourceOption> counter(options);
	while (counter.next() != NULL)
	{
		if (counter()->get_name() == option) return true;
	}
	return false;
}

CL_String CL_ResourceOptions::get_value(CL_String option)
{
//	option.to_lower();

	CL_Iterator<CL_ResourceOption> counter(options);
	while (counter.next() != NULL)
	{
		if (counter()->get_name() == option) return counter()->get_value();
	}
	return "";
}

CL_ResourceOption *CL_ResourceOptions::get_option(CL_String option)
{
//	option.to_lower();

	CL_Iterator<CL_ResourceOption> counter(options);
	while (counter.next() != NULL)
	{
		if (counter()->get_name() == option) return counter();
	}
	return NULL;
}

CL_List<CL_ResourceOption> *CL_ResourceOptions::get_options() 
{
	return &options;
}
