/*
	$Id: resourcetype_boolean.cpp,v 1.1 1999/07/18 18:35:29 arjanl Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Boolean resource type

*/

#include "../../clanlib.h"
#include "resourcetype_boolean.h"

/******************************************************************************
						class CL_Res_Boolean
******************************************************************************/

CL_Res_Boolean::CL_Res_Boolean() : CL_ResourceType("boolean")
{
}

CL_Resource *CL_Res_Boolean::try_create_from_file(CL_String name, CL_String prefix, CL_String res_id, CL_ResourceOptions *options, CL_ResourceManager *parent)
{

        bool is_boolean_type = false;

        if (options->exists("type"))
        {
                if (options->get_value("type") != "boolean") return NULL;
                is_boolean_type = true;
        }

        if (is_boolean_type /* || insert other guessing here */ )
        {
                return new CL_BooleanFileResource(name, prefix, res_id, options, parent );
        }

        return NULL;
}

CL_Resource *CL_Res_Boolean::try_create_from_datafile(CL_String name, CL_String prefix, CL_String res_id, CL_ResourceOptions *options, CL_ResourceManager *parent)
{
        bool is_boolean_type = false;

        if (options->exists("type"))
        {
                if (options->get_value("type") != "boolean") return NULL;
                is_boolean_type = true;
        }

        if (is_boolean_type /* || insert other guessing here */ )
        {
                return new CL_BooleanDatafileResource(name, prefix, res_id, options, parent );
        }

        return NULL;
}

bool CL_Res_Boolean::load(CL_String res_id, CL_ResourceManager &manager, bool _default)
{
	CL_Resource *res = manager.get_resource(res_id);
	if( res==NULL ) 
		return _default;
        else
		return ((CL_WritableBooleanResource *) res)->create_boolean();
}

bool CL_Res_Boolean::load(CL_String res_id, CL_ResourceManager &manager )
{
	CL_Resource *res = manager.get_resource(res_id);
	cl_assert( res!=NULL );
	return ((CL_WritableBooleanResource *) res)->create_boolean();
}


/******************************************************************************
						class CL_WritableBooleanResource
******************************************************************************/

CL_WritableBooleanResource::CL_WritableBooleanResource() : CL_Resource("boolean") 
{
	loaded = false;
	loaded_boolean = 0;
}

void CL_WritableBooleanResource::write(CL_OutputSource *output)
{
	bool b = create_boolean();

	if( b )
	  	output->write_int32( 1 );
	else
	  	output->write_int32( 0 );

}

bool CL_WritableBooleanResource::load()
{
	if (loaded) return true;

	loaded_boolean = create_boolean();
	loaded = true;
	return true;
}

/******************************************************************************
						class CL_BooleanFileResource
******************************************************************************/

CL_BooleanFileResource::CL_BooleanFileResource(CL_String _boolean_id, CL_String prefix, CL_String _boolean_location, CL_ResourceOptions *_options, CL_ResourceManager *_parent)
{
	name = _boolean_id;
	if (prefix == "") full_name = name; else full_name = prefix + "/" + name;
	boolean_location = _boolean_location;
	options = _options;
	parent = _parent;
}

bool CL_BooleanFileResource::create_boolean()
{
	if( boolean_location=="true" || 
            boolean_location=="yes"  ||
            boolean_location=="True" ||
            boolean_location=="Yes"  )
          loaded_boolean = true;
        else if( boolean_location=="false" ||
                 boolean_location=="no"  ||
                 boolean_location=="False" ||
                 boolean_location=="No"  )
          loaded_boolean = false;
        else
          throw new CL_Error("Unrecognized boolean resourcetype");

        return loaded_boolean;
}

/******************************************************************************
						class CL_BooleanDatafileResource
******************************************************************************/

CL_BooleanDatafileResource::CL_BooleanDatafileResource(
	CL_String _boolean_id,
	CL_String prefix,
	CL_String _boolean_location,
	CL_ResourceOptions *_options,
	CL_ResourceManager *_parent)
{
	name = _boolean_id;
	if (prefix == "") full_name = name; else full_name = prefix + "/" + name;
	boolean_location = _boolean_location;
	options = _options;
	parent = _parent;
}

bool CL_BooleanDatafileResource::create_boolean()
{
        if (parent->get_resource_provider() == NULL) return 0;

	CL_InputSource *input = parent->get_resource_provider()->open_source(full_name);
	if (input == NULL) return 0;

	if( input->read_int32()==1 )
		return true;
	else
		return false;
}

