
//! l2_component="Resources"

#ifndef header_resourcetype_boolean
#define header_resourcetype_boolean

#include "../../clanlib.h"
#include "resource_manager.h"

class CL_Res_Boolean : public CL_ResourceType
//: The Boolean resource is used to read boolean values from a resource file.
// CL_Res_Boolean, CL_Res_Integer and CL_Res_String helps you with seperating
// the game code from the constants used in the game.
// <br>
// Using these resource types, you can easily change the text used in the game.
// Just by changing the resource file, you can make you game speak Danish, or
// any other language. You can change the constants like weapon damage, movement
// speed, and so on.
// <br>
// These resources can also be used to apply theme support into your application.
// By letting each theme become a datafile, you can give the player full control
// of the visual parts of the game - and without using much work on it yourself.
//!res_type: boolean
//!res_desc: The boolean resource is used to read a boolean from a resourcefile
{
public:
	static bool load(CL_String res_id, CL_ResourceManager &manager, bool _default);
	static bool load(CL_String res_id, CL_ResourceManager &manager );

	CL_Res_Boolean();

private:
friend CL_ResourceManager;
	virtual CL_Resource *try_create_from_file(
		CL_String name,
		CL_String prefix,
		CL_String res_id,
		CL_ResourceOptions *options,
		CL_ResourceManager *parent);

	virtual CL_Resource *try_create_from_datafile(
		CL_String name,
		CL_String prefix,
		CL_String res_id,
		CL_ResourceOptions *options,
		CL_ResourceManager *parent);

};

//// start here ////

class CL_WritableBooleanResource : public CL_Resource
{
protected:
	bool loaded_boolean;
	bool loaded;
	CL_String boolean_location;
	CL_ResourceOptions *options;
	CL_ResourceManager *parent;
public:
	CL_WritableBooleanResource();
	virtual ~CL_WritableBooleanResource() { ; }

	virtual void write(CL_OutputSource *output);
	virtual bool load();

	virtual bool create_boolean()=0;
};

class CL_BooleanFileResource : public CL_WritableBooleanResource
{
public:
	CL_BooleanFileResource(CL_String boolean_id, CL_String prefix, CL_String palette_location, CL_ResourceOptions *options, CL_ResourceManager *parent);
	virtual bool create_boolean();
};

class CL_BooleanDatafileResource : public CL_WritableBooleanResource
{
public:
	CL_BooleanDatafileResource(CL_String boolean_id, CL_String prefix, CL_String boolean_location, CL_ResourceOptions *options, CL_ResourceManager *parent);
	virtual bool create_boolean();
};

#endif
