/*
	$Id: resourcetype_integer.cpp,v 1.2 1999/06/30 12:53:36 arjanl Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Integer resource type

*/

#include "../../clanlib.h"
#include "resourcetype_integer.h"

/******************************************************************************
						class CL_Res_Integer
******************************************************************************/

CL_Res_Integer::CL_Res_Integer() : CL_ResourceType("integer")
{
}

CL_Resource *CL_Res_Integer::try_create_from_file(CL_String name, CL_String prefix, CL_String res_id, CL_ResourceOptions *options, CL_ResourceManager *parent)
{

        bool is_integer_type = false;

        if (options->exists("type"))
        {
                if (options->get_value("type") != "integer") return NULL;
                is_integer_type = true;
        }

        if (is_integer_type /* || insert other guessing here */ )
        {
                return new CL_IntegerFileResource(name, prefix, res_id, options, parent );
        }

        return NULL;
}

CL_Resource *CL_Res_Integer::try_create_from_datafile(CL_String name, CL_String prefix, CL_String res_id, CL_ResourceOptions *options, CL_ResourceManager *parent)
{
        bool is_integer_type = false;

        if (options->exists("type"))
        {
                if (options->get_value("type") != "integer") return NULL;
                is_integer_type = true;
        }

        if (is_integer_type /* || insert other guessing here */ )
        {
                return new CL_IntegerDatafileResource(name, prefix, res_id, options, parent );
        }

        return NULL;
}

int CL_Res_Integer::load(CL_String res_id, CL_ResourceManager &manager, int Default)
{
	CL_Resource *res = manager.get_resource(res_id);
	if( res==NULL ) 
		return Default;
        else
		return ((CL_WritableIntegerResource *) res)->create_integer();
}

int CL_Res_Integer::load(CL_String res_id, CL_ResourceManager &manager )
{
	CL_Resource *res = manager.get_resource(res_id);
	cl_assert( res!=NULL );
	return ((CL_WritableIntegerResource *) res)->create_integer();
}


/******************************************************************************
						class CL_WritableIntegerResource
******************************************************************************/

CL_WritableIntegerResource::CL_WritableIntegerResource() : CL_Resource("integer") 
{
	loaded = false;
	loaded_integer = 0;
}

void CL_WritableIntegerResource::write(CL_OutputSource *output)
{
	int i = create_integer();

	output->write_int32( i );
}

bool CL_WritableIntegerResource::load()
{
	if (loaded) return true;

	loaded_integer = create_integer();
	loaded = true;
	return true;
}

/******************************************************************************
						class CL_IntegerFileResource
******************************************************************************/

CL_IntegerFileResource::CL_IntegerFileResource(CL_String _integer_id, CL_String prefix, CL_String _integer_location, CL_ResourceOptions *_options, CL_ResourceManager *_parent)
{
	name = _integer_id;
	if (prefix == "") full_name = name; else full_name = prefix + "/" + name;
	integer_location = _integer_location;
	options = _options;
	parent = _parent;
}

int CL_IntegerFileResource::create_integer()
{
	int base = 0;

        if (options->exists("base"))
		base = strtol(options->get_value("base"), NULL, 0 );

	loaded_integer = strtol( integer_location, NULL, base );
        return loaded_integer;
}

/******************************************************************************
						class CL_IntegerDatafileResource
******************************************************************************/

CL_IntegerDatafileResource::CL_IntegerDatafileResource(
	CL_String _integer_id,
	CL_String prefix,
	CL_String _integer_location,
	CL_ResourceOptions *_options,
	CL_ResourceManager *_parent)
{
	name = _integer_id;
	if (prefix == "") full_name = name; else full_name = prefix + "/" + name;
	integer_location = _integer_location;
	options = _options;
	parent = _parent;
}

int CL_IntegerDatafileResource::create_integer()
{
        if (parent->get_resource_provider() == NULL) return 0;

	CL_InputSource *input = parent->get_resource_provider()->open_source(full_name);
	if (input == NULL) return 0;

	return input->read_int32();
}

