/*
	$Id: resourcetype_raw.cpp,v 1.4 1999/04/05 13:05:16 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Raw resource type

*/

#include "clanlib.h"
#include "resourcetype_raw.h"

/**************************
     class CL_Res_Raw
**************************/

CL_Res_Raw::CL_Res_Raw() : CL_ResourceType("raw")
{
}

CL_Resource *CL_Res_Raw::try_create_from_file(
	CL_String name,
	CL_String prefix,
	CL_String res_id,
	CL_ResourceOptions *options,
	CL_ResourceManager *parent)
{
	CL_String ext = res_id.right(4);
	ext.to_lower();

	bool is_raw_type = false;
	if (options->exists("type")) 
	{
		if (options->get_value("type") != "raw") return NULL;
		is_raw_type = true;
	}
	
	if (is_raw_type ||
		options->exists("raw") || ext == ".raw")
	{
		return new CL_RawFileResource(name, prefix, res_id, options, parent);
	}

	return NULL;
}

CL_Resource *CL_Res_Raw::try_create_from_datafile(
	CL_String name,
	CL_String prefix,
	CL_String res_id,
	CL_ResourceOptions *options,
	CL_ResourceManager *parent)
{
	CL_String ext = res_id.right(4);
	ext.to_lower();

	bool is_font_type = false;
	if (options->exists("type")) 
	{
		if (options->get_value("type") != "raw") return NULL;
		is_font_type = true;
	}
	
	if (is_font_type ||
		options->exists("raw") || ext == ".raw")
	{
		return new CL_RawDatafileResource(name, prefix, res_id, options, parent);
	}

	return NULL;
}

CL_InputSource *CL_Res_Raw::load(CL_String res_id, CL_ResourceManager &manager)
{
	CL_Resource *res = manager.get_resource(res_id);
	cl_assert(res != NULL);
//	cl_assert(res->get_type() == "raw");
	if (res->get_type() != "raw")
		throw CL_Error("Resource is not a raw!!!");

	return ((CL_WritableRawResource *) res)->create_raw();
}

/***************************************
     class CL_WritableRawResource
***************************************/

CL_WritableRawResource::CL_WritableRawResource() : CL_Resource("raw")
{
	loaded = false;
}

void CL_WritableRawResource::write(CL_OutputSource *output)
{
	cl_assert(load());
	
	int size = loaded_raw->size();
	char *data = new char[size];

	loaded_raw->read(data, size);
	output->write(data, size);
	
	delete[] data;
}

bool CL_WritableRawResource::load()
{
	if (loaded) return true;

	loaded_raw = create_raw();
	if (loaded_raw == NULL) return false;

	loaded = true;

	return true;
}

/*********************************
     class CL_RawFileResource
*********************************/

CL_RawFileResource::CL_RawFileResource(
	CL_String id,
	CL_String prefix,
	CL_String _location,
	CL_ResourceOptions *_options,
	CL_ResourceManager *_parent)
{
	name = id;
	if (prefix == "") full_name = name; else full_name = prefix + "/" + name;
	location = _location;
	options = _options;
	parent = _parent;
}

CL_InputSource *CL_RawFileResource::create_raw()
{
	CL_InputSourceProvider *provider = CL_InputSourceProvider::create_file_provider(".");
	CL_InputSource *raw = provider->open_source(location);
	delete provider;

	return raw;
}

/************************************
     class CL_RawDatafileResource
************************************/

CL_RawDatafileResource::CL_RawDatafileResource(
	CL_String _id,
	CL_String prefix,
	CL_String _location,
	CL_ResourceOptions *_options,
	CL_ResourceManager *_parent)
{
	name = _id;
	if (prefix == "") full_name = name; else full_name = prefix + "/" + name;
	location = _location;
	options = _options;
	parent = _parent;
}

CL_InputSource *CL_RawDatafileResource::create_raw()
{
	if (parent->get_resource_provider() == NULL) return NULL;

	CL_InputSource *raw = parent->get_resource_provider()->open_source(full_name);

	return raw;
}

