
//! l2_component="Resources"

#ifndef header_resourcetype_raw
#define header_resourcetype_raw

#include "../../clanlib.h"
#include "resource_manager.h"

class CL_Res_Raw : public CL_ResourceType
//: The raw resource type.
// Using this resource type, you can load a raw file using the resource
// management system.
//!res_type: raw
//!res_desc: Includes a file into the datafile without altering the data in any way.
{
friend CL_ResourceManager;

	virtual CL_Resource *try_create_from_file(
		CL_String name,
		CL_String prefix,
		CL_String res_id,
		CL_ResourceOptions *options,
		CL_ResourceManager *parent);

	virtual CL_Resource *try_create_from_datafile(
		CL_String name,
		CL_String prefix,
		CL_String res_id,
		CL_ResourceOptions *options,
		CL_ResourceManager *parent);

public:
	static CL_InputSource *load(CL_String res_id, CL_ResourceManager &manager);

	CL_Res_Raw();
};

class CL_WritableRawResource : public CL_Resource
{
protected:
	CL_ResourceOptions *options;
	CL_ResourceManager *parent;
	bool loaded;
	CL_InputSource *loaded_raw;
	CL_String location;
	
public:
	CL_WritableRawResource();
	virtual ~CL_WritableRawResource() { ; }

	virtual void write(CL_OutputSource *output);
	virtual bool load();

	virtual CL_InputSource *create_raw()=0;
};

class CL_RawFileResource : public CL_WritableRawResource
{
public:
	CL_RawFileResource(
		CL_String id,
		CL_String prefix,
		CL_String location,
		CL_ResourceOptions *options,
		CL_ResourceManager *parent);

	virtual CL_InputSource *create_raw();
};

class CL_RawDatafileResource : public CL_WritableRawResource
{
public:
	CL_RawDatafileResource(
		CL_String id,
		CL_String prefix,
		CL_String location,
		CL_ResourceOptions *options,
		CL_ResourceManager *parent);

	virtual CL_InputSource *create_raw();
};

#endif
