/*
	$Id: resourcetype_string.cpp,v 1.2 1999/06/30 12:54:44 arjanl Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		String resource type

*/

#include "../../clanlib.h"
#include "resourcetype_string.h"

/******************************************************************************
						class CL_Res_String
******************************************************************************/

CL_Res_String::CL_Res_String() : CL_ResourceType("string")
{
}

CL_Resource *CL_Res_String::try_create_from_file(CL_String name, CL_String prefix, CL_String res_id, CL_ResourceOptions *options, CL_ResourceManager *parent)
{

        bool is_string_type = false;

        if (options->exists("type"))
        {
                if (options->get_value("type") != "string") return NULL;
                is_string_type = true;
        }

        if (is_string_type /* || insert other guessing here */ )
        {
                return new CL_StringFileResource(name, prefix, res_id, options, parent );
        }

        return NULL;
}

CL_Resource *CL_Res_String::try_create_from_datafile(CL_String name, CL_String prefix, CL_String res_id, CL_ResourceOptions *options, CL_ResourceManager *parent)
{
        bool is_string_type = false;

        if (options->exists("type"))
        {
                if (options->get_value("type") != "string") return NULL;
                is_string_type = true;
        }

        if (is_string_type /* || insert other guessing here */ )
        {
                return new CL_StringDatafileResource(name, prefix, res_id, options, parent );
        }

        return NULL;
}

CL_String CL_Res_String::load(CL_String res_id, CL_ResourceManager &manager, CL_String _default)
{
	CL_Resource *res = manager.get_resource(res_id);
	if( res==NULL ) 
		return _default;
        else
		return ((CL_WritableStringResource *) res)->create_string();
}

CL_String CL_Res_String::load(CL_String res_id, CL_ResourceManager &manager )
{
	CL_Resource *res = manager.get_resource(res_id);
	cl_assert( res!=NULL );

	return ((CL_WritableStringResource *) res)->create_string();
}


/******************************************************************************
						class CL_WritableStringResource
******************************************************************************/

CL_WritableStringResource::CL_WritableStringResource() : CL_Resource("string") 
{
	loaded = false;
	loaded_string = "";
}

void CL_WritableStringResource::write(CL_OutputSource *output)
{
	CL_String s = create_string();

	output->write_string( s );
}

bool CL_WritableStringResource::load()
{
	if (loaded) return true;

	loaded_string = create_string();
	loaded = true;
	return true;
}

/******************************************************************************
						class CL_StringFileResource
******************************************************************************/

CL_StringFileResource::CL_StringFileResource(CL_String _string_id, CL_String prefix, CL_String _string_location, CL_ResourceOptions *_options, CL_ResourceManager *_parent)
{
	name = _string_id;
	if (prefix == "") full_name = name; else full_name = prefix + "/" + name;
	string_location = _string_location;
	options = _options;
	parent = _parent;
}

CL_String CL_StringFileResource::create_string()
{
	loaded_string = string_location;
        return loaded_string;
}

/******************************************************************************
						class CL_StringDatafileResource
******************************************************************************/

CL_StringDatafileResource::CL_StringDatafileResource(
	CL_String _string_id,
	CL_String prefix,
	CL_String _string_location,
	CL_ResourceOptions *_options,
	CL_ResourceManager *_parent)
{
	name = _string_id;
	if (prefix == "") full_name = name; else full_name = prefix + "/" + name;
	string_location = _string_location;
	options = _options;
	parent = _parent;
}

CL_String CL_StringDatafileResource::create_string()
{
        if (parent->get_resource_provider() == NULL) return 0;

	CL_InputSource *input = parent->get_resource_provider()->open_source(full_name);
	if (input == NULL) return 0;

	return input->read_clanstring();
}

