/*
	$Id: wave_sample.h,v 1.4 1999/05/27 21:44:34 starch Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Simple sample support.

*/

//! l2_component="Static SoundProviders"

#ifndef header_sampleprovider
#define header_sampleprovider

#include <iostream.h>
#include "../../clanlib.h"

class CL_Sample : public CL_StaticSoundProvider
//: ClanLib's datafile samples. This class can read samples from datafiles.
{
	struct WAVE_FORMAT
	{
		short formatTag;
		short nChannels;
		int nSamplesPerSec;
		int nAvgBytesPerSec;
		short nBlockAlign;
	};

protected:
	CL_InputSourceProvider *provider;
	CL_String sample_id;

	char *sample_data;
	SoundFormat sample_format;
	int sample_size;
	int sample_freq;

	void load_data();

public:
	static CL_SoundBuffer *create(const char *sample_id, CL_InputSourceProvider *provider);
	//: Loads the sample 'sample_id' from the inputsource provider 'provider'. 
	//: Creates a CL_SoundBuffer using this sample and returns it.

	CL_Sample(const char *s_id, CL_InputSourceProvider *_provider);
	//: Constructs a static sound provider that represents 'sample_id' from the
	//: inputsource provider 'provider'.

	virtual ~CL_Sample();

	virtual void lock();
	//: Locks the sample data.

	virtual void unlock();
	//: Unlocks the sample data.

	virtual SoundFormat get_format() const;
	//: Returns the sample format used by this sample.

	virtual int data_size() const;
	//: Returns the size of the sample.

	virtual void *get_data() const;
	//: Returns the sample data. Must be locked before this is done.

	virtual int get_frequency() const;
	//: Returns the default frequency used when playing this sample.
};

#endif
