/*
	$Id: provider_convpoly.h,v 1.5 1999/08/23 18:41:46 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Header file to the ConvexPolygon surface provider.

*/

#ifndef header_provider_convpoly
#define header_provider_convpoly

#include "provider_convpoly_basics.h"

class CL_ConvexPolygon : public CL_ConvexPolygon_Basics
{

public:
	static CL_Surface *create(CL_SurfaceProvider *src, bool delete_provider=true)
	{
		return CL_Surface::create(new CL_ConvexPolygon(src, false), delete_provider);
	}

	CL_ConvexPolygon(CL_SurfaceProvider *_src, bool _delete_provider=false);
	virtual ~CL_ConvexPolygon();

	virtual int get_no_sprs() const { return src->get_no_sprs(); }
	virtual EPixelFormat get_pixel_format() const { return src->get_pixel_format(); }
	virtual CL_Palette *get_palette() const { return src->get_palette(); }
	virtual int get_transcol() const { return trans_col; }
	virtual void *get_surface_data() const { return surface_data; }

	virtual void perform_lock();
	virtual void perform_unlock();

protected:
	CL_SurfaceProvider *src;
	bool delete_provider;

	void *surface_data;
	int trans_col;

	// Shared locking data

	unsigned char *src_data;
	int src_pitch;
	int ysurf_pos;
	int xsurf_pos;
	int pitch;
	int width;
	int height;
	int *map_y_left;
	int *map_y_right;
	long *src_xd;
	long *src_yd;

	void create_mapping_deltas();
	void lock_8();
};

#endif
