/*
	$Id: provider_mpeg.cpp,v 1.2 1999/09/09 13:55:41 vogel Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		mpeg reader.

*/

#include "provider_mpeg.h"

CL_MPEGProvider::CL_MPEGProvider(const char *filename, int dithermode)
{
	mpeg = fopen(filename, "r");

	// currently only FULL_COLOR_DITHER
	SetMPEGOption(MPEG_DITHER, FULL_COLOR_DITHER);
	OpenMPEG(mpeg, &img);
	
	pixels = new char [img.Size];
	moreframes = true;
}

CL_MPEGProvider::~CL_MPEGProvider()
{
	delete[] pixels;
	fclose(mpeg);
}

void CL_MPEGProvider::rewind()
{
	moreframes = true;
	RewindMPEG(mpeg, &img);
}

int CL_MPEGProvider::get_framerate() const
{
	// new in 1.2.2
	return img.PictureRate;	
}

int CL_MPEGProvider::get_width() const
{
	return img.Width;
}

int CL_MPEGProvider::get_height() const
{
	return img.Height;
}

int CL_MPEGProvider::get_pitch() const
{
	// as BitmapPad is ignored in the calculation of size - why is it there???
	return img.Size / img.Height;
}

int CL_MPEGProvider::get_no_sprs() const
{
	return 1;
}

EPixelFormat CL_MPEGProvider::get_pixel_format() const
{
	return RGB888;
}

CL_Palette *CL_MPEGProvider::get_palette() const
{
	return NULL;
}

int CL_MPEGProvider::get_transcol() const
{
	return -1;
}

void *CL_MPEGProvider::get_surface_data() const
{
	return pixels;
}

void CL_MPEGProvider::perform_lock()
{
	if (moreframes) moreframes = GetMPEGFrame(pixels);
}

void CL_MPEGProvider::perform_unlock()
{
}

