/*
	$Id: provider_mpeg.h,v 1.2 1999/09/09 13:55:41 vogel Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		mpeg player using libmpeg.

*/

//! l2_component="SurfaceProviders"

#ifndef header_mpegprovider
#define header_mpegprovider

#include <stdio.h>
#include <mpeg.h>
#include "../../clanlib.h"
#include "generic_surfaceprovider.h"

class CL_MPEGProvider : public CL_SurfaceProvider_Generic
//: A surface provider that can show MPEG animations.
{
public:
	static CL_Surface *create(char *filename)
	{
		return CL_Surface::create(new CL_MPEGProvider(filename, FULL_COLOR_DITHER), true);
	}
	//: Creates a surface using the mpeg file 'filename' as source.
	//!param: filename - The mpeg file to use.

	static CL_Surface *create(char *filename, int dithermode)
	{
		return CL_Surface::create(new CL_MPEGProvider(filename, dithermode), true);
	}
	//: Creates a surface using the mpeg 'filename' as source and 
	//: dithermode passed to libmpeg.


	CL_MPEGProvider(const char *filename, int dithermode);
	//: Constructs a surfaceprovider using the mpeg file 'filename' as source, 
	//: passing flags to libmpeg.
	//!param: filename - mpeg file to use.
	//!param: dithermode - passed to libmpeg.
	
	virtual ~CL_MPEGProvider();

	virtual void rewind();
	//: Rewinds the mpeg stream
	
	virtual int get_framerate() const;
	//: Returns the framerate

	virtual int get_width() const;
	//: Returns the width of the image.
	
	virtual int get_height() const;
	//: Returns the height of the image.

	virtual int get_pitch() const;
	//: Returns the pitch of the Image.
	
	virtual int get_no_sprs() const;
	//: Returns the number of subsprites in the image.

	virtual EPixelFormat get_pixel_format() const;
	//: Returns the pixel format used by the image.
	
	virtual CL_Palette *get_palette() const;
	//: Returns the palette used by the image. NULL if system palette.
	
	virtual int get_transcol() const;
	//: Returns the transparency color used the image. -1 if none.

	virtual void *get_surface_data() const;
	//: Returns a pointer to the image data. Must be locked before data is valid.

	virtual void perform_lock();
	//: Locks the surface provider.
	
	virtual void perform_unlock();
	//: Unlocks the surface provider.

private:
	FILE *mpeg;
	ImageDesc img;
	char *pixels;
	bool moreframes;
};

#endif
