/*
	$Id: sprite2.cpp,v 1.1 1999/06/11 14:47:17 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Simple sprite2 support.

*/

#include <stdio.h>
#include "sprite2.h"

CL_Sprite2Provider::CL_Sprite2Provider(
	CL_InputSource *datafile)
{
	palette = NULL;
	surface_data = NULL;

	load_data(datafile);
}

CL_Sprite2Provider::CL_Sprite2Provider(
	CL_SurfaceProvider *src,
	int x, int y,
	int width, int height,
	int *tcols, int tcols_num)
{
	this->width = width;
	this->height = height;
	no_sprs = 1;
	if (tcols_num > 0) transcol = tcols[0];
	else transcol = -1;

	if (src->get_palette() != NULL)
		palette = new CL_Palette(
			src->get_palette()->palette,
			src->get_palette()->num_colors);

//	cout << "Palette: " << palette << endl;

	pixelformat = src->get_pixel_format();
//	cout << "Pixelformat: " << pixelformat << endl;

	// Quick-verify data:
	cl_assert(width > 0);
	cl_assert(height > 0);
	cl_assert(transcol >= -1);
	cl_assert(no_sprs > 0);
 
	int size_surface_data = width*height*no_sprs*get_bytes_pr_pixel();
	unsigned char *dest_data = new unsigned char[size_surface_data];
	surface_data = dest_data;

	src->lock();
	unsigned char *src_data = (unsigned char *) src->get_surface_data();
	
	bool transcol_table[256];
	for (int i=0; i<256; i++) transcol_table[i] = false;
	for (int j=0; j<tcols_num; j++) transcol_table[tcols[j]] = true;

	for (int spr_y=0; spr_y<height; spr_y++)
	{
		for (int spr_x=0; spr_x<width; spr_x++)
		{
			unsigned char val = src_data[
				x+spr_x+
				(y+spr_y)*src->get_pitch()];

			if (transcol_table[val])
			{
				dest_data[spr_x+spr_y*get_pitch()] = transcol;
			}
			else
			{
				dest_data[spr_x+spr_y*get_pitch()] = val;
			}

/*			if (spr_x == 0 || spr_y == 0 || spr_x+1 == width || spr_y+1 == height)
				dest_data[spr_x+spr_y*width] = 16;*/
		}
	}

	src->unlock();
}

CL_Sprite2Provider::~CL_Sprite2Provider()
{
	delete[] surface_data;
	surface_data = NULL;

	delete palette;
	palette = NULL;
}

int CL_Sprite2Provider::get_width() const
{
	return width;
}

int CL_Sprite2Provider::get_height() const
{
	return height;
}

int CL_Sprite2Provider::get_pitch() const
{
	return width*get_bytes_pr_pixel();
}

int CL_Sprite2Provider::get_no_sprs() const
{
	return no_sprs;
}

EPixelFormat CL_Sprite2Provider::get_pixel_format() const
{
	return pixelformat;
}

CL_Palette *CL_Sprite2Provider::get_palette() const
{
	return palette;
}

int CL_Sprite2Provider::get_transcol() const
{
	return transcol;
}

void *CL_Sprite2Provider::get_surface_data() const
{
	return surface_data;
}

void CL_Sprite2Provider::load_data(CL_InputSource *datafile)
{
	cl_assert(datafile != NULL);

	width = datafile->read_int32();
	height = datafile->read_int32();
	no_sprs = datafile->read_int32();
	transcol = datafile->read_int32();

	char has_palette = datafile->read_char8();
	if (has_palette)
	{
		unsigned char temp[768];
		datafile->read(temp, 768);
		palette = new CL_Palette(temp);
	}

	pixelformat = (EPixelFormat) datafile->read_int32();

	// Quick-verify data:
	cl_assert(width > 0);
	cl_assert(height > 0);
	cl_assert(transcol >= -1);
	cl_assert(no_sprs > 0);
 
	int size_surface_data = width*height*no_sprs*get_bytes_pr_pixel();
	surface_data = new unsigned char[size_surface_data];
	int read = datafile->read(surface_data, size_surface_data);
	cl_assert(read == size_surface_data);

//	delete datafile;
}

void CL_Sprite2Provider::perform_lock()
{
}

void CL_Sprite2Provider::perform_unlock()
{
}
