/*
	$Id: sprite2.h,v 1.1 1999/06/11 14:47:17 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Sprite surface provider especially designed for the font class.
		Most likely not very usable other places.

*/

//! l2_component="SurfaceProviders"

#ifndef header_sprite2
#define header_sprite2

#include "../../clanlib.h"
#include "generic_surfaceprovider.h"

class CL_Sprite2Provider : public CL_SurfaceProvider_Generic
{
protected:
	CL_String surface_id;
	void *surface_data;
	int width, height, no_sprs, transcol;
	CL_Palette *palette;
	EPixelFormat pixelformat;

	void load_data(CL_InputSource *datafile);

public:
	CL_Sprite2Provider(CL_InputSource *source);
	CL_Sprite2Provider(
		CL_SurfaceProvider *src,
		int x, int y, int width, int height,
		int *tcols, int tcols_num);
	virtual ~CL_Sprite2Provider();

	virtual int get_width() const;
	//: Returns the width of the sprite2.

	virtual int get_height() const;
	//: Returns the height of the sprite2.

	virtual int get_pitch() const;
	//: Returns the pitch of the sprite2.
	
	virtual int get_no_sprs() const;
	//: Returns the number of subsprite2s in this sprite2.
	
	virtual EPixelFormat get_pixel_format() const;
	//: Returns the pixel format used by the sprite2.

	virtual CL_Palette *get_palette() const;
	//: Returns the palette used by the surface. NULL if the system palette is 
	//: used.

	virtual int get_transcol() const;
	//: Returns the transparency color used, or -1 if none.
	
	virtual void *get_surface_data() const;
	//: Returns a pointer to the sprite2s image data. Can only be called between 
	//: lock() and unlock().

	virtual void perform_lock();
	//: Locks the surface provider. This causes the sprite2 to retrieve its 
	//: surface data from the datafile.
	
	virtual void perform_unlock();
	//: Unlocks the surface provider. The sprite2 releases its surface data when
	//: it is called.
};

#endif
