
#include <stdio.h>

#include "../../clanlib.h"
#include "../../clanlayer2.h"

#include "datafile_writer.h"

int main(int argc, char **argv)
{
	cout << "ClanLib Datafile Compiler" << endl;
	cout << "$Id: datafile_compiler.cpp,v 1.6 1999/08/09 17:53:18 mbn Exp $" << endl << endl;

	bool found_inputfile = false, found_outputfile = false;
	bool use_scriptfilename = false;

  	CL_String input_file("");
	CL_String output_file("");
	CL_String parameter("");

	for( int arg=1; arg < argc; arg++ )
	{
		parameter = argv[arg];

		// find out what parameter it is
		if( parameter=="--use_scriptfilename" )
			use_scriptfilename = true;

		else // must be filename
		{
			if( found_inputfile ) // must be outputfile name
			{
				output_file = parameter;
				found_outputfile = true;
			}
			else // must be inputfile name
			{
				input_file = parameter;
				found_inputfile = true;
			}
		}	
	}			
			
	if ( !found_inputfile || !found_outputfile )
	{
		cout << "Syntax: ClanCompiler <input_resource.scr> <output_datafile.dat> [--use_scriptfilename]" << endl << endl;
                cout << "--use_scriptfilename : Use the scriptfilename as name for the scriptfilesection, instead of the default name" << endl;
                cout << "                       Use this for backward compatibility." << endl;
		return 1;
	}

	try
	{
		CL_ResourceManager manager(input_file, NULL, true);

		cout << "Getting list of all resources" << endl; 
		CL_List<CL_String> *resource_list = manager.get_all_resources();
	
		cout << "Preparing datafile for writing..." << endl;
		OutputSourceProvider_Datafile output(output_file.get_string());

		// Write the config-file to the datafile
		{
			// open the section with either input_file or default_scriptfile_id
			CL_OutputSource *config_osource;

			if( use_scriptfilename )
			{
				config_osource = output.open_source(input_file);
				cout << "Using " << input_file << " as sectionname for scriptfile." << endl;
			}
			else // use default_scriptfile_id for id
			{
				config_osource = output.open_source(default_scriptfile_id);
				cout << "Using " << default_scriptfile_id << " as sectionname for scriptfile." << endl;
			}
	
			CL_InputSourceProvider *file_provider = CL_InputSourceProvider::create_file_provider("");
			CL_InputSource *config_file = file_provider->open_source(input_file);

			int total_size = config_file->size();
			char *config_file_data = new char[total_size];
			config_file->read(config_file_data, total_size);
			config_osource->write(config_file_data, total_size);

			delete file_provider;
			delete config_file;
			delete config_osource;
		}
	

		CL_Iterator<CL_String> counter(resource_list);
		while (counter.next() != NULL)
		{
			CL_OutputSource *osource = output.open_source(counter()->get_string());

			CL_Resource *res = manager.get_resource(*counter());
			if (res != NULL)
			{
				cout << "Writing resource '" << *counter() << "'" << endl;
				res->write(osource);
			}
			delete osource;
			delete counter();
		}
		delete resource_list;
	}
	catch (CL_Error err)
	{
		cout << "Error occoured: " << err.message << endl;
		
		remove (output_file.get_string());
	}

	return 0;
}
