/* $Id: mmio.h,v 1.2 1999/04/01 01:02:54 marcus Exp $
******************************************************************************

   LibGGI - Millennium II acceleration for fbdev target

   Copyright (C) 1999 Marcus Sundberg	[marcus@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

/* This file is included by m2164w.h */

static inline void
mga_out32(uint8 *mmioaddr, uint32 value, uint32 reg)
{
	*((uint32*)(mmioaddr+reg)) = value;
}

static inline uint32
mga_in32(uint8 *mmioaddr, uint32 reg)
{
	return *((uint32*)(mmioaddr+reg));
}


/* Wait for idle accelerator */
static inline void
mga_waitidle(uint8 *mmioaddr)
{
	int count = 0;

	while (mga_in32(mmioaddr, STATUS) & 0x10000) {
		if (! (count%1000)) {
			fprintf(stderr, "mga_waitidle: %d\n", count);
		}
		count++;
	}
}


/* Set the accelerator's foreground color */
static inline void
mga_setfg(uint8 *mmioaddr, ggi_mode *mode, ggi_pixel pixelfg)
{
	switch (GT_SIZE(mode->graphtype)) {
	case 8:
		pixelfg &= 0xff;   
		pixelfg |= (pixelfg << 8) |
			(pixelfg << 16) | (pixelfg << 24);
		break;
	case 16:
		pixelfg &= 0xffff;   
		pixelfg |= pixelfg << 16;
		break;
#ifdef GGI_LITTLE_ENDIAN
	case 24:
		pixelfg &= 0xffffff;
		break;
#endif
	case 32:
		pixelfg |= 0xff000000;
		break;
	default:
		return;
	}

	mga_out32(mmioaddr, pixelfg, FCOL);
}


/* Set the accelerator's clipping rectangle */
static inline void
mga_setclip(uint8 *mmioaddr, ggi_gc *gc, int virtx)
{
	mga_out32(mmioaddr, (gc->cliptl.x & 0x07FF)
		  | (((gc->clipbr.x-1) & 0x07FF) << 16), CXBNDRY);
	mga_out32(mmioaddr, (virtx * gc->cliptl.y)
		  & 0x00FFFFFF, YTOP);
	mga_out32(mmioaddr, (virtx * (gc->clipbr.y-1))
		  & 0x00FFFFFF, YBOT);
}
