/* $Id: visual.c,v 1.27 1999/02/25 15:06:29 marcus Exp $
******************************************************************************

   Display-monotext: visual management

   Copyright (C) 1998 Andrew Apted	[andrew@ggi-project.org]
   Copyright (C) 1999 Marcus Sundber	[marcus@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <ggi/display/monotext.h>


static gg_option options[] =
{
	{ "a",	"0" },
	{ "x",	"2" },
	{ "y",	"4" }
};
#define OPT_A		0
#define OPT_X		1
#define OPT_Y		2
#define NUM_OPTIONS	(sizeof(options)/sizeof(gg_option))


int GGIdlinit(ggi_visual *vis, const char *args, void *argptr)
{
	MonotextHook *priv;
	ggi_visual *parentvis;
	char target[1024] = "";
	int  val;

	GGIDPRINT("display-monotext: GGIdlinit start.\n");

	if (args) {
		args = ggParseOptions(args, options, NUM_OPTIONS);
		if (args == NULL) {
			fprintf(stderr,
				"display-monotext: error in arguments\n");
			return GGI_DL_ERROR;
		}
	}

	/* open the parent visual */
	GGIDPRINT("display-monotext: opening target: %s\n", args);
	
	if (args != NULL) {
		if (ggParseTarget((char *) args, target, 1024) == NULL) {
			/* error occurred */
			return GGI_DL_ERROR;
		}
	}

	if (*target == '\0') {
		strcpy(target, "auto");
	}

	parentvis = ggiOpen(target, NULL);
	if (parentvis == NULL) {
		fprintf(stderr,
			"display-monotext: Failed to open target: %s\n",
			target);
		return GGI_DL_ERROR;
	}
	ggiSetFlags(parentvis, GGIFLAG_ASYNC);

	priv = malloc(sizeof(MonotextHook));
	if (priv == NULL) {
		ggiClose(parentvis);
		return GGI_DL_ERROR;
	}

	LIBGGI_GC(vis) = malloc(sizeof(ggi_gc));
	if (LIBGGI_GC(vis) == NULL) {
		free(priv);
		ggiClose(parentvis);
		return GGI_DL_ERROR;
	}

	/* set defaults */
	priv->parent = parentvis;
	priv->parent_gt = GT_TEXT16;
	priv->flags = 0;
	priv->squish.x = priv->squish.y = 1;

	val = strtol(options[OPT_A].result, NULL, 0);
	if (val != 0) {
		priv->accuracy.x = priv->accuracy.y = val;
	} else {
		priv->accuracy.x = strtol(options[OPT_X].result, NULL, 0);
		priv->accuracy.y = strtol(options[OPT_Y].result, NULL, 0);
	}

	/* add giiInputs, if we have them */
	if (priv->parent->input) {
		vis->input = giiJoinInputs(vis->input, priv->parent->input);
		priv->parent->input = NULL; /* destroy old reference */
	}
	
	LIBGGI_PRIVATE(vis) = priv;

	vis->opdisplay->getmode   = GGI_monotext_getmode;
	vis->opdisplay->setmode   = GGI_monotext_setmode;
	vis->opdisplay->checkmode = GGI_monotext_checkmode;
	vis->opdisplay->getapi    = GGI_monotext_getapi;
	vis->opdisplay->flush     = GGI_monotext_flush;
	vis->opdisplay->setflags  = GGI_monotext_setflags;
	
	GGIDPRINT("display-monotext: GGIdlinit succeeded.\n");

	return GGI_DL_OPDISPLAY;
}

int GGIdlcleanup(ggi_visual *vis)
{
	MonotextHook *priv = LIBGGI_PRIVATE(vis);

	GGIDPRINT("display-monotext: GGIdlcleanup start.\n");

	if (priv->fb_ptr != NULL) {
		_ggi_monotextClose(vis);
		free(priv->fb_ptr);
	}

	if (priv->parent != NULL) {
		ggiClose(priv->parent);

		giiClose(vis->input);
		vis->input = NULL;
	}

	free(priv);
	free(LIBGGI_GC(vis));

	GGIDPRINT("display-monotext: GGIdlcleanup done.\n");

	return 0;
}

#include <ggi/internal/ggidlinit.h>
