/*	MikMod sound library
	(c) 1998, 1999 Miodrag Vallat and others - see file AUTHORS for
	complete list.

	This library is free software; you can redistribute it and/or modify
	it under the terms of the GNU Library General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.
 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Library General Public License for more details.
 
	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/

/*==============================================================================

  $Id: drv_sun.c,v 1.30 1999/10/25 16:31:41 miod Exp $

  Driver for output on the Sun audio device (/dev/audio).
  Also works under NetBSD and OpenBSD

==============================================================================*/

/*

	Written by Valtteri Vuorikoski <vuori@sci.fi>

*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#include <stdlib.h>
#if !defined(SUNOS)
#include <string.h>
#endif
#ifdef HAVE_SYS_IOCTL_H
#include <sys/ioctl.h>
#endif
#include <sys/types.h>

#ifdef HAVE_SUN_AUDIOIO_H
#include <sun/audioio.h>
#endif
#ifdef HAVE_SYS_AUDIOIO_H
#include <sys/audioio.h>
#endif

#include "mikmod_internals.h"

#define DEFAULT_FRAGSIZE 12

#if !defined(__NetBSD__) && !defined(__OpenBSD__) && !defined(SUNOS)
#define SOLARIS
#endif

static	int sndfd=-1;
#if defined(SUNOS)||defined(SOLARIS)
static unsigned int port = AUDIO_SPEAKER;
#endif
static	int play_encoding;
static	int play_precision;
static	int fragsize=1<<DEFAULT_FRAGSIZE;
static	SBYTE *audiobuffer=NULL;

/* Sound device to open */
#ifdef SUNOS
#define SOUNDDEVICE "/dev/sound"
#else /* Solaris, *BSD */
#define SOUNDDEVICE "/dev/audio"
#endif

/* Compatibility defines, for old *BSD or SunOS systems */
#ifndef AUDIO_ENCODING_PCM16
#define AUDIO_ENCODING_PCM16 AUDIO_ENCODING_LINEAR
#endif
#ifndef AUDIO_ENCODING_PCM8
#define AUDIO_ENCODING_PCM8 AUDIO_ENCODING_LINEAR8
#endif
#ifndef AUDIO_ENCODING_SLINEAR_LE
#define AUDIO_ENCODING_SLINEAR_LE AUDIO_ENCODING_PCM16
#endif
#ifndef AUDIO_ENCODING_ULINEAR_LE
#define AUDIO_ENCODING_ULINEAR_LE AUDIO_ENCODING_PCM8
#endif

static void Sun_CommandLine(CHAR *cmdline)
{
	CHAR *ptr;

	if((ptr=MD_GetAtom("buffer",cmdline,0))) {
		int buf=atoi(ptr);

		if((buf<7)||(buf>17)) buf=DEFAULT_FRAGSIZE;
		fragsize=1<<buf;

		free(ptr);
	}
#if defined(SUNOS)||defined(SOLARIS)
	if((ptr=MD_GetAtom("headphone",cmdline,1))) {
		port = AUDIO_HEADPHONE;
		free(ptr);
	} else if ((ptr = MD_GetAtom("speaker", cmdline, 1))) {
		port = AUDIO_SPEAKER;
		free(ptr);
	}
#endif
}

static BOOL Sun_IsThere(void)
{
	return (access(getenv("AUDIODEV")?getenv("AUDIODEV"):SOUNDDEVICE,W_OK)==0);
}

static BOOL Sun_Init(void)
{
	int play_stereo,play_rate;
#ifdef SUNOS
	int audiotype;
#else
	audio_device_t audiotype;	
#endif
	struct audio_info audioinfo;
	char *env;

	if((sndfd=open(getenv("AUDIODEV")?getenv("AUDIODEV"):SOUNDDEVICE,O_WRONLY))<0) {
		_mm_errno=MMERR_OPENING_AUDIO;
		return 1;
	}

	if (fragsize==DEFAULT_FRAGSIZE) {
		if((env=getenv("MM_FRAGSIZE"))) fragsize=atoi(env);
		if((fragsize<(1<<7))||(fragsize>(1<<17))) fragsize=1<<DEFAULT_FRAGSIZE;
	}

	if(!(audiobuffer=(SBYTE*)_mm_malloc(fragsize)))
		return 1;

	play_precision=(md_mode&DMODE_16BITS)?16:8;
	play_stereo=(md_mode&DMODE_STEREO)?2:1;
	play_rate=md_mixfreq;
	/* attempt to guess the encoding */
	play_encoding=-1;

	if(ioctl(sndfd,AUDIO_GETDEV,&audiotype)<0) {
#ifdef MIKMOD_DEBUG
		fputs("\rSun driver warning: could not determine audio device type\n",stderr);
#endif
	} else {
#if defined(SUNOS) /* SunOS 4 */
		switch(audiotype) {
			case AUDIO_DEV_AMD:
				/* AMD 79C30 */
				/* 8bit mono ulaw 8kHz */
				if((play_precision==8)&&(play_stereo==1)&&(play_rate==8000))
					play_encoding=AUDIO_ENCODING_ULAW;
				else {
					_mm_errno=MMERR_SUN_INIT;
					return 1;
				}
				break;
			case AUDIO_DEV_SPEAKERBOX:
			case AUDIO_DEV_CODEC:
				/* CS 4231 or DBRI or speaker box */
				/* 16bit mono/stereo linear 8kHz - 48kHz */
				if(play_precision==16)
					play_encoding=AUDIO_ENCODING_LINEAR;
				/* 8bit mono ulaw 8kHz - 48kHz */
				else if((play_precision==8)&&(play_stereo==1))
					play_encoding=AUDIO_ENCODING_ULAW;
				else {
					_mm_errno=MMERR_SUN_INIT;
					return 1;
				}
				break;
		}
#elif defined(SOLARIS) /* Solaris */
		if (!strcmp(audiotype.name,"SUNW,am79c30")) {
			/* AMD 79C30 */
			/* 8bit mono ulaw 8kHz */
			if((play_precision==8)&&(play_stereo==1)&&(play_rate==8000))
				play_encoding=AUDIO_ENCODING_ULAW;
			else {
				_mm_errno=MMERR_SUN_INIT;
				return 1;
			}
		} else
		  if ((!strcmp(audiotype.name,"SUNW,CS4231"))||
		      (!strcmp(audiotype.name,"SUNW,dbri"))||
		      (!strcmp(audiotype.name,"speakerbox"))) {
			/* CS 4231 or DBRI or speaker box */
			/* 16bit mono/stereo linear 8kHz - 48kHz */
			if(play_precision==16)
				play_encoding=AUDIO_ENCODING_LINEAR;
			/* 8bit mono ulaw 8kHz - 48kHz */
			else if((play_precision==8)&&(play_stereo==1))
				play_encoding=AUDIO_ENCODING_ULAW;
			else {
				_mm_errno=MMERR_SUN_INIT;
				return 1;
			}
		}
#else /* NetBSD, OpenBSD */
		if (!strcmp(audiotype.name,"amd7930")) {
			/* AMD 79C30 */
			/* 8bit mono ulaw 8kHz */
			if((play_precision==8)&&(play_stereo==1)&&(play_rate==8000))
				play_encoding=AUDIO_ENCODING_ULAW;
			else {
				_mm_errno=MMERR_SUN_INIT;
				return 1;
			}
		}
		if(!strcmp(audiotype.name,"Am78C201")) {
			/* AMD Interwave */
			/* 16bit stereo linear 44kHz */
			if((play_precision!=8)||(play_stereo!=2)||(play_rate!=44100)) {
				_mm_errno=MMERR_SUN_INIT;
				return 1;
			} else
				play_encoding=AUDIO_ENCODING_SLINEAR_LE;
		}
#endif
	}

	/* Sound devices which were not handled above don't have specific
	   limitations, so try and guess optimal settings */
	if(play_encoding==-1) {
		if((play_precision==8)&&(play_stereo==1)&&(play_rate<=8000))
			play_encoding = AUDIO_ENCODING_ULAW;
		else
#ifdef SUNOS
			play_encoding=AUDIO_ENCODING_LINEAR;
#else
			play_encoding=(play_precision==16)?AUDIO_ENCODING_SLINEAR_LE:
			                                   AUDIO_ENCODING_ULINEAR_LE;
#endif
	}

	AUDIO_INITINFO(&audioinfo);

	/* get current audio settings */
	if (ioctl(sndfd, AUDIO_GETINFO, &audioinfo) < 0) {
		_mm_errno = MMERR_SUN_INIT;
		return 1;
	}

	audioinfo.play.precision=play_precision;
	audioinfo.play.channels=play_stereo;
	audioinfo.play.sample_rate=play_rate;
	audioinfo.play.encoding=play_encoding;
#if defined(SUNOS)||defined(SOLARIS)
	/* only change the output device if requested */
	if (port)
		audioinfo.play.port = port;
#endif
#if defined(__NetBSD__)||defined(__OpenBSD__)
	audioinfo.mode=AUMODE_PLAY;
#endif
	
	if(ioctl(sndfd,AUDIO_SETINFO,&audioinfo)<0) {
		_mm_errno=MMERR_SUN_INIT;
		return 1;
	}

	/* Check if our changes were accepted */
	if(ioctl(sndfd,AUDIO_GETINFO,&audioinfo)<0) {
		_mm_errno=MMERR_SUN_INIT;
		return 1;
	}
	if ((audioinfo.play.precision!=play_precision)||
	    (audioinfo.play.channels!=play_stereo)||
	    (audioinfo.play.encoding!=play_encoding)||
	    (audioinfo.play.sample_rate!=play_rate)) {
		_mm_errno=MMERR_SUN_INIT;
		return 1;
	}

	return VC_Init();
}

static void Sun_Exit(void)
{
	VC_Exit();
	_mm_free(audiobuffer);
	if (sndfd>=0) {
		close(sndfd);
		sndfd=-1;
	}
}

static void Sun_Update(void)
{
	int done;

	done=VC_WriteBytes((signed char*)audiobuffer,fragsize);
	if(play_encoding==AUDIO_ENCODING_ULAW) unsignedtoulaw(audiobuffer,done);
	write(sndfd,audiobuffer,done);
}

static void Sun_Pause(void)
{
	int done;

	done=VC_SilenceBytes((signed char*)audiobuffer,fragsize);
	write(sndfd,audiobuffer,done);
}

static BOOL Sun_Reset(void)
{
	Sun_Exit();
	return Sun_Init();
}

static BOOL Sun_PlayStart(void)
{
	struct audio_info audioinfo;

	if(ioctl(sndfd,AUDIO_GETINFO,&audioinfo)<0)
		return 1;
	audioinfo.play.pause=0;
	if(ioctl(sndfd,AUDIO_SETINFO,&audioinfo)<0)
		return 1;

	return VC_PlayStart();
}

static void Sun_PlayStop(void)
{
	struct audio_info audioinfo;

	VC_PlayStop();

	if(ioctl(sndfd,AUDIO_DRAIN)<0)
		return;
	if(ioctl(sndfd,AUDIO_GETINFO,&audioinfo)<0)
		return;
	audioinfo.play.pause=1;
	ioctl(sndfd,AUDIO_SETINFO,&audioinfo);
}

MDRIVER drv_sun={
	NULL,
#if defined(__OpenBSD__)
	"OpenBSD Audio",
	"OpenBSD audio driver v0.4",
#elif defined(__NetBSD__)
	"NetBSD Audio",
	"NetBSD audio driver v0.4",
#elif defined(SUNOS)
	"Sun Audio",
	"Sun audio driver v1.3",
#elif defined(SOLARIS)
	"Solaris Audio",
	"Solaris audio driver v1.3",
#endif
	0,255,
	"audio",

	Sun_CommandLine,
	Sun_IsThere,
	VC_SampleLoad,
	VC_SampleUnload,
	VC_SampleSpace,
	VC_SampleLength,
	Sun_Init,
	Sun_Exit,
	Sun_Reset,
	VC_SetNumVoices,
	Sun_PlayStart,
	Sun_PlayStop,
	Sun_Update,
	Sun_Pause,
	VC_VoiceSetVolume,
	VC_VoiceGetVolume,
	VC_VoiceSetFrequency,
	VC_VoiceGetFrequency,
	VC_VoiceSetPanning,
	VC_VoiceGetPanning,
	VC_VoicePlay,
	VC_VoiceStop,
	VC_VoiceStopped,
	VC_VoiceGetPosition,
	VC_VoiceRealVolume
};

/* ex:set ts=4: */
