
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef WIN32
#include <windows.h>
#else
#include <unistd.h>
#endif
#include <math.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include "pu.h"

puSlider *hs ;
puButtonBox *bb ;

GLfloat light_diffuse[] = {1.0, 0.0, 0.0, 1.0};  /* Red diffuse light. */
GLfloat light_position[] = {1.0, 1.0, 1.0, 0.0};  /* Infinite light location. */
GLfloat n[6][3] = {  /* Normals for the 6 faces of a cube. */
  {-1.0, 0.0, 0.0}, {0.0, 1.0, 0.0}, {1.0, 0.0, 0.0},
  {0.0, -1.0, 0.0}, {0.0, 0.0, 1.0}, {0.0, 0.0, -1.0} };
GLint faces[6][4] = {  /* Vertex indices for the 6 faces of a cube. */
  {0, 1, 2, 3}, {3, 2, 6, 7}, {7, 6, 5, 4},
  {4, 5, 1, 0}, {5, 6, 2, 1}, {7, 4, 0, 3} };
GLfloat v[8][3];  /* Will be filled in with X,Y,Z vertexes. */
 void
drawBox(void)
{
  int i;

  for (i = 0; i < 6; i++) {
    glBegin(GL_QUADS);
    glNormal3fv(&n[i][0]);
    glVertex3fv(&v[faces[i][0]][0]);
    glVertex3fv(&v[faces[i][1]][0]);
    glVertex3fv(&v[faces[i][2]][0]);
    glVertex3fv(&v[faces[i][3]][0]);
    glEnd();
  }
}
void init(void)
{
  /* Setup cube vertex data. */
  v[0][0] = v[1][0] = v[2][0] = v[3][0] = -1;
  v[4][0] = v[5][0] = v[6][0] = v[7][0] = 1;
  v[0][1] = v[1][1] = v[4][1] = v[5][1] = -1;
  v[2][1] = v[3][1] = v[6][1] = v[7][1] = 1;
  v[0][2] = v[3][2] = v[4][2] = v[7][2] = 1;
  v[1][2] = v[2][2] = v[5][2] = v[6][2] = -1;

  /* Enable a single OpenGL light. */
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
  glLightfv(GL_LIGHT0, GL_POSITION, light_position);
  glEnable(GL_LIGHT0);
  glEnable(GL_LIGHTING);

  /* Use depth buffering for hidden surface elimination. */
  glEnable(GL_DEPTH_TEST);

  /* Setup the view of the cube. */
  glMatrixMode(GL_PROJECTION);
  gluPerspective( /* field of view in degree */ 40.0,    /* aspect ratio */ 1.0,
    /* Z near */ 1.0, /* Z far */ 10.0);
  glMatrixMode(GL_MODELVIEW);
  gluLookAt(0.0, 0.0, 5.0,  /* eye is at (0,0,5) */
    0.0, 0.0, 0.0,      /* center is at (0,0,0) */
    0.0, 1.0, 0.);      /* up is in positive Y direction */

  /* Adjust cube position to be asthetic angle. */
  glTranslatef(0.0, 0.0, -1.0);
  glRotatef(60, 1.0, 0.0, 0.0);
}
 
static void exit_menu ( int value )
{
  switch ( value )
  {
    case 1:
      exit ( 0 ) ;
  }

  glutPostRedisplay () ;
}



static void specialfn ( int key, int, int )
{
  puKeyboard ( key + PU_KEY_GLUT_SPECIAL_OFFSET, PU_DOWN ) ;
  glutPostRedisplay () ;
}

static void keyfn ( unsigned char key, int, int )
{
  puKeyboard ( key, PU_DOWN ) ;
  glutPostRedisplay () ;
}

static void motionfn ( int x, int y )
{
  puMouse ( x, y ) ;
  glutPostRedisplay () ;
}

static void mousefn ( int button, int updown, int x, int y )
{
  puMouse ( button, updown, x, y ) ;
  glutPostRedisplay () ;
}

puPopupMenu *p ;

static void my_cb5 ( puObject *hs )
{
  float val ;
  hs -> getValue ( &val ) ;
  printf ( "%f...\n", val ) ;
}

static void my_cb4 ( puObject *ob )
{
  if ( ob -> getValue () )
    p -> hide () ;
  else
    p -> reveal () ;
}

static void my_cb ( puObject * )
{
  fprintf ( stderr, "Thanks!\n" ) ;
/*
  bb -> setValue ( 1 ) ;
*/
}

static void displayfn (void)
{
  glClearColor ( 0.1, 0.4, 0.1, 1.0 ) ;
  glClear      ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ) ;

  float val ; hs->getValue ( &val ) ;

  glRotatef( val * 20.0f, 0.0, 0.0, 1.0 );
  drawBox() ;

  puDisplay () ;
  
/*
  bb->setValue(bb->getValue()+1);
*/
  glutSwapBuffers   () ;
  glutPostRedisplay () ;
}

void null_cb ( puObject *ob )
{
  fprintf ( stderr, "%s not implemented yet.\n", ob -> getLegend() ) ;
}

void exit_cb ( puObject * )
{
  exit ( 1 ) ;
}

char *file_submenu [] =
{
  "Exit", "Close", "--------", "Print", "--------", "SaveAs...", "Save", "New", NULL
} ;

puCallback file_submenu_cb [] =
{
  exit_cb, null_cb, NULL, null_cb, NULL, null_cb, null_cb, null_cb, NULL
} ;

char *help_submenu [] =
{
  "About...", "Help", NULL
} ;

puCallback help_submenu_cb [] =
{
  null_cb, null_cb, NULL
} ;

int main ( int argc, char **argv )
{
#ifdef VOODOO
  glutInitWindowPosition ( 0, 0 ) ;
#endif
  glutInitWindowSize    ( 640, 480 ) ;
  glutInit              ( &argc, argv ) ;
  glutInitDisplayMode   ( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH ) ;
  glutCreateWindow      ( "PUI Application"  ) ;
  glutDisplayFunc       ( displayfn ) ;
  glutKeyboardFunc      ( keyfn     ) ;
  glutSpecialFunc       ( specialfn ) ;
  glutMouseFunc         ( mousefn   ) ;
  glutPassiveMotionFunc ( motionfn  ) ;
  glutMotionFunc        ( motionfn  ) ;
  glutCreateMenu        ( exit_menu ) ;
  glutAddMenuEntry      ( "Exit", 1 ) ;
  glutAttachMenu        ( GLUT_RIGHT_BUTTON ) ;

  init () ;
  puInit () ;

#ifdef VOODOO
  puShowCursor () ;
#endif

  puFrame *f = new puFrame ( 20, 20, 220, 100 ) ;
  f->setLegend    ( "PicoGUI Test Suite" ) ;
  f->setColourScheme( 0.6, 0.2, 0.2 ) ;

  hs = new puSlider ( 50, 200, 250, TRUE ) ;
  hs->setDelta ( 0.1 ) ;
  hs->setCBMode ( PUSLIDER_DELTA ) ;
  hs->setCallback ( my_cb5 ) ;

  puInput *inp = new puInput ( 50, 150, 300, 180 ) ;
  inp->setLegend    ( "Legend" ) ;
  inp->setValue     ( "Hello World." ) ;
  inp->setLabel     ( "<=="   ) ;
  inp->acceptInput  () ;
  inp->setCursor ( 3 ) ;
  inp->setSelectRegion ( 5, 9 ) ;

  char *labs [] = { "Pop Up", "Pop Down", "Don't Pop", NULL } ;
  bb = new puButtonBox ( 50, 50, 160, 120, labs, TRUE ) ;
  bb->setCallback ( my_cb4 ) ;

  p = new puPopupMenu ( 170, 50 ) ;
  p -> add_item ( "Style Change", my_cb ) ;
  p -> add_item ( "Another Style", my_cb ) ;
  p -> add_item ( "Yet Another Style", my_cb ) ;
  p -> close   () ;

  puMenuBar *menu = new puMenuBar () ;
  menu->add_submenu ( "File", file_submenu, file_submenu_cb ) ;
  menu->add_submenu ( "Help", help_submenu, help_submenu_cb ) ;
  menu->close () ; 
  glutMainLoop () ;
  return 0 ;
}


