////////////////////////////////////////////////////////
//
// drawStereo
//
// mdanks@cybermed.ucsd.edu
//
// Implementation file
//
//    Copyright (c) 1998 Mark Danks.
//    For information on usage and redistribution, and for a DISCLAIMER OF ALL
//    WARRANTIES, see the file, "LICENSE.TERMS" in this distribution.
//
/////////////////////////////////////////////////////////

#include "EventProc.h"
#include <stdlib.h>

/////////////////////////////////////////////////////////
// The callbacks
//
/////////////////////////////////////////////////////////
struct CallbackList
{
    CallbackList() : data(NULL), func(NULL), next(NULL), window(NULL) {}
    void *data;
    void *func;
	HWND window;
    CallbackList *next;
};

static CallbackList *s_motionList = NULL;
static CallbackList *s_buttonList = NULL;
static CallbackList *s_keyboardList = NULL;
static CallbackList *s_resizeList = NULL;
static CallbackList *s_paintList = NULL;
static CallbackList *s_closeList = NULL;

/////////////////////////////////////////////////////////
// Motion callbacks
//
/////////////////////////////////////////////////////////
void setMotionCallback(void (*callback)(int, int, float, void *), void *data, HWND window)
{
    CallbackList *newCallback = new CallbackList;
    newCallback->data = data;
    newCallback->func = (void *)callback;
	newCallback->window = window;
    if (!s_motionList)
        s_motionList = newCallback;
    else
    {
        CallbackList *theList = s_motionList;
        while(theList->next)
            theList = theList->next;
        theList->next = newCallback;
    }
}
void removeMotionCallback(void (*callback)(int, int, float, void *), void *data)
{
    CallbackList *theList = s_motionList;
    if (!theList)
        return;
    else if (theList->func == (void *)callback &&
             theList->data == data)
    {
        s_motionList = theList->next;
        delete theList;
    }
    else
    {
        while(theList->next)
        {
            if (theList->next->func == (void *)callback &&
                theList->next->data == data)
            {
                CallbackList *holder = theList->next;
                theList->next = holder->next;
                delete holder;
                return;
            }
            theList = theList->next;
        }
    }
}
/////////////////////////////////////////////////////////
// Button callbacks
//
/////////////////////////////////////////////////////////
void setButtonCallback(void (*callback)(int, int, int, int, void *), void *data, HWND window)
{
    CallbackList *newCallback = new CallbackList;
    newCallback->data = data;
    newCallback->func = (void *)callback;
	newCallback->window = window;

    if (!s_buttonList)
        s_buttonList = newCallback;
    else
    {
        CallbackList *theList = s_buttonList;
        while(theList->next)
            theList = theList->next;
        theList->next = newCallback;
    }
}
void removeButtonCallback(void (*callback)(int, int, int, int, void *), void *data)
{
    CallbackList *theList = s_buttonList;
    if (!theList)
        return;
    else if (theList->func == (void *)callback &&
             theList->data == data)
    {
        s_buttonList = theList->next;
        delete theList;
    }
    else
    {
        while(theList->next)
        {
            if (theList->next->func == (void *)callback &&
                theList->next->data == data)
            {
                CallbackList *holder = theList->next;
                theList->next = holder->next;
                delete holder;
                return;
            }
            theList = theList->next;
        }
    }
}
/////////////////////////////////////////////////////////
// Keyboard callbacks
//
/////////////////////////////////////////////////////////
void setKeyboardCallback(void (*callback)(char, int, void *), void *data, HWND window)
{
    CallbackList *newCallback = new CallbackList;
    newCallback->data = data;
    newCallback->func = (void *)callback;
	newCallback->window = window;

    if (!s_keyboardList)
        s_keyboardList = newCallback;
    else
    {
        CallbackList *theList = s_keyboardList;
        while(theList->next)
            theList = theList->next;
        theList->next = newCallback;
    }
}
void removeKeyboardCallback(void (*callback)(char, int, void *), void *data)
{
    CallbackList *theList = s_keyboardList;
    if (!theList)
        return;
    else if (theList->func == (void *)callback &&
             theList->data == data)
    {
        s_keyboardList = theList->next;
        delete theList;
    }
    else
    {
        while(theList->next)
        {
            if (theList->next->func == (void *)callback &&
                theList->next->data == data)
            {
                CallbackList *holder = theList->next;
                theList->next = holder->next;
                delete holder;
                return;
            }
            theList = theList->next;
        }
    }
}
/////////////////////////////////////////////////////////
// Resize callbacks
//
/////////////////////////////////////////////////////////
void setResizeCallback(void (*callback)(int, int, void *), void *data, HWND window)
{
    CallbackList *newCallback = new CallbackList;
    newCallback->data = data;
    newCallback->func = (void *)callback;
	newCallback->window = window;

    if (!s_resizeList)
        s_resizeList = newCallback;
    else
    {
        CallbackList *theList = s_resizeList;
        while(theList->next)
            theList = theList->next;
        theList->next = newCallback;
    }
}
void removeResizeCallback(void (*callback)(int, int, void *), void *data)
{
    CallbackList *theList = s_resizeList;
    if (!theList)
        return;
    else if (theList->func == (void *)callback &&
             theList->data == data)
    {
        s_resizeList = theList->next;
        delete theList;
    }
    else
    {
        while(theList->next)
        {
            if (theList->next->func == (void *)callback &&
                theList->next->data == data)
            {
                CallbackList *holder = theList->next;
                theList->next = holder->next;
                delete holder;
                return;
            }
            theList = theList->next;
        }
    }
}
/////////////////////////////////////////////////////////
// Paint callbacks
//
/////////////////////////////////////////////////////////
void setPaintCallback(void (*callback)(void *), void *data, HWND window)
{
    CallbackList *newCallback = new CallbackList;
    newCallback->data = data;
    newCallback->func = (void *)callback;
	newCallback->window = window;

    if (!s_paintList)
        s_paintList = newCallback;
    else
    {
        CallbackList *theList = s_paintList;
        while(theList->next)
            theList = theList->next;
        theList->next = newCallback;
    }
}
void removePaintCallback(void (*callback)(void *), void *data)
{
    CallbackList *theList = s_paintList;
    if (!theList)
        return;
    else if (theList->func == (void *)callback &&
             theList->data == data)
    {
        s_paintList = theList->next;
        delete theList;
    }
    else
    {
        while(theList->next)
        {
            if (theList->next->func == (void *)callback &&
                theList->next->data == data)
            {
                CallbackList *holder = theList->next;
                theList->next = holder->next;
                delete holder;
                return;
            }
            theList = theList->next;
        }
    }
}
/////////////////////////////////////////////////////////
// Close callbacks
//
/////////////////////////////////////////////////////////
void setCloseCallback(void (*callback)(void *), void *data, HWND window)
{
    CallbackList *newCallback = new CallbackList;
    newCallback->data = data;
    newCallback->func = (void *)callback;
	newCallback->window = window;

    if (!s_closeList)
        s_closeList = newCallback;
    else
    {
        CallbackList *theList = s_closeList;
        while(theList->next)
            theList = theList->next;
        theList->next = newCallback;
    }
}
void removeCloseCallback(void (*callback)(void *), void *data)
{
    CallbackList *theList = s_closeList;
    if (!theList)
        return;
    else if (theList->func == (void *)callback &&
             theList->data == data)
    {
        s_closeList = theList->next;
        delete theList;
    }
    else
    {
        while(theList->next)
        {
            if (theList->next->func == (void *)callback &&
                theList->next->data == data)
            {
                CallbackList *holder = theList->next;
                theList->next = holder->next;
                delete holder;
                return;
            }
            theList = theList->next;
        }
    }
}

/////////////////////////////////////////////////////////
// Trigger events
//
/////////////////////////////////////////////////////////
void triggerMotionEvent(int x, int y, float pressure, HWND window)
{
    CallbackList *theList = s_motionList;
    while(theList)
    {
        void (*callback)(int, int, float, void *) = (void (*)(int, int, float, void *))theList->func;
		if (theList->window == NULL || theList->window == window)
			(*callback)(x, y, pressure, theList->data);
        theList = theList->next;
    }
}
void triggerButtonEvent(int which, int state, int x, int y, HWND window)
{
    CallbackList *theList = s_buttonList;
    while(theList)
    {
        void (*callback)(int, int, int, int, void *) = (void (*)(int, int, int, int, void *))theList->func;
		if (theList->window == NULL || theList->window == window)
			(*callback)(which, state, x, y, theList->data);
        theList = theList->next;
    }
}
void triggerKeyboardEvent(char value, int state, HWND window)
{
    CallbackList *theList = s_keyboardList;
    while(theList)
    {
        void (*callback)(char, int, void *) = (void (*)(char, int, void *))theList->func;
		if (theList->window == NULL || theList->window == window)
			(*callback)(value, state, theList->data);
        theList = theList->next;
    }
}
void triggerResizeEvent(int xSize, int ySize, HWND window)
{
    CallbackList *theList = s_resizeList;
    while(theList)
    {
        void (*callback)(int, int, void *) = (void (*)(int, int, void *))theList->func;
		if (theList->window == NULL || theList->window == window)
			(*callback)(xSize, ySize, theList->data);
        theList = theList->next;
    }
}
void triggerPaintEvent(HWND window)
{
    CallbackList *theList = s_paintList;
    while(theList)
    {
        void (*callback)(void *) = (void (*)(void *))theList->func;
		if (theList->window == NULL || theList->window == window)
			(*callback)(theList->data);
        theList = theList->next;
    }
}
void triggerCloseEvent(HWND window)
{
    CallbackList *theList = s_closeList;
    while(theList)
    {
        void (*callback)(void *) = (void (*)(void *))theList->func;
		if (theList->window == NULL || theList->window == window)
			(*callback)(theList->data);
        theList = theList->next;
    }
}
