/*-----------------------------------------------------------------
LOG
    drawStereo

    Event handling

    Copyright (c) 1998 Mark Danks.
    For information on usage and redistribution, and for a DISCLAIMER OF ALL
    WARRANTIES, see the file, "LICENSE.TERMS" in this distribution.

-----------------------------------------------------------------*/

#ifndef INCLUDE_EVENTPROC_H_
#define INCLUDE_EVENTPROC_H_

#ifdef WIN32
#include <windows.h>
#else
typedef void * HWND;
#endif

//////////
// Set a mouse motion callback
extern void setMotionCallback(void (*callback)(int x, int y, float pressure, void *), void *data, HWND window = NULL);

//////////
// Remove a mouse motion callback
extern void removeMotionCallback(void (*callback)(int x, int y, float pressure, void *), void *data);

//////////
// Set a button callback
// which == 0 == left
// which == 1 == middle
// which == 2 == right
extern void setButtonCallback(void (*callback)(int which, int state, int x, int y, void *), void *data, HWND window = NULL);

//////////
// Remove a button callback
extern void removeButtonCallback(void (*callback)(int which, int state, int x, int y, void *), void *data);

//////////
// Set a keyboard callback
extern void setKeyboardCallback(void (*callback)(char value, int state, void *), void *data, HWND window = NULL);

//////////
// Remove a keyboard callback
extern void removeKeyboardCallback(void (*callback)(char value, int state, void *), void *data);

//////////
// Set a resize callback
extern void setResizeCallback(void (*callback)(int x, int y, void *), void *data, HWND window = NULL);

//////////
// Remove a resize callback
extern void removeResizeCallback(void (*callback)(int x, int y, void *), void *data);

//////////
// Set a paint callback
extern void setPaintCallback(void (*callback)(void *), void *data, HWND window = NULL);

//////////
// Remove a paint callback
extern void removePaintCallback(void (*callback)(void *), void *data);

//////////
// Set a close callback
extern void setCloseCallback(void (*callback)(void *), void *data, HWND window = NULL);

//////////
// Remove a close callback
extern void removeCloseCallback(void (*callback)(void *), void *data);

//////////
// Trigger an event
extern void triggerMotionEvent(int x, int y, float pressure, HWND window = NULL);
extern void triggerButtonEvent(int which, int state, int x, int y, HWND window = NULL);
extern void triggerKeyboardEvent(char value, int state, HWND window = NULL);
extern void triggerResizeEvent(int xSize, int ySize, HWND window = NULL);
extern void triggerPaintEvent(HWND window = NULL);
extern void triggerCloseEvent(HWND window = NULL);

#endif  // for header file
