////////////////////////////////////////////////////////
//
// pui test
//
// mdanks@cybermed.ucsd.edu
//
// Implementation file
//
//    Copyright (c) 1998 Mark Danks.
//    For information on usage and redistribution, and for a DISCLAIMER OF ALL
//    WARRANTIES, see the file, "LICENSE.TERMS" in this distribution.
//
/////////////////////////////////////////////////////////

#include <iostream.h>
#include <windows.h>
#include "pu.h"

#include "EventProc.h"
#include "WindowMan.h"

WindowMan newWindow;

int xWindowSize;
int yWindowSize;

static void redrawScene()
{
    newWindow.makeMainCurrent();
    glLoadIdentity();
	puDisplay();
    newWindow.swapMainBuffer();
}
void button_cb ( puObject *obj)
{
	float val;
	obj->getValue(&val);
	cout << "Button: " << val << endl;
}
void slider_cb ( puObject *obj)
{
	float val;
	obj->getValue(&val);
	cout << "Slider: " << val << endl;
}
static void motionFunc(int x, int y, float pressure, void *data)
{
	puMouse(x, y);
}
static void buttonFunc(int which, int state, int x, int y, void *data)
{
	puMouse(which, !state, x, y);
}

static void exitFunc(void *)
{
    exit(0);
}

void init(int width, int height)
{
    float halfX = (float)width / 2.f;
    float halfY = (float)height / 2.f;

    glDisable(GL_ALPHA_TEST);
    glDisable(GL_BLEND);

    glDisable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);
    glClearDepth(1.0);    
    glClearColor(0.f, 0.f, 0.f, 0.f);

    // setup the viewpoint
	glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
        gluOrtho2D(-halfX, halfX,
                   -halfY, halfY);
	glMatrixMode(GL_MODELVIEW);
	puSetWindowSize(width, height);
}
static void resizeFunc(int xSize, int ySize, void *)
{
	init(xSize, ySize);
}

int main(int argc, char *argv[])
{
    // set the size
    xWindowSize = 220;
    yWindowSize = 200;

    // Set up the hints
    WindowHints myHints;
    myHints.width = xWindowSize;
    myHints.height = yWindowSize;
    myHints.buffer = 2;

    // Register the callbacks
    setMotionCallback(motionFunc, NULL);
    setButtonCallback(buttonFunc, NULL);
    setCloseCallback(exitFunc, NULL);
    setResizeCallback(resizeFunc, NULL);

    // create the window
    if (!newWindow.createWindow(myHints))
    {
        cerr << "Unable to create window" << endl;
        exit(-1);
    }
    // set the title
    newWindow.setTitle("puiTest");
    
    // set up the rendering
    newWindow.makeMainCurrent();
    init(myHints.width, myHints.height);
	newWindow.displayWindow();

	puInit () ;

	puOneShot *b = new puOneShot ( 10, 50, 200, 80) ;
	b->setLegend( "Say Hello" );
	b->setCallback( button_cb );

	puSlider *s = new puSlider ( 10, 100, 200) ;
	s->setLegend( "Slider" );
	s->setCallback( slider_cb );
    
	newWindow.swapMainBuffer();

    redrawScene();

    // main loop
    // Process all messages
    MSG msg;

	while(1)
	{
		while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE) )
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		redrawScene();
	}

    return(0);
}

