
#ifndef _SSG_H_
#define _SSG_H_  1

#ifdef WIN32
#include <windows.h>
#endif

#include <assert.h>
#include "sg.h"
#include <GL/gl.h>
#include <GL/glu.h>

#ifndef TRUE
#define TRUE  1
#define FALSE 0
#endif

#ifndef _SSG_PUBLIC
#define _SSG_PUBLIC  protected
#endif

extern int sgebug ;

enum ssgCullResult
{
  SSG_OUTSIDE  = SG_OUTSIDE,
  SSG_INSIDE   = SG_INSIDE,
  SSG_STRADDLE = SG_STRADDLE
} ;

#define SSG_MAXPATH   50

#define SSGTRAV_CULL   1
#define SSGTRAV_ISECT  2
#define SSGTRAV_HOT    4

class ssgList        ;
class ssgBase        ;
class ssgEntity      ;
class ssgLeaf        ;
class ssgVTable      ;
class ssgBranch      ;
class ssgInvisible   ;
class ssgBaseTransform ;
class ssgTransform   ;
class ssgTexTrans    ;
class ssgCutout      ;
class ssgSelector    ;
class ssgAnimation   ;
class ssgRoot        ;

#define SSG_TYPE_BASE          0x00000001
#define SSG_TYPE_ENTITY        0x00000002
#define SSG_TYPE_LEAF          0x00000004
#define SSG_TYPE_VTABLE        0x00000008
#define SSG_TYPE_BRANCH        0x00000010
#define SSG_TYPE_BASETRANSFORM 0x00000020
#define SSG_TYPE_TRANSFORM     0x00000040
#define SSG_TYPE_TEXTRANS      0x00000080
#define SSG_TYPE_SELECTOR      0x00000100
#define SSG_TYPE_ANIMATION     0x00000200
#define SSG_TYPE_ROOT          0x00000400
#define SSG_TYPE_CUTOUT        0x00000800


inline int ssgTypeBase       () { return SSG_TYPE_BASE ; }
inline int ssgTypeEntity     () { return SSG_TYPE_ENTITY    | ssgTypeBase    () ; }
inline int ssgTypeLeaf       () { return SSG_TYPE_LEAF      | ssgTypeEntity  () ; }
inline int ssgTypeVTable     () { return SSG_TYPE_VTABLE    | ssgTypeLeaf    () ; }
inline int ssgTypeBranch     () { return SSG_TYPE_BRANCH    | ssgTypeEntity  () ; }
inline int ssgTypeBaseTransform(){return SSG_TYPE_BASETRANSFORM | ssgTypeBranch  () ; }
inline int ssgTypeTransform  () { return SSG_TYPE_TRANSFORM | ssgTypeBaseTransform () ; }
inline int ssgTypeTexTrans   () { return SSG_TYPE_TEXTRANS  | ssgTypeBaseTransform () ; }
inline int ssgTypeSelector   () { return SSG_TYPE_SELECTOR  | ssgTypeBranch  () ; }
inline int ssgTypeAnimation  () { return SSG_TYPE_ANIMATION | ssgTypeSelector() ; }
inline int ssgTypeRoot       () { return SSG_TYPE_ROOT      | ssgTypeBranch  () ; }
inline int ssgTypeCutout     () { return SSG_TYPE_CUTOUT    | ssgTypeBranch  () ; }

/*
  It's critical that these numbers don't change without
  some pretty pressing need because significant change to
  ssgSimpleState.cxx and ssgStateTables.cxx would be needed.
*/

#define SSG_GL_TEXTURE_EN        0
#define SSG_GL_CULL_FACE_EN      1
#define SSG_GL_COLOR_MATERIAL_EN 2
#define SSG_GL_BLEND_EN          3
#define SSG_GL_ALPHA_TEST_EN     4
#define SSG_GL_LIGHTING_EN       5
 
#define SSG_GL_TEXTURE           6
#define SSG_GL_COLOR_MATERIAL    7
#define SSG_GL_DIFFUSE           8
#define SSG_GL_AMBIENT           9
#define SSG_GL_SPECULAR         10
#define SSG_GL_EMISSION         11
#define SSG_GL_SHININESS        12
#define SSG_GL_ALPHA_TEST       13
#define SSG_GL_SHADE_MODEL      14

int ssgGetFrameCounter () ;


class ssgList
{
  unsigned int total ;  /* The total number of entities in the list */
  unsigned int limit ;  /* The current limit on number of entities  */
  unsigned int next  ;  /* The next entity when we are doing getNext operations */

  ssgEntity **entity_list ;
  void sizeChk (void) ;

  int i_am_a_parent_list ;

public:

  ssgList ( int am_parent = FALSE, int init_max = 1 ) ;
  virtual ~ssgList (void) ;

  void amParent () { i_am_a_parent_list = TRUE ; }

  ssgEntity *getEntity ( unsigned int n )
  {
    next = n ;
    return ( n >= total ) ? (ssgEntity *) NULL : entity_list [ n ] ;
  }

  void addEntity ( ssgEntity *entity ) ;

  void removeAllEntities () ;

  void removeEntity ( ssgEntity *entity )
  {
    removeEntity ( searchForEntity ( entity ) ) ;
  }

  int        getNumEntities    (void) { return total ; }
  ssgEntity *getNextEntity     (void) { return getEntity ( next+1 ) ; }
  int        searchForEntity   ( ssgEntity *entity ) ;
  void       removeEntity      ( unsigned int n ) ;
} ;


class ssgBase 
{
  int   refc ;

protected :

  int   type ;

public:

  ssgBase (void) ;
  virtual ~ssgBase (void) ;

  void ref   () { refc++ ; }
  void deRef () { assert ( refc != 0 ) ; refc-- ; }
  int  getRef() { return refc ; }

  /* Type checking mechanism */

  virtual char *getTypeName(void) ;

  int   getType    (void)     { return type ; }
  int   isA        ( int ty ) { return getType() == ty ; }
  int   isAKindOf  ( int ty ) { return ( getType() & ty ) == ty ; }

  virtual void print ( FILE *fd = stderr, char *indent = "" ) ;
} ;


class ssgImageLoader
{
  void make_mip_maps  ( GLubyte *image, int x, int y, int bytes_per_texel ) ;

  void loadTextureSGI ( char *fname ) ;
  void loadTextureBMP ( char *fname ) ;
  void loadTexturePNG ( char *fname ) ;
  void loadDummyTexture () ;

public:

  void loadTexture    ( char *fname ) ;
} ;


class ssgTexture : public ssgBase
{
  GLuint handle ;

public:

  ssgTexture ( char *fname, int wrapu = TRUE, int wrapv = TRUE )
  {
#ifdef GL_VERSION_1_1
    glGenTextures ( 1, & handle ) ;
    glBindTexture ( GL_TEXTURE_2D, handle ) ;
#else
    /* This is only useful on some ancient SGI hardware */
    glGenTexturesEXT ( 1, & handle ) ;
    glBindTextureEXT ( GL_TEXTURE_2D, handle ) ;
#endif

    ssgImageLoader loader ;

    loader . loadTexture ( fname ) ;

    glTexEnvi ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE ) ;

    glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR ) ;
    glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
                                              GL_LINEAR_MIPMAP_LINEAR ) ;
    glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, wrapu ? GL_REPEAT : GL_CLAMP ) ;
    glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, wrapv ? GL_REPEAT : GL_CLAMP ) ;
#ifdef GL_VERSION_1_1
    glBindTexture ( GL_TEXTURE_2D, 0 ) ;
#else
    glBindTextureEXT ( GL_TEXTURE_2D, 0 ) ;
#endif
  }

  GLuint getHandle () { return handle ; }
  virtual void print ( FILE *fd = stderr, char *indent = "" ) ;
  virtual char *getTypeName(void) ;
} ;


class ssgState : public ssgBase
{
  int translucent ;

  int external_property_index ;
public:
  ssgState (void) ;
  virtual ~ssgState (void) ;

  virtual char *getTypeName(void) ;

  int  getExternalPropertyIndex () { return external_property_index ; }
  void setExternalPropertyIndex ( int i ) { external_property_index = i ; }

  virtual int  isTranslucent ()  { return translucent ;  }
  virtual void setTranslucent () { translucent = TRUE  ; }
  virtual void setOpaque      () { translucent = FALSE ; }
  virtual void force () = 0 ;
  virtual void apply () = 0 ;
} ;


class ssgSimpleState : public ssgState
{
_SSG_PUBLIC:

  int    dont_care ; 
  int    enables   ; 
  GLuint texture_handle ;

  int colour_material_mode ;
  sgVec4 specular_colour ;
  sgVec4 emission_colour ;
  sgVec4  ambient_colour ;
  sgVec4  diffuse_colour ;

  GLenum shade_model ;

  float  shininess ;
  float  alpha_clamp ;

  ssgSimpleState ( int I_am_current_state ) ;

public:

  ssgSimpleState (void) ;
  virtual ~ssgSimpleState (void) ;
  virtual char *getTypeName(void) ;

  virtual void force () ;
  virtual void apply () ;

  void      care_about ( int mode ) { dont_care &= ~(1<<mode) ; }
  void dont_care_about ( int mode ) { dont_care |=  (1<<mode) ; }

  int  isEnabled ( GLenum mode ) ;
  void disable   ( GLenum mode ) ;
  void enable    ( GLenum mode ) ;
  void set       ( GLenum mode, int val ) { val ? enable(mode) : disable(mode) ; }

  void setTexture ( char *fname, int wrapu = TRUE, int wrapv = TRUE )
  {
    ssgTexture *tex = new ssgTexture ( fname, wrapu, wrapv ) ;
    setTexture ( tex ) ;
    delete tex ;
  }

  GLuint getTextureHandle ()
  {
    return texture_handle ;
  }

  void setTexture ( ssgTexture *tex )
  {
    setTexture ( tex -> getHandle () ) ;
  }

  void setTexture ( GLuint tex )
  {
    texture_handle = tex ;
    care_about ( SSG_GL_TEXTURE ) ;
  }

  void setColourMaterial ( GLenum which )
  {
    colour_material_mode = which ;
    care_about ( SSG_GL_COLOR_MATERIAL ) ;
  }

  void setMaterial ( GLenum which, float r, float g, float b, float a = 1.0f )
  {
    sgVec4 rgba ;
    sgSetVec4 ( rgba, r, g, b, a ) ;
    setMaterial ( which, rgba ) ;
  }

  void setMaterial   ( GLenum which, sgVec4 rgba )
  {
    switch ( which )
    {
      case GL_EMISSION : sgCopyVec4 ( emission_colour, rgba ) ; care_about ( SSG_GL_EMISSION ) ; break ;
      case GL_SPECULAR : sgCopyVec4 ( specular_colour, rgba ) ; care_about ( SSG_GL_SPECULAR ) ; break ;
      case GL_AMBIENT  : sgCopyVec4 ( ambient_colour , rgba ) ; care_about ( SSG_GL_AMBIENT  ) ; break ;
      case GL_DIFFUSE  : sgCopyVec4 ( diffuse_colour , rgba ) ; care_about ( SSG_GL_DIFFUSE  ) ; break ;
      default: break ;
    }
  }

  float *getMaterial ( GLenum which )
  {
    switch ( which )
    {
      case GL_EMISSION : return emission_colour ;
      case GL_SPECULAR : return specular_colour ;
      case GL_AMBIENT  : return ambient_colour  ;
      case GL_DIFFUSE  : return diffuse_colour  ;
      default: break ;
    }

    return NULL ;
  }

  float getShininess ()
  {
    return shininess ;
  }
 
  void setShininess ( float sh )
  {
    care_about ( SSG_GL_SHININESS ) ;
    shininess = sh ;
  }
 
  void setShadeModel ( GLenum model )
  {
    care_about ( SSG_GL_SHADE_MODEL ) ;
    shade_model = model ;
  }
 
  void setAlphaClamp ( float clamp )
  {
    care_about ( SSG_GL_ALPHA_TEST ) ;
    alpha_clamp = clamp ;
  }
 
  virtual void print ( FILE *fd, char *indent = "" ) ;
} ;


class ssgEntity : public ssgBase
{
  ssgList parents ;

  char *name ;

  char *user_data ;
  int traversal_mask ;

protected:
  sgSphere bsphere ;
  int bsphere_is_invalid ;

  void emptyBSphere  ()              { bsphere.empty  ()    ; }
  void visualiseBSphere () ;
  void extendBSphere ( sgSphere *s ) { bsphere.extend ( s ) ; }
  void extendBSphere ( sgBox    *b ) { bsphere.extend ( b ) ; }
  void extendBSphere ( sgVec3    v ) { bsphere.extend ( v ) ; }

  virtual ssgCullResult cull_test  ( sgFrustum *f, sgMat4 m, int test_needed ) ;
  virtual ssgCullResult isect_test ( sgSphere  *s, sgMat4 m, int test_needed ) ;
  virtual ssgCullResult hot_test   ( sgVec3     s, sgMat4 m, int test_needed ) ;

public:
 
  ssgEntity (void) ;
  virtual ~ssgEntity (void) ;
  
  int  getTraversalMask     ()        { return traversal_mask ;}
  void setTraversalMask     ( int t ) { traversal_mask  =  t ; }
  void setTraversalMaskBits ( int t ) { traversal_mask |=  t ; }
  void clrTraversalMaskBits ( int t ) { traversal_mask &= ~t ; }

  char *getUserData ()            { return user_data ; }
  void  setUserData ( char *s )   { user_data = s ; }

  virtual void recalcBSphere (void) = 0 ;
  int  isDirtyBSphere (void) { return bsphere_is_invalid ; }
  void dirtyBSphere  () ;

  void  setName ( char *nm ) { name = nm ; }
  char *getName () { return name ; }

  sgSphere *getBSphere ()
  {
    if ( isDirtyBSphere () )
      recalcBSphere () ;

    return & bsphere ;
  }

  virtual int getNumKids (void) { return 0 ; }
  int getNumParents () { return parents.getNumEntities () ; }
  ssgBranch *getParent ( int p ) { return (ssgBranch *) parents.getEntity ( p ) ; }
  ssgBranch *getNextParent () { return (ssgBranch *) parents.getNextEntity () ; }
  void addParent    ( ssgEntity *entity ) { parents.addEntity    ( entity ) ; }
  void removeParent ( ssgEntity *entity ) { parents.removeEntity ( entity ) ; }

  virtual char *getTypeName(void) ;

  virtual void cull  ( sgFrustum *f, sgMat4 m, int test_needed ) = 0 ;
  virtual void isect ( sgSphere  *s, sgMat4 m, int test_needed ) = 0 ;
  virtual void hot   ( sgVec3     s, sgMat4 m, int test_needed ) = 0 ;
  virtual void print ( FILE *fd = stderr, char *indent = "" ) ;
} ;


class ssgLeaf : public ssgEntity
{
  int cull_face ;
  ssgState *state ;

protected:

  GLuint dlist ;
  virtual void draw_geometry () = 0 ;

  void preDraw () ;

public:
  ssgLeaf (void) ;
  virtual ~ssgLeaf (void) ;

  void makeDList () ;
  void deleteDList () ;
  GLuint getDListIndex () { return dlist ; }

  int  getExternalPropertyIndex ()
                 { return state ? state->getExternalPropertyIndex() : 0 ; }

  int  isTranslucent () { return state ? state->isTranslucent() : FALSE ; }
  int       hasState () { return state != NULL ; }
  ssgState *getState () { return state ; }
  void      setState ( ssgState *st ) { state = st ; }

  virtual float *getVertex   ( int i ) = 0 ;
  virtual float *getNormal   ( int i ) = 0 ;
  virtual float *getColour   ( int i ) = 0 ;
  virtual float *getTexCoord ( int i ) = 0 ;
  virtual int  getNumTriangles () = 0 ;
  virtual void getTriangle ( int n, short *v1, short *v2, short *v3 ) = 0 ;

  virtual void transform ( sgMat4 m ) = 0 ;

  void setCullFace ( int cf ) { cull_face = cf ; }
  int  getCullFace () { return cull_face ; }

  virtual void recalcBSphere () = 0 ;
  virtual char *getTypeName(void) ;
  virtual void print ( FILE *fd = stderr, char *indent = "" ) ;
  virtual void cull  ( sgFrustum *f, sgMat4 m, int test_needed ) ;
  virtual void isect ( sgSphere  *s, sgMat4 m, int test_needed ) ;
  virtual void hot   ( sgVec3     s, sgMat4 m, int test_needed ) ;
  virtual void isect_triangles ( sgSphere *s, sgMat4 m, int test_needed ) = 0 ;
  virtual void hot_triangles   ( sgVec3    s, sgMat4 m, int test_needed ) = 0 ;
  virtual void draw  () = 0 ;
} ;

extern sgVec3 _tfVertex000   ;
extern sgVec4 _tfColourWhite ;
extern sgVec3 _tfNormalUp    ;
extern sgVec2 _tfTexCoord00  ;

class ssgVTable : public ssgLeaf
{
  sgBox bbox ;
  int indexed ;

  sgVec3 *vertices  ; int num_vertices  ; unsigned short *v_index ;
  sgVec3 *normals   ; int num_normals   ; unsigned short *n_index ;
  sgVec2 *texcoords ; int num_texcoords ; unsigned short *t_index ;
  sgVec4 *colours   ; int num_colours   ; unsigned short *c_index ;

  GLenum gltype ;

  virtual void draw_geometry () ;
public:
  ssgVTable ( GLenum ty,
              int nv, unsigned short *vi, sgVec3 *vl,
              int nn, unsigned short *ni, sgVec3 *nl,
              int nt, unsigned short *ti, sgVec2 *tl,
              int nc, unsigned short *ci, sgVec4 *cl ) ;

  ssgVTable ( GLenum ty,
              int nv, sgVec3 *vl,
              int nn, sgVec3 *nl,
              int nt, sgVec2 *tl,
              int nc, sgVec4 *cl ) ;

  virtual void transform ( sgMat4 m ) ;

  int getNumVertices  () { return num_vertices  ; }
  int getNumNormals   () { return num_normals   ; }
  int getNumColours   () { return num_colours   ; }
  int getNumTexCoords () { return num_texcoords ; }
  int getNumTriangles () ;
  void getTriangle ( int n, short *v1, short *v2, short *v3 ) ;

  void getColourList ( void **list, unsigned short **idx )
  {
    *list = colours ;
    *idx  = c_index  ; 
  }

  void getTexCoordList ( void **list, unsigned short **idx )
  {
    *list = texcoords ;
    *idx  = t_index  ; 
  }

  void getNormalList ( void **list, unsigned short **idx )
  {
    *list = normals ;
    *idx  = n_index  ; 
  }

  void getVertexList ( void **list, unsigned short **idx )
  {
    *list = vertices ;
    *idx  = v_index  ; 
  }

  float *getVertex  (int i){ if(i>=num_vertices)i=num_vertices-1;
                             return (num_vertices<=0) ? _tfVertex000 :
                                    ((indexed)?vertices [v_index[i]]:vertices [i]);}
  float *getColour  (int i){ if(i>=num_colours)i=num_colours-1;
                             return (num_colours<=0) ? _tfColourWhite :
                                    ((indexed)?colours  [c_index[i]]:colours  [i]);}
  float *getNormal  (int i){ if(i>=num_normals)i=num_normals-1;
                             return (num_normals<=0) ? _tfNormalUp :
                                    ((indexed)?normals  [n_index[i]]:normals  [i]);}
  float *getTexCoord(int i){ if(i>=num_texcoords)i=num_texcoords-1;
                             return (num_texcoords<=0) ? _tfTexCoord00 :
                                    ((indexed)?texcoords[t_index[i]]:texcoords[i]);}

  GLenum getGLtype () { return gltype ; }

  virtual ~ssgVTable (void) ;

  virtual char *getTypeName(void) ;
  virtual void recalcBSphere () ;
  virtual void draw () ;

  virtual void isect_triangles ( sgSphere  *s, sgMat4 m, int test_needed ) ;
  virtual void hot_triangles   ( sgVec3     s, sgMat4 m, int test_needed ) ;
  virtual void print ( FILE *fd = stderr, char *indent = "" ) ;
} ;


class ssgBranch : public ssgEntity
{
  ssgList kids ;

public:

  ssgBranch (void) ;
  virtual ~ssgBranch (void) ;

  virtual int getNumKids (void)   { return kids.getNumEntities() ; }
  ssgEntity *getKid     ( int n ) { return kids.getEntity  ( n ) ; }
  ssgEntity *getNextKid (void)    { return kids.getNextEntity () ; }
  int        searchForKid ( ssgEntity *entity )
                                  { return kids.searchForEntity(entity); }

  void addKid        ( ssgEntity *entity ) ;
  void removeKid     ( int n ) ;
  void removeKid     ( ssgEntity *entity ) ;
  void removeAllKids (void) ;

  virtual char *getTypeName(void) ;
  virtual void cull          ( sgFrustum *f, sgMat4 m, int test_needed ) ;
  virtual void isect         ( sgSphere  *s, sgMat4 m, int test_needed ) ;
  virtual void hot           ( sgVec3     s, sgMat4 m, int test_needed ) ;
  virtual void print         ( FILE *fd = stderr, char *indent = "" ) ;
  virtual void recalcBSphere () ;
} ;

class ssgInvisible : public ssgBranch
{
public:

  ssgInvisible (void) ;
  virtual ~ssgInvisible (void) ;

  virtual void cull ( sgFrustum *f, sgMat4 m, int test_needed ) ;
} ;



class ssgSelector : public ssgBranch
{
  unsigned int selection ;
public:

  ssgSelector (void) ;
  virtual ~ssgSelector (void) ;

  void selectStep ( unsigned int s )
  {
    selection = (1<<s) ;
  }

  void select ( unsigned int s )
  {
    selection = s ;
  }

  unsigned int getSelect () { return selection ; }

  virtual char *getTypeName(void) ;
  virtual void cull  ( sgFrustum *f, sgMat4 m, int test_needed ) ;
  virtual void isect ( sgSphere  *s, sgMat4 m, int test_needed ) ;
  virtual void hot   ( sgVec3     s, sgMat4 m, int test_needed ) ;
} ;


enum ssgAnimEnum
{
  SSG_ANIM_START,
  SSG_ANIM_STOP,
  SSG_ANIM_PAUSE,
  SSG_ANIM_RESUME,
  SSG_ANIM_SWING,
  SSG_ANIM_SHUTTLE
} ;


class ssgAnimation : public ssgSelector
{
  int step ;
  int mode ;
  float times [ 32 ] ;
  int curr ;
  int start ;
  int end ;
public:

  ssgAnimation (void) ;
  virtual ~ssgAnimation (void) ;

  virtual char *getTypeName(void) ;

  void setDuration ( float ti, int i )
  {
    if ( i >= 0 && i < 32 )
      times [ i ] = ti ;
    else
    for ( int j = 0 ; j < 32 ; j++ )
      times [ j ] = ti ;
  }

  void setLimits ( ssgAnimEnum m, int st, int en )
         { mode = m ; start = st ; end = en ; step = 1 ; curr = 0 ; }
} ;


class ssgBaseTransform : public ssgBranch
{
protected:

  sgMat4 transform ;
  sgMat4 last_transform ;
  int    last_updated ;
public:

  ssgBaseTransform (void) ;
  virtual ~ssgBaseTransform (void) ;

  void updateTransform ()
  {
    sgCopyMat4 ( last_transform, transform ) ;
    last_updated = ssgGetFrameCounter () ;
  }

  void getLastTransform ( sgMat4 xform )
  {
    /*
      If the transform was not updated this - or last frame
      then we need to equate the two transforms.
    */

    if ( last_updated < ssgGetFrameCounter () - 1 )
      updateTransform () ;

    sgCopyMat4 ( xform, last_transform ) ;
  }

  void getTransform ( sgMat4 xform )
  {
    sgCopyMat4 ( xform, transform ) ;
  }

  virtual void setTransform ( sgVec3 xyz ) = 0 ;
  virtual void setTransform ( sgCoord *xform ) = 0 ;
  virtual void setTransform ( sgCoord *xform, float sx, float sy, float sz ) = 0 ;
  virtual void setTransform ( sgMat4 xform ) = 0 ;

  virtual char *getTypeName(void) ;
} ;


class ssgTransform : public ssgBaseTransform
{
public:

  ssgTransform (void) ;
  ssgTransform ( sgCoord *c ) ;
  virtual ~ssgTransform (void) ;

  virtual void setTransform ( sgVec3 xyz ) ;
  virtual void setTransform ( sgCoord *xform ) ;
  virtual void setTransform ( sgCoord *xform, float sx, float sy, float sz ) ;
  virtual void setTransform ( sgMat4 xform ) ;

  virtual char *getTypeName(void) ;
  virtual void cull  ( sgFrustum *f, sgMat4 m, int test_needed ) ;
  virtual void isect ( sgSphere  *s, sgMat4 m, int test_needed ) ;
  virtual void hot   ( sgVec3     s, sgMat4 m, int test_needed ) ;
  virtual void recalcBSphere () ;
} ;


class ssgTexTrans : public ssgBaseTransform
{
public:

  ssgTexTrans (void) ;
  ssgTexTrans ( sgCoord *c ) ;
  virtual ~ssgTexTrans (void) ;

  virtual void setTransform ( sgVec3 xyz ) ;
  virtual void setTransform ( sgCoord *xform ) ;
  virtual void setTransform ( sgCoord *xform, float sx, float sy, float sz ) ;
  virtual void setTransform ( sgMat4 xform ) ;

  virtual char *getTypeName(void) ;
  virtual void cull  ( sgFrustum *f, sgMat4 m, int test_needed ) ;
} ;


class ssgCutout : public ssgBranch
{
  int point_rotate ;
public:

  ssgCutout (int pntrot=FALSE) ;
  virtual ~ssgCutout (void) ;

  virtual char *getTypeName(void) ;
  virtual void cull  ( sgFrustum *f, sgMat4 m, int test_needed ) ;
  virtual void isect ( sgSphere  *s, sgMat4 m, int test_needed ) ;
  virtual void hot   ( sgVec3     s, sgMat4 m, int test_needed ) ;
} ;


class ssgRoot : public ssgBranch
{
public:

  ssgRoot (void) ;
  virtual ~ssgRoot (void) ;
  virtual char *getTypeName(void) ;
} ;


class ssgLight
{
  int id ;
  int is_headlight ;
  int is_turned_on ;

  sgVec4 ambient ;
  sgVec4 diffuse ;
  sgVec4 specular ;

  sgVec4 position ;

public:

  ssgLight ()
  {
    id = 0 ;
    is_turned_on = FALSE ;
    is_headlight = FALSE ;
    sgSetVec4 ( position, 0.0f, 0.0f, 1.0f, 0.0f ) ;
    sgSetVec4 ( ambient , 0.2f, 0.2f, 0.2f, 1.0f ) ;
    sgSetVec4 ( diffuse , 1.0f, 1.0f, 1.0f, 1.0f ) ;
    sgSetVec4 ( specular, 1.0f, 1.0f, 1.0f, 1.0f ) ;
  }

  void setID ( int i ) { id = i ; }
  int  isOn () { return is_turned_on  ; }
  void on  () { is_turned_on = TRUE  ; }
  void off () { is_turned_on = FALSE ; }
  void setPosition ( sgVec3 pos ) { sgCopyVec3 ( position, pos ) ; }
  void setHeadlight ( int head ) { is_headlight = head ; }
  int  isHeadlight () { return is_headlight ; }

  void setup ()
  {
    if ( is_turned_on )
    {
      glEnable  ( (GLenum)(GL_LIGHT0+id) ) ;
      glLightfv ( (GLenum)(GL_LIGHT0+id), GL_AMBIENT , ambient  ) ;
      glLightfv ( (GLenum)(GL_LIGHT0+id), GL_DIFFUSE , diffuse  ) ;
      glLightfv ( (GLenum)(GL_LIGHT0+id), GL_SPECULAR, specular ) ;
      glLightfv ( (GLenum)(GL_LIGHT0+id), GL_POSITION, position ) ;
    }
    else
      glDisable ( (GLenum)(GL_LIGHT0+id) ) ;
  }
} ;

class ssgHit
{
_SSG_PUBLIC:

  int num_entries ;
  ssgEntity *path [ SSG_MAXPATH ] ;

public:
  ssgLeaf *leaf ;
  int      triangle ;
  sgVec4   plane ;
  sgMat4   matrix ;

  ssgHit ()
  {
    leaf = NULL ;
    init () ;
  } ;

  void init () { num_entries = 0 ; }

  void addPath ( ssgEntity *e )
  {
    if ( num_entries < SSG_MAXPATH )
      path [ num_entries++ ] = e ;
  }
 
  int getNumPathEntries () { return num_entries ; }

  ssgEntity *getPathEntry ( int i )
  {
    return ( i >= 0 && i < num_entries ) ? path[i] : (ssgEntity *) NULL ;
  }
} ;


void ssgInit () ;
void ssgSetFOV      ( float w, float h ) ;
void ssgSetNearFar  ( float n, float f ) ;
void ssgSetCamera   ( sgCoord *coord ) ;
void ssgCullAndDraw ( ssgRoot *root ) ;
int  ssgIsect       ( ssgRoot *root, sgSphere *s, sgMat4 m, ssgHit **results ) ;
int  ssgHOT         ( ssgRoot *root, sgVec3    s, sgMat4 m, ssgHit **results ) ;
int  ssgLOS         ( ssgRoot *root, sgVec3    s, sgMat4 m, ssgHit **results ) ;
void ssgGetCameraPosition ( sgVec3 pos ) ;

typedef ssgBranch *(*ssgHookFunc)(char *) ;

ssgEntity *ssgLoadAC ( char *fname, ssgHookFunc hookfunc = NULL ) ;
void       ssgSaveAC ( char *fname, ssgEntity *ent ) ;
void ssgFlatten  ( ssgEntity *ent ) ;
void ssgStripify ( ssgEntity *ent ) ;

void ssgModelPath   ( char *path ) ;
void ssgTexturePath ( char *path ) ;

ssgLight *ssgGetLight ( int i ) ;

int ssgGetNumTexelsLoaded () ;

void ssgOverrideTexture ( int on_off ) ;

void ssgSetAppStateCallback ( ssgState *(*cb)(char *) ) ;

char *ssgShowStats () ;

void ssgDelete ( ssgBranch *br ) ;

#endif

