
#include "ssgLocal.h"


ssgBranch::ssgBranch (void)
{
  type |= SSG_TYPE_BRANCH ;
}


ssgBranch::~ssgBranch (void)
{
}



ssgInvisible::ssgInvisible (void)
{
}


ssgInvisible::~ssgInvisible (void)
{
}


void ssgBranch::recalcBSphere (void)
{
  emptyBSphere () ;

  for ( ssgEntity *k = getKid ( 0 ) ; k != NULL ; k = getNextKid () )
    extendBSphere ( k -> getBSphere () ) ;

  bsphere_is_invalid = FALSE ;
}


void ssgBranch::addKid ( ssgEntity *entity )
{
  kids. addEntity ( entity ) ;
  entity -> addParent ( this ) ;
  dirtyBSphere () ;
}


void ssgBranch::removeKid  ( int n )
{
  ssgEntity *k = kids.getEntity ( n ) ;

  if ( k != NULL )
    removeKid ( k ) ;
}


void ssgBranch::removeKid ( ssgEntity *entity )
{
/* SJB */
  entity -> removeParent ( this ) ;

  kids.removeEntity ( entity ) ;
  dirtyBSphere () ;
}


void ssgBranch::removeAllKids (void)
{
/* SJB */
  for ( ssgEntity *k = getKid ( 0 ) ; k != NULL ; k = getNextKid () )
    removeKid ( k ) ;

  kids . removeAllEntities () ;
  dirtyBSphere () ;
}

void ssgBranch::print ( FILE *fd, char *indent )
{
  ssgEntity::print ( fd, indent ) ;
  fprintf ( fd, "%s  Num Kids=%d\n", indent, getNumKids() ) ;

  if ( getNumKids() + getNumParents() != getRef() )
    fprintf ( stderr, "****** SNAFU: Ref count doesn't tally with kid+parent counts\n" ) ;

  fprintf ( fd, "%s  BSphere  R=%g, C=(%g,%g,%g)\n", indent,
    bsphere.getRadius(), bsphere.getCenter()[0], bsphere.getCenter()[1], bsphere.getCenter()[2] ) ;

  char in [ 100 ] ;
  sprintf ( in, "%s  ", indent ) ;

  for ( ssgEntity *e = getKid ( 0 ) ; e != NULL ; e = getNextKid() )
    e -> print ( fd, in ) ;
}



void ssgInvisible::cull ( sgFrustum *, sgMat4, int /* test_needed */ )
{
}



void ssgBranch::cull ( sgFrustum *f, sgMat4 m, int test_needed )
{
  int cull_result = cull_test ( f, m, test_needed ) ;

  if ( cull_result == SSG_OUTSIDE )
    return ;

  for ( ssgEntity *e = getKid ( 0 ) ; e != NULL ; e = getNextKid() )
    e -> cull ( f, m, cull_result != SSG_INSIDE ) ;
}



void ssgBranch::hot ( sgVec3 s, sgMat4 m, int test_needed )
{
  int hot_result = hot_test ( s, m, test_needed ) ;

  if ( hot_result == SSG_OUTSIDE )
    return ;

  _ssgPushPath ( this ) ;

  for ( ssgEntity *e = getKid ( 0 ) ; e != NULL ; e = getNextKid() )
    e -> hot ( s, m, hot_result != SSG_INSIDE ) ;

  _ssgPopPath () ;
}



void ssgBranch::isect ( sgSphere *s, sgMat4 m, int test_needed )
{
  int isect_result = isect_test ( s, m, test_needed ) ;

  if ( isect_result == SSG_OUTSIDE )
    return ;

  _ssgPushPath ( this ) ;

  for ( ssgEntity *e = getKid ( 0 ) ; e != NULL ; e = getNextKid() )
    e -> isect ( s, m, isect_result != SSG_INSIDE ) ;

  _ssgPopPath () ;
}


