
#include "ssgLocal.h" 

ssgEntity::ssgEntity (void)
{
  user_data = NULL ;
  traversal_mask = 0xFFFFFFFF ;
  parents . amParent () ;
  name  = "NoName" ;
  type |= SSG_TYPE_ENTITY ;
}

ssgEntity::~ssgEntity (void)
{
  delete user_data ;
}

void ssgEntity::dirtyBSphere ()
{
  bsphere_is_invalid = TRUE ;

  int np = getNumParents () ; 

  for ( int i = 0  ; i < np ; i++ )
    getParent ( i ) -> dirtyBSphere () ;
}

void ssgEntity::visualiseBSphere ()
{
  glDisable ( GL_LIGHTING ) ;
  glTranslatef(bsphere.getCenter()[0],bsphere.getCenter()[1],bsphere.getCenter()[2]);

  int spherebotch = (int)(this) % 9 ;

  switch ( spherebotch++ )
  {
    case 0 : glutWireSphere(bsphere.getRadius(),10,10); break ;
    case 1 : glutWireSphere(bsphere.getRadius(),11,10); break ;
    case 2 : glutWireSphere(bsphere.getRadius(),12,10); break ;
    case 3 : glutWireSphere(bsphere.getRadius(),10,11); break ;
    case 4 : glutWireSphere(bsphere.getRadius(),11,11); break ;
    case 5 : glutWireSphere(bsphere.getRadius(),12,11); break ;
    case 6 : glutWireSphere(bsphere.getRadius(),10,12); break ;
    case 7 : glutWireSphere(bsphere.getRadius(),11,12); break ;
    case 8 : glutWireSphere(bsphere.getRadius(),12,12); spherebotch = 0 ; break ;
  }
  glTranslatef(-bsphere.getCenter()[0],-bsphere.getCenter()[1],-bsphere.getCenter()[2]);
  glEnable ( GL_LIGHTING ) ;
}

void ssgEntity::print ( FILE *fd, char *indent )
{
  ssgBase::print ( fd, indent ) ;

  fprintf ( fd, "%s  Name = \"%s\"\n", indent,(name==NULL)?"No Name" : name ) ;
  fprintf ( fd, "%s  Num Parents=%d\n", indent, parents . getNumEntities () ) ;
}

ssgCullResult ssgEntity::cull_test ( sgFrustum *f, sgMat4 m, int test_needed )
{
  if ( (getTraversalMask() & SSGTRAV_CULL) == 0 )
    return SSG_OUTSIDE ;

  if ( ! test_needed )
    return SSG_INSIDE ;

  stats_cull_test++ ;
  sgSphere tmp = *(getBSphere()) ;
  tmp . orthoXform ( m ) ;

  return (ssgCullResult) f -> contains ( &tmp ) ;
}


ssgCullResult ssgEntity::isect_test ( sgSphere *s, sgMat4 m, int test_needed )
{
  if ( (getTraversalMask() & SSGTRAV_ISECT) == 0 )
    return SSG_OUTSIDE ;

  if ( ! test_needed )
    return SSG_INSIDE ;

  stats_isect_test++ ;
  sgSphere tmp = *(getBSphere()) ;
  tmp . orthoXform ( m ) ;

  /* Check axial distances for trivial reject */

  sgVec3 center_vec ;
  float sum_radii = s->getRadius() + tmp.getRadius() ;

  sgSubVec3 ( center_vec, s->getCenter(), tmp.getCenter() ) ;

  if ( center_vec[0] > sum_radii ||
       center_vec[1] > sum_radii ||
       center_vec[2] > sum_radii )
    return SSG_OUTSIDE ;

  float separation_sqd = sgScalarProductVec3 ( center_vec, center_vec ) ;

  float tmp_radius_sqd = sgSquare ( tmp.getRadius() ) ;
  float sph_radius_sqd = sgSquare ( s ->getRadius() ) ;

  float sum_radii_sqd = tmp_radius_sqd + sph_radius_sqd ;

  if ( separation_sqd >= sum_radii_sqd )
    return SSG_OUTSIDE ;

  if ( separation_sqd + tmp_radius_sqd <= sph_radius_sqd )
    return SSG_INSIDE ;

  return SSG_STRADDLE ;
}



ssgCullResult ssgEntity::hot_test ( sgVec3 s, sgMat4 m, int test_needed )
{
  stats_hot_test++ ;

  if ( !test_needed )
{
stats_hot_triv_accept++ ;
    return SSG_INSIDE ;
}
   
  if ((getTraversalMask() & SSGTRAV_HOT) == 0 )
{
stats_hot_no_trav++ ;
    return SSG_OUTSIDE ;
}

  sgSphere tmp = *(getBSphere()) ;
  tmp . orthoXform ( m ) ;

  float d = sgSquare ( s[0] - tmp.getCenter()[0] ) +
            sgSquare ( s[1] - tmp.getCenter()[1] ) ;

  if ( d > sgSquare ( tmp.getRadius() ) )
{
stats_hot_radius_reject++ ;
    return SSG_OUTSIDE ;
}

stats_hot_straddle++ ;
  return SSG_STRADDLE ;
}


