/*
	$Id: clanstring.h,v 1.4 1999/07/11 11:14:05 fischlustig Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		String handling class.


	$Log: clanstring.h,v $
	Revision 1.4  1999/07/11 11:14:05  fischlustig
	Switched to Makefile.BeOS
	
	Revision 1.3  1999/01/04 04:05:18  starch
	Made case-insensitive compare work on win32
	
	Revision 1.2  1998/12/31 20:16:29  mbn
	Added more usefull operations on the CL_String class. They are need by the
	XML filter.

	Revision 1.1.1.1  1998/12/20 23:33:18  mbn
	ClanLib 0.1.10 development version importet.

	Revision 1.8  1998/11/19 23:05:38  starch
	LOTS of changes from HELL...

	Revision 1.7  1998/11/12 13:36:27  mbn
	Serveral fixes so ClanLib can compile under HP unix.

	Revision 1.6  1998/10/24 01:58:50  starch
	Added function find() to clanstring which makes a strchr, and tried to
	make a copy constructor in the list class.

	Revision 1.5  1998/10/18 23:30:42  starch
	Added resource manager to makefile

	Revision 1.4  1998/10/18 23:07:55  starch
	Added support for streamed sound+small fixes

	Revision 1.3  1998/09/17 12:05:20  mbn
	Converted all spaces to tabs by using unexpand.

	Revision 1.2	1998/02/17 22:08:49	 mbn
	Changed class prefix to 'CL'.

	Revision 1.1.1.1	1998/02/07 16:51:17	 mbn
	Finally uploaded ClanLib-0.1.3 to CVS.

*/

#ifndef header_clanstring
#define header_clanstring

#include <stdlib.h>
#include <string.h>
#include <iostream.h>

#ifndef WIN32
#include <ctype.h>
#endif

#ifdef __BEOS__
extern char *fcvt (double, int, int *, int *);
#endif

class CL_String
{
#ifndef WIN32

// Why is this function not supported under Linux???

void itoa(int num, char *string, const int number_format)
{
	int num1 = num;
	int num_chars = 0;
	
	while (num1>0)
	{
		num_chars++;
		num1 /= number_format;
	}

	if (num_chars == 0) num_chars = 1;

	string[num_chars] = 0; // Null-terminate the string
	
	for (int pos = num_chars-1; pos>=0; pos--)
	{
		int cur_char = num % number_format;
		
		if (cur_char < 10) // Insert number
		{
			string[pos] = cur_char + '0';
		}
		else // Insert letter
		{
			string[pos] = cur_char-10 + 'A';
		}
		
		num /= number_format;
	}
}
#else
#pragma warning( disable : 4244 )

#endif

	char *string;

	char *int_to_string(const char *prefix, const int number)
	{
		char buf[20];
		itoa(number, buf, 10);

		int len=0;
		if (prefix!=NULL) len=strlen(prefix);

		char *text=new char[len+strlen(buf)+1];
		if (prefix!=NULL)
		{
			strcpy(text, prefix);
			strcat(text, buf);
		}
		else
		{
			strcpy(text, buf);
		}

		return text;
	}
	
	char *float_to_string(const char *prefix, const float _float_number)
	{
		char buf[25];
		buf[0]=0;

		int decimal, sign;
		int precision=4;
		char temp;

		float float_number = _float_number;

		const char *_float_buffer = fcvt(
			float_number,
			precision,
			&decimal,
			&sign);
			
		char *float_buffer = new char[strlen(_float_buffer)+1];
		strcpy(float_buffer, _float_buffer);

		if (sign) strcpy(buf, "-");
		if (decimal<=0)
		{
			strcat(buf, "0.");
			strcat(buf, float_buffer);
			goto copy_strings;
		}

		temp=float_buffer[decimal];
		float_buffer[decimal]=0;
		strcat(buf, float_buffer);
		float_buffer[decimal]=temp;

		strcat(buf, ".");
		strcat(buf, &float_buffer[decimal]);

	copy_strings:
		int len=0;
		if (prefix!=NULL) len=strlen(prefix);

		char *text=new char[len+strlen(buf)+1];
		if (prefix!=NULL)
		{
			strcpy(text, prefix);
			strcat(text, buf);
		}
		else strcpy(text, buf);

		return text;
	}

	char *append_string(const char *prefix, const char *postfix)
	{
		int len=0;
		if (prefix!=NULL) len=strlen(prefix);
		if (postfix!=NULL) len+=strlen(postfix);
		char *text=new char[len+1];
		if (prefix!=NULL)
		{
			strcpy(text, prefix);
			if (postfix!=NULL) strcat(text, postfix);
		}
		else if (postfix!=NULL)
		{
			strcpy(text, postfix);
		}

		return text;
	}

public:
	CL_String()
	{
		string=NULL;
	}

	CL_String(const int value)
	{
		string=int_to_string(NULL, value);
	}

	CL_String(const float float_value)
	{
		string=float_to_string(NULL, float_value);
	}

	CL_String(const char *text)
	{
		string=append_string(NULL, text);
	}
	
	CL_String(const CL_String &clone)
	{
		string=append_string(NULL, clone.string);
	}

	CL_String(const CL_String &text1, const CL_String &text2)
	{
		string=append_string(text1.string, text2.string);
	}

	CL_String(const char *text1, const char *text2)
	{
		string=append_string(text1, text2);
	}

	CL_String(const char *text, int length)
	{
		string = new char[length+1];
		memcpy(string, text, length);
		string[length] = 0;
	}

	virtual ~CL_String()
	{
		delete string;
	}

	CL_String operator+ (const CL_String &other_string)
	{
		return CL_String(append_string(string, other_string.string));
	}

	CL_String operator+ (const char *other_text)
	{
		return CL_String(append_string(string, other_text));
	}

	CL_String operator+ (const int number)
	{
		return CL_String(int_to_string(string, number));
	}

	CL_String operator+ (const float number)
	{
		return CL_String(float_to_string(string, number));
	}

	CL_String &operator+= (const CL_String &other_string)
	{
		char *new_string=append_string(string, other_string.string);
		delete string;
		string=new_string;

		return *this;
	}

	CL_String &operator+= (const char *other_text)
	{
		char *new_string=append_string(string, other_text);
		delete string;
		string=new_string;

		return *this;
	}

	CL_String &operator+= (const int number)
	{
		char *new_string=int_to_string(string, number);
		delete string;
		string=new_string;

		return *this;
	}

	CL_String &operator+= (const float number)
	{
		char *new_string=float_to_string(string, number);
		delete string;
		string=new_string;

		return *this;
	}

	CL_String &operator+= (double float_number)
	{
		return operator+= ((float) float_number);
	}

	CL_String &operator= (const CL_String &other_string)
	{
		delete string;
		string=append_string(NULL, other_string.string);

		return *this;
	}

	CL_String &operator= (const char *other_text)
	{
		if (string!=NULL) delete string;
		string=append_string(NULL, other_text);

		return *this;
	}

	CL_String &operator= (const int number)
	{
		if (string!=NULL) delete string;
		string=int_to_string(NULL, number);
		return *this;
	}

	CL_String &operator= (const float float_number)
	{
		delete string;
		string=float_to_string(NULL, float_number);
		return *this;
	}

	CL_String &operator= (double float_number)
	{
		return operator= ((float) float_number);
	}

	bool operator== (const CL_String &other_string)
	{
		if (string==NULL || other_string.string==NULL) return false;
		if (strcmp(string, other_string.string)==0) return true;
		return false;
	}

	bool operator== (const char *other_string)
	{
		if (string==NULL || other_string==NULL) return false;
		if (strcmp(string, other_string)==0) return true;
		return false;
	}
	
	bool operator!= (const CL_String &other_string)
	{
		return !operator==(other_string);
	}

	bool operator!= (const char *other_text)
	{
		return !operator==(other_text);
	}

	char *get_string()
	{
		if (string==NULL)
		{
			return "";
		}
		return string;
	}

	int get_length()
	{
		if (string==NULL) return 0;
		return strlen(string);
	}

	int get_as_int()
	{
		return atoi(string);
	}

	float get_as_float()
	{
		return atof(string);
	}

	operator const char* ()
	{
		return get_string();
	}

	CL_String &operator<< (const CL_String &other_string)
	{
		char *new_string=append_string(string, other_string.string);
		delete string;
		string=new_string;

		return *this;
	}

	CL_String &operator<< (const char *other_string)
	{
		char *new_string=append_string(string, other_string);
		if (string!=NULL) delete string;
		string=new_string;

		return *this;
	}

	CL_String &operator<< (const int number)
	{
		char *new_string=int_to_string(string, number);
		if (string!=NULL) delete string;
		string=new_string;

		return *this;
	}

	CL_String &operator<< (const float float_number)
	{
		char *new_string=float_to_string(string, float_number);
		if (string!=NULL) delete string;
		string=new_string;

		return *this;
	}

	CL_String &operator<< (double float_number)
	{
		return operator<< ((float) float_number);
	}

	CL_String get_word(int word_no)
	{
		if (string==NULL) return CL_String("");

		int len=get_length();
		int cur_word=-1;
		int word_start=0;

		int cur_state=0;
		for (int i=0; i<len; i++)
		{
			if (cur_state == 0)
			{
				if (string[i]!=32)
				{
					if (string[i]==34)
					{
						word_start = i+1;
						cur_state = 2;
					}
					else
					{
						word_start = i;
						cur_state = 1;
					}
					cur_word++;
				}
			}
			else if (cur_state == 1)
			{
				if (string[i]==32)
				{
					if (cur_word == word_no) return mid(word_start, i-word_start);
					cur_state = 0;
				}
			}
			else if (cur_state == 2)
			{
				if (string[i]==34)
				{
					if (cur_word == word_no) return mid(word_start, i-word_start);
					cur_state = 0;
				}
			}
		}

		if (cur_word == word_no) return mid(word_start);

		return CL_String("");
	}

	CL_String mid(int start, int len=-1)
	{
		if (len==-1) len=get_length()-start;

		char *new_string = new char[len+1];
		for (int i=start; i<start+len; i++) new_string[i-start]=string[i];
		new_string[len]=0;

		CL_String retval(new_string);

		delete[] new_string;

		return retval;
	}

	CL_String right(int len)
	{
		int strlength = get_length();

		if (strlength < len) return *this;

		char *new_string = new char[len+1];
		for (int i=0; i<len+1; i++) new_string[i] = string[strlength-len+i];

		CL_String retval(new_string);

		delete[] new_string;

		return retval;
	}

	int find(int character, int start_pos=0)
	{
		char *pos = strchr(string+start_pos, character);
		if (pos == NULL) return -1;
		return int(pos-string);
	}

	void to_lower()
	{
		if (string == NULL) return;

		char *p = string;
		while (*p != 0)
		{
			*p = tolower(*p);
			p++;
		}
	}

	void to_upper()
	{
		if (string == NULL) return;

		char *p = string;
		while (*p != 0)
		{
			*p = toupper(*p);
			p++;
		}
	}
	
	bool case_compare(const char *other)
	{
		if (string==NULL || other==NULL) return false;

#ifdef WIN32
		if (_stricmp(string, other)==0) return true;
#else
		if (strcasecmp(string, other)==0) return true;
#endif
		return false;
	}
};

#endif

