/*
  Copyright (c)1999 ClanSoft.
*/

#ifndef header_formatteddata
#define header_formatteddata

#ifdef WIN32
#include <winsock.h>
#else
#include <netinet/in.h>
#endif

#include "clanstring.h"

class CL_DataWriter
{
private:
	unsigned char *data;
	int pos;
	int max_size;
	
	void expand_data()
	{
		max_size *= 2;
		unsigned char *new_data = new unsigned char[max_size];
		memcpy(new_data, data, pos);
		delete[] data;
		data = new_data;
	}
public:
	CL_DataWriter(int size)
	{
		data = new unsigned char[size];
		max_size = size;
		pos = 0;
	}

	~CL_DataWriter()
	{
		delete[] data;
	}
	
	void write_byte(unsigned char b)
	{
		while (pos >= max_size) expand_data();
		data[pos++] = b;
	}
	
	void write_int32(int i)
	{
		while (pos+3 >= max_size) expand_data();

		unsigned int write_int = htonl(i);
		memcpy(&data[pos], &write_int, 4);
		pos += 4;
	}
	
	void write_string(CL_String s)
	{
		int slength = s.get_length();
		while (pos+3+slength >= max_size) expand_data();

		write_int32(slength);
		write_data((unsigned char *) s.get_string(), slength);
	}
	
	void write_data(void *write_data, int size_data)
	{
		while (pos+size_data-1 >= max_size) expand_data();

		memcpy(data+pos, write_data, size_data);
		pos += size_data;
	}
	
	unsigned char *get_data() { return data; }
	unsigned int get_size() { return pos; }
};

class CL_DataReader
{
private:
	unsigned char *data;
	int pos;
	int max_size;
	bool delete_data;
	
public:
	CL_DataReader(unsigned char *_data, int size, bool delete_data = false)
	{
		this->delete_data = delete_data;
		data = _data;
		max_size = size;
		pos = 0;
	}

	~CL_DataReader()
	{
		if (delete_data) delete[] data;
	}
	
	unsigned char read_byte()
	{
		return data[pos++];
	}
	
	int read_int32()
	{
		unsigned int ret = ntohl(*((unsigned int *) &(data[pos])));
		pos += 4;
		return ret;
	}
	
	CL_String read_string()
	{
		int size = read_int32();
		char *strdata = new char[size+1];
		read_data((unsigned char *) strdata, size);
		strdata[size] = 0;

		CL_String ret(strdata);
		delete[] strdata;
		
		return ret;
	}
	
	void read_data(void *read_data, int size_data)
	{
		memcpy(read_data, data+pos, size_data);
		pos += size_data;
	}

	unsigned char *get_data_ptr() { return &data[pos]; }

	bool data_left() { return pos < max_size; }
};

#endif
