/*
	$Id: memory.h,v 1.1.1.1 1998/12/20 23:33:18 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Memory debug support.


	$Log: memory.h,v $
	Revision 1.1.1.1  1998/12/20 23:33:18  mbn
	ClanLib 0.1.10 development version importet.
	
	Revision 1.3  1998/09/17 12:05:22  mbn
	Converted all spaces to tabs by using unexpand.

	Revision 1.2	1998/02/17 22:08:50	 mbn
	Changed class prefix to 'CL'.

	Revision 1.1.1.1	1998/02/07 16:51:17	 mbn
	Finally uploaded ClanLib-0.1.3 to CVS.

*/


#ifndef header_memmanager
#define header_memmanager

#ifdef ENABLE_MEMORYDEBUG

#include <iostream.h>

class CL_MemoryEntry
{
public:
	char *file;
	int line;
	int size;
	void *p;

	CL_MemoryEntry(char *_file, int _line, void *_p, int _size)
	{
		file = _file;
		line = _line;
		size = _size;
		p = _p;
	}

	void display()
	{
		cout << size << " bytes allocated in file " << ((file==NULL) ? "!!unknown!!" : file) << ", line " << line << endl;
	}

	bool match(void *match)
	{
		return (p == match);
	}
};


class CL_MemManager
{
public:
	static CL_List<CL_MemoryEntry> entries;
	static char *temp_file;
	static int temp_line;
	static bool enabled;

	static int set_info(char *file, int line)
	{
		temp_file = file;
		temp_line = line;

		return 0;
	}

	static void mark_new(void *p, int alloc_size)
	{
		if (enabled) CL_MemManager::entries.add(new CL_MemoryEntry(temp_file, temp_line, p, alloc_size));
	}

	static void mark_delete(void *p)
	{
		if (!enabled) return;

		CL_Iterator<CL_MemoryEntry> counter(CL_MemManager::entries);
		while (counter.next()!=NULL)
		{
			if (counter()->match(p))
			{
				CL_MemoryEntry *del = counter();
				counter.remove();
				delete del;
			}
		}
	}

	static void enable() { CL_MemManager::enabled = true; }
	static void disable() { CL_MemManager::enabled = false; }

	static void dump_info()
	{
		int total = 0;

		cout << endl << endl << endl;
		cout << "LIST OF MEMORY LEAKS: " << endl;
		cout << "---------------------------------------------------------" << endl;
		CL_Iterator<CL_MemoryEntry> counter(CL_MemManager::entries);
		while (counter.next()!=NULL)
		{
			counter()->display();
			total += counter()->size;
		}
		cout << "---------------------------------------------------------" << endl;
		cout << "TOTAL: " << total << endl;
		cout << endl << endl << endl;
	}

	static CL_MemoryEntry *is_allocated(void *p)
	{
		CL_Iterator<CL_MemoryEntry> counter(CL_MemManager::entries);
		while (counter.next()!=NULL)
		{
			if (counter()->match(p))
			{
				return counter();
			}
		}
		return NULL;
	}
};

#endif

#endif

