/*
	$Id: lockable_surface.h,v 1.2 1999/06/24 21:04:39 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifndef header_lockable_surface
#define header_lockable_surface

#include "target.h"

class CL_LockableSurface : public CL_Target
{
public:
	static CL_LockableSurface *create(
		int width,
		int height,
		int no_sprs = 1,
		int minimum_alpha_mask_size = 0,
		CL_DisplayCard *card = NULL);

	virtual ~CL_LockableSurface() { return; }

	virtual unsigned int get_no_sprs() const=0;
	//: Returns the number of subsprites in the surface.
	//!retval: Number of subsprites in the surface.
	
	virtual void set_colorkey(int colorkey)=0;
	virtual int get_colorkey() const=0;

	virtual void put_screen(int x, int y, int spr_no=0)=0;
//: Draws the surface onto the backbuffer.
//!param: x - x coordinate to draw surface.
//!param: y - y coordinate to draw surface.
//!param: spr_no - subsprite number to use.

	virtual void put_screen(int x, int y, float scale_x, float scale_y, int spr_no=0)=0;
//: Draws the surface onto the backbuffer - scaling it to a specified size.
//!param: x - x coordinate to draw surface.
//!param: y - y coordinate to draw surface.
//!param: scale_x - scale x multiplyer.
//!param: scale_y - scale y multiplyer.
//!param: spr_no - subsprite number to use.

	virtual void put_screen(int x, int y, int size_x, int size_y, int spr_no=0)=0;
//: Draws the surface onto the backbuffer - scaling it to a specified size.
//!param: x - x coordinate to draw surface.
//!param: y - y coordinate to draw surface.
//!param: size_x - width of the image when scaled.
//!param: size_y - height of the image when scaled.
//!param: spr_no - subsprite number to use.
};

#endif
