/*
	$Id: target.h,v 1.1 1999/06/22 02:19:15 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

#ifndef header_target
#define header_target

#include "palette.h"

class CL_Target
{
public:
	virtual ~CL_Target() { return; }

	virtual bool is_video() { return false; }
	//: Returns true if in video memory.
	//!retval: True if stored in video memory, false otherwise.

	virtual bool lock()=0;
	//: Locks the surface. This allows access to the surface data using the
	//: get_data() function call.

	virtual void unlock()=0;
	//: Unlock the surface. Must be called after you're finished with modifying
	//: the surface data.

	virtual void *get_data()=0;
	//: Returns a pointer to the surface data. Only valid between lock/unlock
	//: calls.
	//!retval: A pointer that points to the beginning of the surface pixel data.

	virtual unsigned int get_width()=0;
	//: Returns the width of the surface.
	//!retval: The width of the surface.

	virtual unsigned int get_height()=0;
	//: Returns the height of the surface.
	//!retval: The height of the surface.

	virtual unsigned int get_pitch()=0;
	//: Returns the pitch of the surface.
	//!retval: The pitch of the surface.

	virtual unsigned int get_depth()=0;
	//: Returns the depth of the lockable surface.
	//!retval: Depth of the lockable surface.

	virtual unsigned int get_red_mask()=0;
	//: Returns the red color mask in the lockable surface.
	//!retval: Red color mask in the lockable surface.

	virtual unsigned int get_green_mask()=0;
	//: Returns the green color mask in the lockable surface.
	//!retval: Green color mask in the lockable surface.

	virtual unsigned int get_blue_mask()=0;
	//: Returns the blue color mask in the lockable surface.
	//!retval: Blue color mask in the lockable surface.

	virtual unsigned int get_alpha_mask()=0;
	//: Returns the alpha mask in the lockable surface.
	//!retval: Alpha mask in the lockable surface.

	virtual CL_Palette *get_palette()=0;
	//: Returns the palette used by the lockable surface.
	//!retval: Pointer to surface used by lockable surface.
};

#endif
