/*
	$Id: inputbuffer.h,v 1.8 1999/08/24 21:24:25 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

//! l1_component="Input"

#ifndef header_inputbuffer
#define header_inputbuffer

class CL_Key
{
public:
	int id;
	// Key ID. CL_Keyboard::KEY_xxx if it is the keyboard buffer.

	int ascii;
	// Ascii value or -1 if not available.

	enum
	{
		NoKey,
		Pressed,
		Released
	} state;
	// Key state. Was it pressed or released. NoKey is used when there were
	// no key available in the input buffer.
	
	float x;
	// x-coordinate position if a pointing device was the source. -1 if none.
	
	float y;
	// y-coordinate position if a pointing device was the source. -1 if none.
};

class CL_InputBuffer
//: Input buffer interface.
// This is the interface to an input buffer, such as a keyboard buffer.
{
public:
	virtual ~CL_InputBuffer() { return; }
	virtual CL_Key peek_key() const =0;
	// Returns the next key in the buffer, but unlike get_key() it does not
	// pop it from the input buffer.

	virtual CL_Key get_key()=0;
	// Pops the next key in the buffer and returns it.

	virtual int keys_left()=0;
	// Returns number of keys still available in the buffer.
	//!retval: Number of key values lefts in buffer.

	virtual void clear()=0;
	// Clears the buffer. After this, the buffer will not contain any keys.
};

#endif
