/*
	$Id: system.h,v 1.5 1999/06/29 12:16:48 fischlustig Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

//! l1_component="System"

#ifndef header_system
#define header_system

#define VIDEO_BACKBUFFER (1)
#define SYSTEM_BACKBUFFER (1<<1)
#define SINGLE_BUFFER (1<<2)
#define DOUBLE_BUFFER (1<<3)
#define TRIPLE_BUFFER (1<<4)

class CL_System
//: The System component class. 
// Handles all misc communcation between the application and the system.
{
public:
	virtual ~CL_System() { ; }

	static int get_time();
//: Returns an abstract number indicating the time (in milliseconds).
//: <br>
//: The actual value returned should not be used, except when comparing 
//: the delta-value with other calls to this function. 
//!retval: Returns the abstract time in milliseconds.

	static void sleep(int millis);
//: Puts the program to sleep for the specified amount of milliseconds. 
//!param: The number of milliseconds to sleep.

	static bool keep_alive();
//: This function must be called with short intervals. Used to update 
//: the system state, communicating with the underlying operating system, 
//: listening for keyboard/mouse events/input, etc.
//: <br>
//: A call to this function once every updated frame is sufficient and necessary.
//!retval: Returns whether the program should now exit. It is very important, that the game exits, if keep_alive() returns true.

	static void init_display(int options=VIDEO_BACKBUFFER|DOUBLE_BUFFER);
//: Initializes the ClanLib display component. This function must be called, before any 
//: calls to the display-system is attempted.
//!param: options - Hints to the implementation, so it may optimize the speed.

	static void init_sound();
//: Initialized the ClanLib sound component. This function must be called, before any 
//: calls to the sound-system is attempted.

	static void init_network();
//: Initialized the ClanLib network component. This function must be called, before any 
//: calls to the network-system is attempted.

	static void deinit_display();
//: Deinitializes the ClanLib display component. A call to this function will be ignored 
//: if the display system was not previously initialized by calling init_display(). 
//: Explicit calls to this function before program closure is not necessary, as they 
//: will be performed automatically by ClanLib upon exiting.

	static void deinit_sound();
//: Deinitializes the ClanLib sound component. A call to this function will be ignored 
//: if the sound system was not previously initialized by calling init_sound(). 
//: Explicit calls to this function before program closure is not necessary, as they 
//: will be performed automatically by ClanLib upon exiting.

	static void deinit_network();
//: Deinitializes the ClanLib network component. A call to this function will be ignored 
//: if the network system was not previously initialized by calling init_network(). 
//: Explicit calls to this function before program closure is not necessary, as they 
//: will be performed automatically by ClanLib upon exiting.

	static int suspend_time;
	static bool time_running;
	static int start_pause;
	static void pause_time();
	static void resume_time();
};

#endif
