/*
	$Id: cltime.h,v 1.2 1999/08/31 05:49:26 sphair Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		ClanLib Time - easy access to time structures.

*/

//! l2_component="Misc"

#ifndef header_time
#define header_time

struct TimeInfo
{
	int sec;
	//: The number of seconds after the minute, normally in the 
	//: range 0 to 59, but can be up to 61 to allow for leap seconds.
	int min;
	//: The number of minutes after the hour, in the range 0 to 59.
	int hour;
	//: The number of hours past midnight, in the range 0 to 23.
	int monthday;
	//: The day of the month, in the range 1 to 31.
	int month;
	//: The number of months since January, in the range 0 to 11.
	int year;
	//: The number of years since 1900.
	int weekday;
	//: The number of days since Sunday, in the range 0 to 6.
	int yearday;
	//: The number of days since January 1, in the range 0 to 365.
	int daylightsaving;
	//: A flag that indicates whether daylight saving time is in 
	//: effect at the time described. The value is positive if 
	//: daylight saving time is in effect, zero if is not, and 
	//: negative if the information is not available.
};

class CL_Time
//: Class to get realtime and date.
// Use this class to get a easy access to time and date,
// represented in different ways.
//!also: CL_System::get_time() - Get program lapse in milliseconds.
{
public:
	CL_Time() {};

	static char *get_time_string();
	//!retval: Time string.
	//: Returns todays date and time represented as a string of the form
	//: "Wed Jun 30 21:49:08 1999\n".
	//: The return value points to a statically allocated string which might be 
	//: overwritten by subsequent calls to any of the date and time functions.

	static TimeInfo *CL_Time::get_time_info();
	//!retval: TimeInfo structure.
	//: Returns a TimeInfo structure with time and date info filled in.
	//: The return value points to a statically allocated TimeInfo struct which
	//: will overwritten by subsequent calls this function.
};

#endif
