
//! l2_component="Resources"

#ifndef header_resourcetype_integer
#define header_resourcetype_integer

#include "../../clanlib.h"
#include "resource_manager.h"

class CL_Res_Integer : public CL_ResourceType
//: The Integer resource is used to read boolean values from a resource file.
// CL_Res_Boolean, CL_Res_Integer and CL_Res_String helps you with seperating
// the game code from the constants used in the game.
// <br>
// Using these resource types, you can easily change the text used in the game.
// Just by changing the resource file, you can make you game speak Danish, or
// any other language. You can change the constants like weapon damage, movement
// speed, and so on.
// <br>
// These resources can also be used to apply theme support into your application.
// By letting each theme become a datafile, you can give the player full control
// of the visual parts of the game - and without using much work on it yourself.
//!res_type: integer
//!res_desc: The integer resource is used to read a number from a resourcefile
//!res_option: base=<x> - specifies the base on which the integer is based
{
public:
	static int load(CL_String res_id, CL_ResourceManager &manager, int _default);
	static int load(CL_String res_id, CL_ResourceManager &manager );

	CL_Res_Integer();

private:
friend CL_ResourceManager;
	virtual CL_Resource *try_create_from_file(
		CL_String name,
		CL_String prefix,
		CL_String res_id,
		CL_ResourceOptions *options,
		CL_ResourceManager *parent);

	virtual CL_Resource *try_create_from_datafile(
		CL_String name,
		CL_String prefix,
		CL_String res_id,
		CL_ResourceOptions *options,
		CL_ResourceManager *parent);

};

//// start here ////

class CL_WritableIntegerResource : public CL_Resource
{
protected:
	int loaded_integer;
	bool loaded;
	CL_String integer_location;
	CL_ResourceOptions *options;
	CL_ResourceManager *parent;
public:
	CL_WritableIntegerResource();
	virtual ~CL_WritableIntegerResource() { ; }

	virtual void write(CL_OutputSource *output);
	virtual bool load();

	virtual int create_integer()=0;
};

class CL_IntegerFileResource : public CL_WritableIntegerResource
{
public:
	CL_IntegerFileResource(CL_String integer_id, CL_String prefix, CL_String palette_location, CL_ResourceOptions *options, CL_ResourceManager *parent);
	virtual int create_integer();
};

class CL_IntegerDatafileResource : public CL_WritableIntegerResource
{
public:
	CL_IntegerDatafileResource(CL_String surface_id, CL_String prefix, CL_String integer_location, CL_ResourceOptions *options, CL_ResourceManager *parent);
	virtual int create_integer();
};

#endif
