
//! l2_component="Resources"

#ifndef header_resourcetype_palette
#define header_resourcetype_palette

#include "../../clanlib.h"
#include "resource_manager.h"

class CL_Res_Palette : public CL_ResourceType
//: The palette resource type.
// This class is used to load palette resources from a resource source
// (datafile or script file).
// <br>
// Use the CL_ResourceManager to get an instance to the resource source, and
// then call one of the static functions here to load the resource:
// <code>
// CL_ResourceManager res_manager("my_datafile.dat", true);
// CL_Palette *my_palette = CL_Res_Palette::load("my_palette", res_manager);
// </code>
//!res_type: palette
//!res_option: tga - Interpret this file as a targa image. (does this work?)
//!res_option: pcx - Interpret this file as a pcx image.
//!res_desc: The palette resource is used by the CL_Palette class to load a palette
//!res_desc: from a datafile.
{
public:
	static CL_Palette *load(CL_String res_id, CL_ResourceManager &manager);
	static void set_system_palette(CL_String res_id, CL_ResourceManager &manager);

	CL_Res_Palette();

private:
friend CL_ResourceManager;
	virtual CL_Resource *try_create_from_file(
		CL_String name,
		CL_String prefix,
		CL_String res_id,
		CL_ResourceOptions *options,
		CL_ResourceManager *parent);

	virtual CL_Resource *try_create_from_datafile(
		CL_String name,
		CL_String prefix,
		CL_String res_id,
		CL_ResourceOptions *options,
		CL_ResourceManager *parent);

};

class CL_WritablePaletteResource : public CL_Resource
{
protected:
	CL_Palette *loaded_palette;
	bool loaded;
	CL_String palette_location;
	CL_ResourceOptions *options;
	CL_ResourceManager *parent;
public:
	CL_WritablePaletteResource();
	virtual ~CL_WritablePaletteResource() { ; }

	virtual void write(CL_OutputSource *output);
	virtual bool load();

	virtual CL_Palette *create_palette()=0;
};

class CL_PaletteFileResource : public CL_WritablePaletteResource
{
public:
	CL_PaletteFileResource(CL_String palette_id, CL_String prefix, CL_String palette_location, CL_ResourceOptions *options, CL_ResourceManager *parent);
	virtual CL_Palette *create_palette();
};

class CL_PaletteDatafileResource : public CL_WritablePaletteResource
{
public:
	CL_PaletteDatafileResource(CL_String palette_id, CL_String prefix, CL_String palette_location, CL_ResourceOptions *options, CL_ResourceManager *parent);
	virtual CL_Palette *create_palette();
};

#endif
