/*
	$Id: resourcetype_sample.h,v 1.6 1999/08/22 17:18:47 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Sample resourcetype header file

*/

//! l2_component="Resources"

#ifndef header_resourcetype_sample
#define header_resourcetype_sample

#include "../../clanlib.h"
#include "resource_manager.h"

class CL_Res_Sample : public CL_ResourceType
//: The sound sample resource type.
// This class is used to load sample resources from a resource source
// (datafile or script file).
// <br>
// Use the CL_ResourceManager to get an instance to the resource source, and
// then call one of the static functions here to load the resource:
// <code>
// CL_ResourceManager res_manager("my_datafile.dat", true);
// CL_SoundBuffer *my_sample = CL_Res_Sample::load("my_sample", res_manager);
// </code>
//!res_type: sample
//!res_option: wav - Interpret this file as a PCM Wave file.
//!res_option: stream - This sample should be streamed.
//!res_option: loop - This sample should be looped (only used for streamed samples).
//!res_desc: A sound sample resource in ClanLib. Used by the static and streamed
//!res_desc: sound providers.
{
public:
	static CL_SoundBuffer *load(CL_String res_id, CL_ResourceManager &manager);

	CL_Res_Sample();

private:
friend CL_ResourceManager;
	virtual CL_Resource *try_create_from_file(CL_String name, CL_String prefix, CL_String res_id, CL_ResourceOptions *options, CL_ResourceManager *parent);
	virtual CL_Resource *try_create_from_datafile(CL_String name, CL_String prefix, CL_String res_id, CL_ResourceOptions *options, CL_ResourceManager *parent);
};

class CL_WritableSampleResource : public CL_Resource
{
protected:
	CL_SoundBuffer *loaded_sample;
	bool loaded;
	CL_String sample_location;
	CL_ResourceOptions *options;
	CL_ResourceManager *parent;
public:
	CL_WritableSampleResource();
	virtual ~CL_WritableSampleResource() { ; }

	virtual void write(CL_OutputSource *output);
	virtual bool load();

	virtual CL_StaticSoundProvider *create_static_provider()=0;
	virtual CL_StreamSoundProvider *create_stream_provider()=0;
	virtual CL_SoundBuffer *create_sample();
};

class CL_SampleFileResource : public CL_WritableSampleResource
{
public:
	CL_SampleFileResource(CL_String sample_id, CL_String prefix, CL_String sample_location, CL_ResourceOptions *options, CL_ResourceManager *parent);
	~CL_SampleFileResource();

	virtual CL_StaticSoundProvider *create_static_provider();
	virtual CL_StreamSoundProvider *create_stream_provider();
};

class CL_SampleDatafileResource : public CL_WritableSampleResource
{
public:
	CL_SampleDatafileResource(CL_String sample_id, CL_String prefix, CL_String sample_location, CL_ResourceOptions *options, CL_ResourceManager *parent);
	~CL_SampleDatafileResource();

	virtual CL_StaticSoundProvider *create_static_provider();
	virtual CL_StreamSoundProvider *create_stream_provider();
};


#endif
