
//! l2_component="Resources"

#ifndef header_resourcetype_string
#define header_resourcetype_string

#include "../../clanlib.h"
#include "resource_manager.h"

class CL_Res_String : public CL_ResourceType
//: The String resource is used to read boolean values from a resource file.
// CL_Res_Boolean, CL_Res_Integer and CL_Res_String helps you with seperating
// the game code from the constants used in the game.
// <br>
// Using these resource types, you can easily change the text used in the game.
// Just by changing the resource file, you can make you game speak Danish, or
// any other language. You can change the constants like weapon damage, movement
// speed, and so on.
// <br>
// These resources can also be used to apply theme support into your application.
// By letting each theme become a datafile, you can give the player full control
// of the visual parts of the game - and without using much work on it yourself.
//!res_type: string
//!res_desc: The string resource is used to read a string from a resourcefile
{
public:
	static CL_String load(CL_String res_id, CL_ResourceManager &manager, CL_String _default);
	static CL_String load(CL_String res_id, CL_ResourceManager &manager);

	CL_Res_String();

private:
friend CL_ResourceManager;
	virtual CL_Resource *try_create_from_file(
		CL_String name,
		CL_String prefix,
		CL_String res_id,
		CL_ResourceOptions *options,
		CL_ResourceManager *parent);

	virtual CL_Resource *try_create_from_datafile(
		CL_String name,
		CL_String prefix,
		CL_String res_id,
		CL_ResourceOptions *options,
		CL_ResourceManager *parent);

};

class CL_WritableStringResource : public CL_Resource
{
protected:
	CL_String loaded_string;
	bool loaded;
	CL_String string_location;
	CL_ResourceOptions *options;
	CL_ResourceManager *parent;
public:
	CL_WritableStringResource();
	virtual ~CL_WritableStringResource() { ; }

	virtual void write(CL_OutputSource *output);
	virtual bool load();

	virtual CL_String create_string()=0;
};

class CL_StringFileResource : public CL_WritableStringResource
{
public:
	CL_StringFileResource(CL_String string_id, CL_String prefix, CL_String palette_location, CL_ResourceOptions *options, CL_ResourceManager *parent);
	virtual CL_String create_string();
};

class CL_StringDatafileResource : public CL_WritableStringResource
{
public:
	CL_StringDatafileResource(CL_String surface_id, CL_String prefix, CL_String string_location, CL_ResourceOptions *options, CL_ResourceManager *parent);
	virtual CL_String create_string();
};

#endif
