/*
	$Id: provider_empty.h,v 1.10 1999/08/29 15:22:11 grumbel Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Empty space surface provider header file

*/

//! l2_component="SurfaceProviders"

#include "../../clanlib.h"

class CL_EmptySurfaceProvider : public CL_SurfaceProvider_Generic
//: Empty/canvas surface provider.
// Its usefull for the creation of static surface at runtime (for 
// example gradients, etc.). It is not intendend to be used for
// dynamic surfaces (ie which will be regenerated every frame).
//
// To create a surface, first create a new EmptySurfaceProvider, than
// lock() the provider, write your data to the pointer returned by
// get_surface_data(), unlock() the provider and call
// CL_Surface::create(provider) to generate a the surface.
//
//!also: CL_Surface - putting/blitting images to the screen
//!also: CL_LockableSurfaces - Creating 100% dynamic surfaces
{
	int width, height, no_sprs, trans_col;
	EPixelFormat pixelformat;
	unsigned char *data;
	int bpp;
public:
	static CL_Surface *create(int width, int height, int no_sprs, int trans_col = -1, EPixelFormat pixelformat=PAL8)
	{
		return CL_Surface::create(new CL_EmptySurfaceProvider(width, height, no_sprs, trans_col, pixelformat), true);
	}

	CL_EmptySurfaceProvider(int _width, int _height, int _no_sprs, int _trans_col, EPixelFormat type=PAL8)
	{
		width = _width;
		height = _height;
		no_sprs = _no_sprs;
		trans_col = _trans_col;
		data = NULL;
		pixelformat = type;
		bpp = get_bytes_pr_pixel();

		data = new unsigned char[width * height * no_sprs * bpp]; 
		memset(data, 0, width * height * no_sprs * bpp);
	}  
	//!param: _width - The width of the surface in pixels
	//!param: _height - The height of the surface in pixels
	//!param: _no_sprs - The number of subsprites the image contains, should be at least 1 or greater
	//!param: _trans_col - The color index of the color which should be masked transparent, but PAL8 isn't supported at the moment so say -1 here.
	//!param: pixelformat - Storage format for the color values of the pixel (RGBA8888, ...)
	//!retval: none
	//: Creates an empty surface provider, allocates the needed 
	//: memory and clears all pixels to 0 (transparent)


	virtual ~CL_EmptySurfaceProvider() { delete[] data;}
	//: Deletes the EmptySurfaceProvider and deletes the data
	//: which was allocated in the constructor

	virtual int get_width() const { return width; }
	virtual int get_height() const { return height; }
	virtual int get_no_sprs() const { return no_sprs; }
	virtual EPixelFormat get_pixel_format() const { return pixelformat; }
	//: Returns the pixelformat of the EmptySurfaceProvider as it was set in the constructor

	virtual CL_Palette *get_palette() const { return NULL; }
	//: Returns NULL, because PAL8 isn't supported

	virtual int get_transcol() const { return trans_col; }
	//: Returns the transparent color, which was set in the constructor

	virtual void *get_surface_data() const { cl_assert(data!=NULL); return data; }
	//: Returns the pointer to the surface data, where you
	//: can copy your data to. It should be
	//: only called between lock() and unlock(). 

	virtual void perform_lock() { }
	virtual void perform_unlock() { }
};
